// ***************************************************
// SmVR_CFakeTracker.cpp
//		This module implements the class that handle
//	the Fake Tracker device.
//
//
//  Author: Andr Benvenuti Trombetta
//	Email: trombetta@gmail.com
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://grv.inf.pucrs.br/trombetta
//		http://www.inf,pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2005.
// ***************************************************

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include "SharedMemoryLib.h"


pthread_t thread;
char fileName[20];
int porta;

static double POS[2][3], LastPos[2][3];

// ***************************************************
// ***************************************************
// ***************************************************


void *thread1(void *p);
// **************************************************
//  int IsotrackINIT(char *port )
//		'port' can be "COM1", "COM2", "COM3",....
//
// **************************************************
int FakeTrackerINIT(char *port )
{
	pthread_create(&thread,
			    NULL,
			    thread1,
			   NULL);

	LastPos[0] = LastPos[1] = LastPos[2] = 0.0;
	return 1;
}

void *thread1(void *p){
    char aux[20];
    char aux2[3];
    strcpy(aux, "Tracker.exe COM");
    itoa(porta, aux2, 10);
    aux2[3] = '\0';
    strcat(aux, aux2);
    system(aux);
}


// ***************************************************
// ***************************************************
// ***************************************************

// Implementation of SmVR_CFakeTracker class

#include "SmVR_CFakeTracker.h"

SmVR_CFakeTracker::SmVR_CFakeTracker()
{
	NumberOfSensors = 2;

	for (int i=0; i< NumberOfSensors;i++)
		BindObject[i]=NULL;
}



// ***************************************************
// int SmVR_CIsotrack::Init(void *p)
//
//
// ***************************************************
int SmVR_CFakeTracker::Init(void *p)
{
	char aux[10];

	printf("'Init' method from NEW class....\n");
	strncpy(aux, (char *)p, 3);
	aux[3] = '\0';
	if (strcmp(aux,"COM"))
	{
		printf("The parameter passed to 'Init' method must be \"COM1\", \"COM2\", ...\n");
	}
	// for Isotrack 'p' must be "COM1", "COM2", ...
	strcpy(aux, (char *)p);
	strncpy(aux, &aux[3], 2);
	porta = atoi(aux);
	CreateSharedMemoryArea((char *)p);
    FakeTrackerINIT((char *)p);

    return 0;
}

// ***************************************************
//	int SmVR_CIsotrack::Read(void *data)
//
//
// ***************************************************
int SmVR_CFakeTracker::Read(void *data)
{
	//rotx, roty, rotz, transx, transy, transz
	for(int i = 0; i < 3; i++){
            ReadFromSharedMemory(&(ROT[0][i]), i);
            ReadFromSharedMemory(&(POS[0][i]), i+3);
            ReadFromSharedMemory(&(ROT[1][i]), i+6);
            ReadFromSharedMemory(&(POS[1][i]), i+9);
    }

	//printf("Read from Isotrack: %s\n", aux);

	return 0;
}

// ***************************************************
// void SmVR_CFakeTracker::Bind(int nSensor, SmVR_CGeometricObject *o )
//
//
// ***************************************************
void SmVR_CFakeTracker::Bind(int nSensor, SmVR_CGeometricObject *o )
{

	BindObject[nSensor] = o;
	//printf("'Bind' method from NEW class....\n");
}

// ***************************************************
//
// ***************************************************
void SmVR_CFakeTracker::UpdateAllSensors (int tx, int ty, int tz, int rx, int ry, int rz)
{
	int i;
	SmVR_CPoint Zero(0,0,0), Pos;

	// read the tracker
	Read(NULL);

	for (i=0; i< NumberOfSensors;i++)
	// Update the object linked to the tracker
		if (BindObject)
		{
			// Reset the rotation part of the object matrix
			//BindObject[i]->SetRotationMatrixToIdent();
			BindObject->SetLocalTransformationMatrixToIdentity();

			// Set the the object with data that has come from the tracker
			// reset the tracker position
			BindObject->TranslateBy((tx ? (float)( LastPos[i][0]): 0),
                                    (ty ? (float)( LastPos[i][1]): 0),
                                    (tz ? (float)( LastPos[i][2]): 0));
			// Move the tracker object
			BindObject->TranslateBy((tx ? (float)( POS[i][0]): 0),
                                    (ty ? (float)( POS[i][1]): 0),
                                    (tz ? (float)( POS[i][2]): 0));

			if(ry) BindObject->RotateBy(((int)(180-ROT[i][0])/1),0,1,0);
			if(rx) BindObject->RotateBy(((int)ROT[i][1]/1),1,0,0);
			if(rz) BindObject->RotateBy(((int)(-ROT[i][2])/1),0,0,1);
        }



	//printf("x = %6.2f\ty = %6.2f\tz = %6.2f\n",x,y,z);
	//printf("ROT x = %6.2f\n", ROT[0][1]);
}

// ***************************************************
//
// ***************************************************
void SmVR_CFakeTracker::UpdateAllSensors (void)
{
    UpdateAllSensors (1,1,1,1,1,1);
}



// ***************************************************
// int SmVR_CIsotrack::Reset(void *p)
//
//
// ***************************************************
int SmVR_CFakeTracker::Reset(void *p)
{
	WriteOnSharedMemory(1, 12);
	return 0;
}



// ***************************************************
//  void SmVR_CIsotrack::SetExtraData(void *data)
//
//
// ***************************************************
void SmVR_CFakeTracker::SetExtraData(void *data)
{
	printf("'SetExtraData' method from NEW class....\n");
}

// ***************************************************
//
// ***************************************************
SmVR_CFakeTracker::~SmVR_CFakeTracker(void)
{
	DestroySharedMemoryArea();
}


