// ************************************************
// ComandosDaSmallVR.cpp
//    
//	Sample file for SmallVR
//  This file was created to be used as a sample
//  to link SmallVR and FLTK 1.1.6
//  It uses the "SmVRWindow.fl" file to describe  
//  the interface.
//
//
//   If you need to create a console application,
//   add the "-lgdi32" directive to the linker
//   parameters 
//
//  Important functions:
//     InitObjetos: 
//          Put here all the inititalization code for 
//          SmallVR
//
//     void Ogl::draw():  
//          Method responsible for redraw the opengl
//          window 
//            
//      int Ogl::handle(int event){
//          Method responsible for handle the 
//          key strokes in opengl window             
//
//
//  Author: Marcio Serolli Pinho
//	Email: pinho@inf.pucrs.br		 
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2005.
// ************************************************
     
#include <SmVr.h>
#include <Fl/Fl.h> 

#include "SmVRWindow.h"
//#include "ComandosDaSmallVR.h"


// Functions to Draw Objects

// **********************************************************************
// int DrawDonut(void *p)
//
// **********************************************************************
int DrawDonut(void *p) 
{
    glColor3f(1.0,0.0,0.0);
	glutSolidTorus(0.3, 0.6, 10, 50);
    glColor3f(0.0,0.0,0.0);
	glutWireTorus(0.3, 0.6, 10, 50);    
    return 1;
} 
// **********************************************************************
// int TrianguloBranco(void *p)
//
// **********************************************************************
int DrawQuadB(void *p) 
{	
	glBegin (GL_QUADS);
	glColor3f(1,1,1);
	glVertex3f(0.3,0.3,0);
	glVertex3f(-0.3,0.3,0);
	glVertex3f(-0.3,-0.3,0);
	glVertex3f(0.3,-0.3,0);
	glEnd();
	return 1;
} 
// **********************************************************************
// int TrianguloAzul (void *p)
//
// **********************************************************************
int TriAzul(void *p) 
{
	//printf("TriAzul:\n");	
	glBegin (GL_TRIANGLES);
	glColor3f(0,1,1);
	glVertex3f(-0.3,-0.3,0);
	glVertex3f(0.3,-0.3,0);
	glVertex3f(0,0.3,0);
    glEnd();
	return 1;
}

SmVR_CGeometricObject *Root, *TA, *QB, *Donut;

// **********************************************************************
// void InitObjetos()
//
// **********************************************************************
void InitObjetos()
{ 
	Root = SmVR_Init(NULL);
	
	TA = new SmVR_CGeometricObject ("Tringulo Azul", TriAzul);
	QB = new SmVR_CGeometricObject ("Quadrado Branco", DrawQuadB);
	Donut = new SmVR_CGeometricObject ("Donut", DrawDonut);	
	
	printf("InitObjetos\n");		
	Root->AddChild(TA);
	TA->TranslateBy(1.5,0,0);
	printf("Depois do Add\n");	
	TA->AddChild(QB);
	QB->AddChild(Donut);
	Donut->TranslateBy(0,1.5,0);
	
}
// **********************************************************************
// Ogl::Ogl 
//      class constructor
//      It must run the Fl_Gl_Window class contructor
//      NEVER PUT OPENGL COMMANDS INSIDE IT !
// **********************************************************************
Ogl::Ogl(int X, int Y, int W, int H) : Fl_Gl_Window(X, Y, W, H)
{
      
}
        

// **********************************************************************
// void Ogl::draw()
//    Method responsible for redraw the opengl window every time it is 
//    necessary
// **********************************************************************
void Ogl::draw()
{
  
    static int first = true;
    if (first) // esta parte roda apenas 1 vez !
    {
        InitObjetos(); 
        first = false;
    }    
	if (!valid())
    {						
		glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
		glClear (GL_COLOR_BUFFER_BIT);     
		
		// Set the clipping volume
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(60,1,0.01,200);
		
		// Set the user Position
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	    gluLookAt(0,0,5, 0,0,0, 0.0f,1.0f,0.0f);
		
		Root->Render();
				
		swap_buffers();
    }    	
}
// **********************************************************************
// void Ogl::handle(int event)
//     Method responsible for handle the key strokes in opengl window   
//     NEVER PUT OPENGL COMMANDS INSIDE IT !
//
// **********************************************************************
int Ogl::handle(int event)
{
	switch (event){
    case FL_SHORTCUT: {
        if ( (Fl::event_key() == 'a') || (Fl::event_key() == 'A') ) {
		}
        if ( (Fl::event_key() == 's') || (Fl::event_key() == 'S') ) {
		}
        if ( (Fl::event_key() == 'd') || (Fl::event_key() == 'D') ) {
		}
        if ( (Fl::event_key() == 'f') || (Fl::event_key() == 'F') ) {
		}
        if ( (Fl::event_key() == 'g') || (Fl::event_key() == 'g') ) {
		}
        if ( (Fl::event_key() == 'h') || (Fl::event_key() == 'H') ) {
		}
        break;
					  }            
    case FL_PUSH: {
        break;
				  }
    }
	if (!valid()) 
      show();
	return 0;
}
SmVRWindow *MainWindow;
// **********************************************************************
//  int main(int argc, char* argv[])
//
//
// **********************************************************************
int main(int argc, char* argv[])
{
  // Create the main window 
  MainWindow = new SmVRWindow(argc, argv);
  // Start FLTK
  return Fl::run();
} 
