//
// Copyright 2005 by Mauro Charo, Rafael Rieder.
// Porto Alegre, Brazil.
//
// This library is free software; you can redistribute it and/or modify it under the
// terms of the GNU Library General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the GNU Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public License along with
// this library; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA 02111-1307 USA.
//
// Please report bugs & problems to maurocharao@yahoo.com.br or rafaelrieder@gmail.com
//

// *************************************************************************************
//  Acao.cxx
//  Control undo and redo actions in the Scene Editor
// *************************************************************************************
#include "Acao.h"
#include "SceneEditor.h"

vector<Acao*> Acao::vetorDesfazer;
vector<Acao*> Acao::vetorRefazer;

// *************************************************************************************
// Constructor...: Initialize variables for the control of any action to be executed
// *************************************************************************************
Acao::Acao(SmVR_CGeometricObject *obj, tipo_acao tipo, float valor, char *str_valor, SmVR_Matrix mRot) {
  this->tipo = tipo;
  this->valor = valor;
  if(str_valor) {
    this->str_valor = new char[strlen(str_valor) + 1];
    sprintf(this->str_valor, str_valor);
  }
  else this->str_valor = NULL;
  this->nome = new char[strlen(obj->GetName()) + 1];
  sprintf(this->nome, obj->GetName());
  if(tipo == RESETAR || tipo == REMOVER || tipo == INSERIR || tipo == ROT) {
    obj->GetLocalTransformationMatrix(m);
    loader = obj->GetObjectFromFile();
  } else {
    delete m;
    loader = NULL;
  }
  if(tipo == ROT) SmVR_CopyMatrix(this->mRot, mRot);
  //else delete this->mRot;
}

// *************************************************************************************
// Destructor...: Delete name and label of selected object
// *************************************************************************************
Acao::~Acao() {
  delete nome;
  if(m) delete m;
  //if(mRot) delete mRot;
  if(str_valor) delete str_valor;
}

// *************************************************************************************
// Method.....: limpa()
// Function...: Call two functions to clean vectors
// *************************************************************************************
void Acao::limpa() {
  limpaDesfazer();
  limpaRefazer();
}

// *************************************************************************************
// Method.....: limpaDesfazer()
// Function...: Clean the vector that contain undo tasks
// *************************************************************************************
void Acao::limpaDesfazer() {
  while(vetorDesfazer.size()) {
    delete vetorDesfazer[vetorDesfazer.size() - 1];
    vetorDesfazer.pop_back();
  }
}

// *************************************************************************************
// Method.....: limpaRefazer()
// Function...: Clean the vector that contain redo tasks
// *************************************************************************************
void Acao::limpaRefazer() {
  while(vetorRefazer.size()) {
    delete vetorRefazer[vetorRefazer.size() - 1];
    vetorRefazer.pop_back();
  }
}

// *************************************************************************************
// Method.....: adicionaAcao(SmVR_CGeometricObject *obj, tipo_acao tipo, float valor, char *str_valor, SmVR_Matrix mRot)
// Function...: Add new actions executed by user
// *************************************************************************************
void Acao::adicionaAcao(SmVR_CGeometricObject *obj, tipo_acao tipo, float valor, char *str_valor, SmVR_Matrix mRot) {
  if(tipo >= ROT_X && tipo <= TRANS_Z && valor == 0) return; // There is no effect
  limpaRefazer();
  vetorDesfazer.push_back(new Acao(obj, tipo, valor, str_valor, mRot));
  while(vetorDesfazer.size() > janelaPref->numMaxDesfazer)
    vetorDesfazer.erase(vetorDesfazer.begin());
}

// *************************************************************************************
// Method.....: existeLoader(SmVR_CObjFromFile *loader)
// Function...: Verify if exists undo or redo tasks
// *************************************************************************************
bool Acao::existeLoader(SmVR_CObjFromFile *loader) {
  int x;
  for(x = 0; x < vetorDesfazer.size(); x++)
    if(loader == vetorDesfazer[x]->loader)
      return true;
  for(x = 0; x < vetorRefazer.size(); x++)
    if(loader == vetorRefazer[x]->loader)
      return true;
  return false;
}

// *************************************************************************************
// Method.....: podeDesfazer()
// Function...: Verify underflow in vetorDesfazer
// *************************************************************************************
bool Acao::podeDesfazer() {
  return(vetorDesfazer.size() > 0);
}

// *************************************************************************************
// Method.....: desfazer()
// Function...: Recovery undo tasks
// *************************************************************************************
void Acao::desfazer() {
  if(!vetorDesfazer.size()) return;
  Acao *ac = vetorDesfazer[vetorDesfazer.size() - 1];
  janela->selecionaObjeto(ac->tipo == RENOMEAR ? ac->str_valor : ac->nome);
  SmVR_CGeometricObject *obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
  if(ac->tipo >= ROT && ac->tipo <= TRANS_Z) {
    if(ac->tipo == ROT) {
      obj_aux->SetLocalTransformationMatrix(ac->m);
    } else if(ac->tipo == ROT_X) {
      obj_aux->RotateBy(-ac->valor, 1, 0, 0);
    } else if(ac->tipo == ROT_Y) {
      obj_aux->RotateBy(-ac->valor, 0, 1, 0);
    } else if(ac->tipo == ROT_Z) {
      obj_aux->RotateBy(-ac->valor, 0, 0, 1);
    } else if(ac->tipo == ESC_X) {
      obj_aux->ScaleBy(janela->valorX->value() / (janela->valorX->value() + ac->valor), 1, 1);
    } else if(ac->tipo == ESC_Y) {
      obj_aux->ScaleBy(1, janela->valorY->value() / (janela->valorY->value() + ac->valor), 1);
    } else if(ac->tipo == ESC_Z) {
      obj_aux->ScaleBy(1, 1, janela->valorZ->value() / (janela->valorZ->value() + ac->valor));
    } else if(ac->tipo == TRANS_X) {
      obj_aux->TranslateBy(-ac->valor, 0, 0);
    } else if(ac->tipo == TRANS_Y) {
      obj_aux->TranslateBy(0, -ac->valor, 0);
    } else if(ac->tipo == TRANS_Z) {
      obj_aux->TranslateBy(0, 0, -ac->valor);
    }
    SceneEditor::callbackObjeto(NULL, NULL);
  } else if(ac->tipo == REMOVER) { // reinsert
    obj_aux = new SmVR_CGeometricObject(ac->nome, ac->loader);
    obj_aux->SetLocalTransformationMatrix(ac->m);
    janela->adicionaObjeto(obj_aux);
  } else if(ac->tipo == INSERIR) {
    janela->apagaObjetoAtual();
  } else if(ac->tipo == RESETAR) {
    obj_aux->SetLocalTransformationMatrix(ac->m);
    SceneEditor::callbackObjeto(NULL, NULL);
  } else if(ac->tipo == RENOMEAR) {
    obj_aux->SetName(ac->nome);
    janela->listObjetos->text(janela->listObjetos->value(), ac->nome);
  }
  vetorRefazer.push_back(ac);
  vetorDesfazer.pop_back();
}

// *************************************************************************************
// Method.....: refazer()
// Function...: Recovery redo tasks
// *************************************************************************************
void Acao::refazer() {
  if(!vetorRefazer.size()) return;
  Acao *ac = vetorRefazer[vetorRefazer.size() - 1];
  janela->selecionaObjeto(ac->nome);
  SmVR_CGeometricObject *obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
  if(ac->tipo >= ROT && ac->tipo <= TRANS_Z) {
    if(ac->tipo == ROT) {
      obj_aux->SetLocalTransformationMatrix(ac->mRot);
    } else if(ac->tipo == ROT_X) {
      obj_aux->RotateBy(ac->valor, 1, 0, 0);
    } else if(ac->tipo == ROT_Y) {
      obj_aux->RotateBy(ac->valor, 0, 1, 0);
    } else if(ac->tipo == ROT_Z) {
      obj_aux->RotateBy(ac->valor, 0, 0, 1);
    } else if(ac->tipo == ESC_X) {
      obj_aux->ScaleBy(janela->valorX->value() / (janela->valorX->value() - ac->valor), 1, 1);
    } else if(ac->tipo == ESC_Y) {
      obj_aux->ScaleBy(1, janela->valorY->value() / (janela->valorY->value() - ac->valor), 1);
    } else if(ac->tipo == ESC_Z) {
      obj_aux->ScaleBy(1, 1, janela->valorZ->value() / (janela->valorZ->value() - ac->valor));
    } else if(ac->tipo == TRANS_X) {
      obj_aux->TranslateBy(ac->valor, 0, 0);
    } else if(ac->tipo == TRANS_Y) {
      obj_aux->TranslateBy(0, ac->valor, 0);
    } else if(ac->tipo == TRANS_Z) {
      obj_aux->TranslateBy(0, 0, ac->valor);
    }
    SceneEditor::callbackObjeto(NULL, NULL);
  } else if(ac->tipo == REMOVER) {
    janela->apagaObjetoAtual();
  } else if(ac->tipo == INSERIR) {
    obj_aux = new SmVR_CGeometricObject(ac->nome, ac->loader);
    obj_aux->SetLocalTransformationMatrix(ac->m);
    janela->adicionaObjeto(obj_aux);
  } else if(ac->tipo == RESETAR) {
    obj_aux->SetLocalTransformationMatrixToIdentity();
    SceneEditor::callbackObjeto(NULL, NULL);
  } else if(ac->tipo == RENOMEAR) {
    obj_aux->SetName(ac->str_valor);
    janela->listObjetos->text(janela->listObjetos->value(), ac->str_valor);
  }
  vetorDesfazer.push_back(ac);
  vetorRefazer.pop_back();
}
