//
// Copyright 2005 by Mauro Charo, Rafael Rieder.
// Porto Alegre, Brazil.
//
// This library is free software; you can redistribute it and/or modify it under the
// terms of the GNU Library General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the GNU Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public License along with
// this library; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA 02111-1307 USA.
//
// Please report bugs & problems to maurocharao@yahoo.com.br or rafaelrieder@gmail.com
//

// *************************************************************************************
//  Gl_Group.cxx
//  Control trackball and viewers
// *************************************************************************************
#include "Gl_Group.h"
#include "SceneEditor.h"
#include "Acao.h"
#include "trackball.h"

float Gl_Group::curquat[4];  // Para a trackball
float Gl_Group::mquat[4][4]; // Para a trackball
int Gl_Group::spinning = 0;  // Para a trackball
SmVR_Matrix Gl_Group::m;     // Para a trackball
SmVR_Matrix Gl_Group::mr;    // Para a trackball
int Gl_Group::especMaterial = 60;
float Gl_Group::luzAmbiente   [4] = { 0.5, 0.5, 0.5, 1.0};
float Gl_Group::luzDifusa     [4] = { 0.7, 0.7, 0.7, 1.0};    // Cor do objeto
float Gl_Group::luzEspecular  [4] = { 0.5, 0.5, 0.5, 1.0};    // Brilho do objeto
float Gl_Group::especularidade[4] = { 0.17, 0.17, 0.17, 1.0}; // Capacidade de brilho do material

const char* Gl_Group::label_vista[8] = {
  "",
  "Perspectiva",
  "Frontal",
  "Traseira",
  "Direita",
  "Esquerda",
  "Superior",
  "Inferior"
};

// *************************************************************************************
// Constructor...: Initialize trackball
// *************************************************************************************
Gl_Group::Gl_Group(int X, int Y, int W, int H, tipo_vista tipo) : Fl_Gl_Window(X, Y, W, H) {
  resizable(this);
  box(FL_NO_BOX);
  color(FL_BACKGROUND_COLOR, FL_BACKGROUND_COLOR);
  end();
  anguloVisao = 90;
  val_near = .1;
  inicializou = 0;
  mouseX = mouseY = -1;
  this->tipo = tipo;
  invalidate(); // Para inicializar
  mode(FL_DOUBLE | FL_RGB | FL_DEPTH);
  trackball(curquat, 0.0, 0.0, 0.0, 0.0); // Inicializa a trackball
  Fl::add_timeout(0.35, callbackTrackball);
}

// *************************************************************************************
// Method.....: draw()
// Function...: Draw scene - define user position in viewers and call the SmallVR Render
// *************************************************************************************
void Gl_Group::draw() {
  if(!valid()) {
    glViewport(0, 0, w(), h());
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    //if(tipo == NENHUM)
      gluPerspective(anguloVisao, w()/h(), val_near, 2.0 * janelaPref->diagonalCenario);
    //else gluOrtho2D(janelaPref->cenarioMinX, janelaPref->cenarioMaxX, janelaPref->cenarioMinZ, janelaPref->cenarioMaxZ);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    if(!inicializou) {
      inicializou = 1;
      glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
      glClearColor(0.0f, 0.0f, 0.0f, 0.0f); // Cor de fundo de tela
      glShadeModel(GL_SMOOTH); // Habilita o modelo de tonalizacao de Gouraud
      glMaterialfv(GL_FRONT, GL_SPECULAR, especularidade); // Define a refletancia do material
      glMateriali(GL_FRONT, GL_SHININESS, especMaterial);  // Define a concentracao do brilho
      // Define os parametros da luz de numero 0
      posicaoLuz1[0] = janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0;
      posicaoLuz1[1] = janelaPref->cenarioMaxY;
      posicaoLuz1[2] = janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0;
      posicaoLuz1[3] = 1.0;
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT, luzAmbiente); // Ativa o uso da luz ambiente
      glLightfv(GL_LIGHT0, GL_AMBIENT , luzAmbiente);
      glLightfv(GL_LIGHT0, GL_DIFFUSE , luzDifusa);
      glLightfv(GL_LIGHT0, GL_SPECULAR, luzEspecular);
      glLightfv(GL_LIGHT0, GL_POSITION, posicaoLuz1);
      glEnable(GL_COLOR_MATERIAL); // Habilita a definicao da cor do material a partir da cor corrente
      glEnable(GL_LIGHTING);       // Habilita o uso de iluminacao
      glEnable(GL_LIGHT0);         // Habilita a luz de numero 0
      glEnable(GL_DEPTH_TEST);     // Habilita o depth-buffering
      glEnable(GL_NORMALIZE);      // Habilita a normalizacao das normais
    }
    if(tipo == NENHUM) { // A janela principal
      gluLookAt(pt_user->X, pt_user->Y, pt_user->Z, // Posicao do observador
                pt_alvo->X, pt_alvo->Y, pt_alvo->Z, // Ponto que o observador esta olhando
                0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
    } else if(tipo == PERSPECTIVA) {
      gluLookAt(janelaPref->cenarioMinX, janelaPref->cenarioMaxY, janelaPref->cenarioMaxZ, // Posicao do observador
                janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Ponto que o observador esta olhando
                0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
    } else if(tipo == FRONTAL) {
      gluLookAt(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMaxZ,                                                             // Posicao do observador
                janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Ponto que o observador esta olhando
                0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
    } else if(tipo == TRASEIRA) {
      gluLookAt(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ,                                                             // Posicao do observador
                janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Ponto que o observador esta olhando
                0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
    } else if(tipo == DIREITA) {
      gluLookAt(janelaPref->cenarioMaxX,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Posicao do observador
                janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Ponto que o observador esta olhando
                0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
    } else if(tipo == ESQUERDA) {
      gluLookAt(janelaPref->cenarioMinX,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Posicao do observador
                janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Ponto que o observador esta olhando
                0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
    } else if(tipo == SUPERIOR) {
      gluLookAt(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMaxY,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Posicao do observador
                janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Ponto que o observador esta olhando
                0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
    } else if(tipo == INFERIOR) {
      gluLookAt(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Posicao do observador
                janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, // Ponto que o observador esta olhando
                0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
    }
  }
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  RootObject->Render();

  // Desenha o grid (Draw the grid)
  glColor3f(0, 1, 0);
  glBegin(GL_LINES);
  for(int x = 0; x < janelaPref->numLinhasGridX; x++) {
    for(int z = 0; z < janelaPref->numLinhasGridZ; z++) {
      glVertex3f(janelaPref->cenarioMinX + x * janelaPref->espacamentoGrid, janelaPref->cenarioMinY, janelaPref->cenarioMinZ);
      glVertex3f(janelaPref->cenarioMinX + x * janelaPref->espacamentoGrid, janelaPref->cenarioMinY, janelaPref->cenarioMaxZ);

      glVertex3f(janelaPref->cenarioMinX, janelaPref->cenarioMinY, janelaPref->cenarioMinZ + z * janelaPref->espacamentoGrid);
      glVertex3f(janelaPref->cenarioMaxX, janelaPref->cenarioMinY, janelaPref->cenarioMinZ + z * janelaPref->espacamentoGrid);
    }
  }
  glEnd();
  glColor3f(1, 1, 1); // Para nao alterar a cor dos outros objetos
  swap_buffers();
}

// *************************************************************************************
// Method.....: int handle(int event)
// Function...: Handle objects with trackball - update data of objects
// *************************************************************************************
int Gl_Group::handle(int event) {
  int diff_mouse, retorno = 0;

  if(mouseX == -1) { // Inicializa as variaveis
    mouseX = Fl::event_x();
    mouseY = Fl::event_y();
  }
  if(event == FL_SHOW) {
    inicializou = 0; // Precisa reinicializar e
    invalidate();    // repintar toda vez que reaparece
    retorno = 1;     // para nao perder as cores
  } else if(Fl::event_state() & FL_BUTTONS) {
    if(janela->btRotacao->value()) {
      if(Fl::event_x() - mouseX || Fl::event_y() - mouseY) {
        trackball(curquat,
            (2.0 * mouseX - w()) / w(),
            (h() - 2.0 * mouseY) / h(),
            (2.0 * Fl::event_x() - w()) / w(),
            (h() - 2.0 * Fl::event_y()) / h());
        spinning = 1;
      } else {
        spinning = 0;
      }
    } else {
      if(diff_mouse = Fl::event_x() - mouseX) {
        janela->valorX->value(janela->valorX->clamp(janela->valorX->value() + diff_mouse / (w() / (janelaPref->cenarioMaxX - janelaPref->cenarioMinX))));
        SceneEditor::callbackMudaCampos2(janela->valorX, NULL);
      }
      if(diff_mouse = Fl::event_y() - mouseY) {
        if(Fl::event_state() & FL_BUTTON1) {
          janela->valorY->value(janela->valorY->clamp(janela->valorY->value() - diff_mouse / (h() / (janelaPref->cenarioMaxY - janelaPref->cenarioMinY))));
          SceneEditor::callbackMudaCampos2(janela->valorY, NULL);
        } else {
          janela->valorZ->value(janela->valorZ->clamp(janela->valorZ->value() - diff_mouse / (h() / (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ))));
          SceneEditor::callbackMudaCampos2(janela->valorZ, NULL);
        }
      }
    }
    retorno = 1;
  }
  mouseX = Fl::event_x();
  mouseY = Fl::event_y();
  if(retorno) {
    Fl_Gl_Window::handle(event); // Chama o handle do pai
    return 1; // Para permitir receber os eventos desejados
  }
  return Fl_Gl_Window::handle(event); // Chama o handle do pai
}

// *************************************************************************************
// Method.....: callbackTrackball(void *val)
// Function...: Start trackball
// *************************************************************************************
void Gl_Group::callbackTrackball(void *val) {
  if(spinning) {
    SmVR_CGeometricObject *obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
    build_rotmatrix(mquat, curquat);
    obj_aux->GetLocalTransformationMatrix(m);
    SmVR_MultMatrix(mquat, m, mr);
    Acao::adicionaAcao(obj_aux, ROT, 0, NULL, mr);
    obj_aux->SetLocalTransformationMatrix(mr);
    SceneEditor::callbackObjeto(NULL, NULL);
  }
  Fl::repeat_timeout(0.35, callbackTrackball);
}

// *************************************************************************************
// Method.....: SetPosicaoLuz(float x, float y, float z, float alfa)
// Function...: Set light position
// *************************************************************************************
void Gl_Group::SetPosicaoLuz(float x, float y, float z, float alfa) {
  posicaoLuz1[0] = x;
  posicaoLuz1[1] = y;
  posicaoLuz1[2] = z;
  posicaoLuz1[3] = alfa;
  glMatrixMode(GL_MODELVIEW);
  glLightfv(GL_LIGHT0, GL_POSITION, posicaoLuz1);
}

// *************************************************************************************
// Method.....: SetTipo(tipo_vista tipo)
// Function...: Set window view
// *************************************************************************************
void Gl_Group::SetTipo(tipo_vista tipo) {
  if(tipo != this->tipo) {
    this->tipo = tipo;
    inicializou = 0; // Precisa reinicializar e
    invalidate();    // repintar para aplicar
  }
}
