//
// Copyright 2005 by Mauro Charo, Rafael Rieder.
//
// This library is free software; you can redistribute it and/or modify it under the
// terms of the GNU Library General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the GNU Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public License along with
// this library; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA 02111-1307 USA.
//
// Please report bugs & problems to maurocharao@yahoo.com.br or rafaelrieder@gmail.com
//

// *************************************************************************************
//  Preferencias.cxx
//  Preference options, when the user fitts the scene size, undo max number and
//  cell space grid
//  This window was modelling using FLUID 1.1.0 - FLTK 1.1.x
// *************************************************************************************
#include "Preferencias.h"
#include "SceneEditor.h"
#include <math.h>
#include <FL/fl_ask.H>

// *************************************************************************************
// Constructor...: Define Preference Options Window
// *************************************************************************************
Preferencias::Preferencias() : Fl_Window((Fl::w() - 215) / 2, (Fl::h() - 226) / 2, 215, 226) {
  cenarioMinX = cenarioMinY = cenarioMinZ = -50;
  cenarioMaxX = cenarioMaxY = cenarioMaxZ =  50;
  diagonalCenario = 86.6;
  numMaxDesfazer = 500;
  espacamentoGrid = 10;
  numLinhasGridX = 11;
  numLinhasGridZ = 11;
  box(FL_FLAT_BOX);
  labelPreferencias = new Fl_Box(5, 0, 205, 35, "Prefer\352ncias");
  labelPreferencias->labeltype(FL_ENGRAVED_LABEL);
  labelPreferencias->labelsize(36);

  grupoTamCenario = new Fl_Group(20, 40, 175, 100, "Tamanho do Cen\341rio");
  grupoTamCenario->box(FL_ENGRAVED_FRAME);
  grupoTamCenario->labeltype(FL_ENGRAVED_LABEL);
  grupoTamCenario->align(FL_ALIGN_TOP|FL_ALIGN_INSIDE);
    inputCenarioMinX = new Fl_Value_Input(48, 63, 60, 22, "X");
    inputCenarioMinX->labeltype(FL_ENGRAVED_LABEL);
    inputCenarioMinX->range(-10000, 1000);
    inputCenarioMinX->step(1);
    inputCenarioMaxX = new Fl_Value_Input(110, 63, 60, 22, "");
    inputCenarioMaxX->range(-1000, 10000);
    inputCenarioMaxX->step(1);

    inputCenarioMinY = new Fl_Value_Input(48, 87, 60, 22, "Y");
    inputCenarioMinY->labeltype(FL_ENGRAVED_LABEL);
    inputCenarioMinY->range(-10000, 1000);
    inputCenarioMinY->step(1);
    inputCenarioMaxY = new Fl_Value_Input(110, 87, 60, 22, "");
    inputCenarioMaxY->range(-1000, 10000);
    inputCenarioMaxY->step(1);

    inputCenarioMinZ = new Fl_Value_Input(48, 111, 60, 22, "Z");
    inputCenarioMinZ->labeltype(FL_ENGRAVED_LABEL);
    inputCenarioMinZ->range(-10000, 1000);
    inputCenarioMinZ->step(1);
    inputCenarioMaxZ = new Fl_Value_Input(110, 111, 60, 22, "");
    inputCenarioMaxZ->range(-1000, 10000);
    inputCenarioMaxZ->step(1);
  grupoTamCenario->end();

  inputNumMaxDesfazer = new Fl_Value_Input(150, 145, 45, 22, "N mx de desfazer:");
  inputNumMaxDesfazer->labeltype(FL_ENGRAVED_LABEL);
  inputNumMaxDesfazer->range(0, 9999);
  inputNumMaxDesfazer->step(1);

  inputEspacamentoGrid = new Fl_Value_Input(150, 167, 45, 22, "Espaamento do Grid:");
  inputEspacamentoGrid->labeltype(FL_ENGRAVED_LABEL);
  inputEspacamentoGrid->range(1, 500);
  inputEspacamentoGrid->step(1);

  btOk = new Fl_Return_Button(110, 195, 75, 25, "&OK");
  btOk->callback((Fl_Callback*)Preferencias::callbackOk, this);
  btCancel = new Fl_Button(30, 195, 75, 25, "&Cancel");
  btCancel->callback((Fl_Callback*)Preferencias::callbackCancel, this);
  set_modal();
  end();
}

// *************************************************************************************
// Method.....: show()
// Function...: Show the Preference Options Window, with actual values
// *************************************************************************************
void Preferencias::show() {
  Fl_Window::show();
  inputCenarioMinX->value(cenarioMinX);
  inputCenarioMinY->value(cenarioMinY);
  inputCenarioMinZ->value(cenarioMinZ);
  inputCenarioMaxX->value(cenarioMaxX);
  inputCenarioMaxY->value(cenarioMaxY);
  inputCenarioMaxZ->value(cenarioMaxZ);
  inputNumMaxDesfazer->value(numMaxDesfazer);
  inputEspacamentoGrid->value(espacamentoGrid);
}

// *************************************************************************************
// Method.....: callbackOk(Fl_Return_Button *bt, void *val)
// Function...: Apply changed preferences
// *************************************************************************************
void Preferencias::callbackOk(Fl_Return_Button *bt, void *val) {
  Preferencias *pref = (Preferencias*)val;
  if(pref->inputCenarioMinX->value() >= pref->inputCenarioMaxX->value()) {
    fl_alert("O valor mnimo de X no pode ser maior que o valor mximo.");
  } else if(pref->inputCenarioMinY->value() >= pref->inputCenarioMaxY->value()) {
    fl_alert("O valor mnimo de Y no pode ser maior que o valor mximo.");
  } else if(pref->inputCenarioMinZ->value() >= pref->inputCenarioMaxZ->value()) {
    fl_alert("O valor mnimo de Z no pode ser maior que o valor mximo.");
  } else {
    pref->hide();
    pref->cenarioMinX = (int)pref->inputCenarioMinX->value();
    pref->cenarioMinY = (int)pref->inputCenarioMinY->value();
    pref->cenarioMinZ = (int)pref->inputCenarioMinZ->value();
    pref->cenarioMaxX = (int)pref->inputCenarioMaxX->value();
    pref->cenarioMaxY = (int)pref->inputCenarioMaxY->value();
    pref->cenarioMaxZ = (int)pref->inputCenarioMaxZ->value();
    pref->numMaxDesfazer = (int)pref->inputNumMaxDesfazer->clamp(pref->inputNumMaxDesfazer->value());
    pref->espacamentoGrid = (int)pref->inputEspacamentoGrid->clamp(pref->inputEspacamentoGrid->value());
    pref->numLinhasGridX = (int)ceil((pref->cenarioMaxX - pref->cenarioMinX) / pref->espacamentoGrid) + 1;
    pref->numLinhasGridZ = (int)ceil((pref->cenarioMaxZ - pref->cenarioMinZ) / pref->espacamentoGrid) + 1;
    pref->diagonalCenario = sqrt((pref->cenarioMaxX - pref->cenarioMinX)*(pref->cenarioMaxX - pref->cenarioMinX)
                               + (pref->cenarioMaxY - pref->cenarioMinY)*(pref->cenarioMaxY - pref->cenarioMinY)
                               + (pref->cenarioMaxZ - pref->cenarioMinZ)*(pref->cenarioMaxZ - pref->cenarioMinZ));
    janela->panelGl->SetPosicaoLuz(pref->cenarioMinX + (pref->cenarioMaxX - pref->cenarioMinX) / 2.0,
                                   pref->cenarioMaxY,
                                   pref->cenarioMinZ + (pref->cenarioMaxZ - pref->cenarioMinZ) / 2.0, 1.0);
    SceneEditor::callbackVisao(NULL, NULL); // Ajusta a visao
  }
}

// *************************************************************************************
// Method.....: callbackCancel(Fl_Button *bt, void *val)
// Function...: Abort preferences
// *************************************************************************************
void Preferencias::callbackCancel(Fl_Button *bt, void *val) {
  ((Preferencias*)val)->hide();
}
