//
// Copyright 2005 by Mauro Charo, Rafael Rieder.
// Porto Alegre, Brazil.
//
// This library is free software; you can redistribute it and/or modify it under the
// terms of the GNU Library General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the GNU Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public License along with
// this library; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA 02111-1307 USA.
//
// Please report bugs & problems to maurocharao@yahoo.com.br or rafaelrieder@gmail.com
//

// *************************************************************************************
//  SceneEditor.cxx
//  Principal Scene Editor Source: create the most important Application parts
//  Used toolkits: FLTK 1.1.x e FLUID 1.1.0
// *************************************************************************************
#include "SceneEditor.h"
#include "About.h"
#include "Acao.h"
#include "Splash.h"
#include "3DLoaders/OBJ/SmVR_COBJLoader.h"
#include "SmVR_CMyEnvelope.h"
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Help_Dialog.H>
#include <FL/fl_ask.H>
#include <string.h>
#include <math.h>
#include <stdlib.h>

SmVR_CGeometricObject *RootObject = NULL;
SmVR_CGeometricObject *user = NULL, *alvo = NULL, *obj_recortado = NULL;
SmVR_CPoint *pt_user = new SmVR_CPoint(0, 0, 50);
SmVR_CPoint *pt_alvo = new SmVR_CPoint(0, 0, 0);
SmVR_CPoint Zero(0, 0, 0);
SmVR_CMyEnvelope *envelope = NULL;
float translateX = 0, translateY = 0, translateZ = 0; // Valores das transformacoes
float rotateX = 0, rotateY = 0, rotateZ = 0;          // atuais no ultimo objeto
float scaleX = 1, scaleY = 1, scaleZ = 1;             // manipulado pelo usuario

SceneEditor *janela = NULL;
About *janelaAbout = NULL;
Fl_Help_Dialog *janelaHelp = NULL;
Preferencias *janelaPref = NULL;
Splash *janelaSplash = NULL;
char nome_arq_cenario[512], nome_arq_obj[512];

Fl_Menu_Item SceneEditor::menuPrincipal_itens[] = {
 {"&Arquivo", 0, 0, 0, FL_SUBMENU},
   {"&Novo...", FL_CTRL + 'N', (Fl_Callback*)SceneEditor::callbackMenus},
   {"Abrir &Cenrio...", FL_ALT + 'O', (Fl_Callback*)SceneEditor::callbackMenus},
   {"Abrir &Objeto...", FL_CTRL + 'O', (Fl_Callback*)SceneEditor::callbackMenus},
   {"&Salvar...", FL_CTRL + 'S', (Fl_Callback*)SceneEditor::callbackMenus},
   {"S&alvar Como...", FL_CTRL + FL_SHIFT + 'S', (Fl_Callback*)SceneEditor::callbackMenus, 0, FL_MENU_DIVIDER},
   {"Sai&r...", FL_ALT + FL_F + 4, (Fl_Callback*)SceneEditor::callbackMenus},
 {0},
 {"&Editar", 0, 0, 0, FL_SUBMENU},
   {"&Desfazer", FL_CTRL + 'Z', (Fl_Callback*)SceneEditor::callbackMenus},
   {"R&efazer", FL_CTRL + 'Y', (Fl_Callback*)SceneEditor::callbackMenus, 0, FL_MENU_DIVIDER},
   {"Re&nomear", FL_CTRL + 'R', (Fl_Callback*)SceneEditor::callbackMenus},
   {"&Recortar", FL_CTRL + 'X', (Fl_Callback*)SceneEditor::callbackMenus},
   {"&Copiar", FL_CTRL + 'C', (Fl_Callback*)SceneEditor::callbackMenus},
   {"Co&lar", FL_CTRL + 'V', (Fl_Callback*)SceneEditor::callbackMenus},
   {"&Apagar...", FL_CTRL + FL_Delete, (Fl_Callback*)SceneEditor::callbackMenus},
   {"Rese&tar", FL_CTRL + 'T', (Fl_Callback*)SceneEditor::callbackMenus, 0, FL_MENU_DIVIDER},
   {"Selec Pr&ximo", 'P', (Fl_Callback*)SceneEditor::callbackMenus},
   {"Selec Anteri&or", 'O', (Fl_Callback*)SceneEditor::callbackMenus, 0, FL_MENU_DIVIDER},
   {"&Preferncias", FL_CTRL + 'P', (Fl_Callback*)SceneEditor::callbackMenus},
 {0},
 {"&Janela", 0, 0, 0, FL_SUBMENU},
   {"&Tela Cheia", FL_F + 11, (Fl_Callback*)SceneEditor::callbackMenus, 0, FL_MENU_TOGGLE | FL_MENU_DIVIDER},
   {"&Vises", FL_ALT + 'V', (Fl_Callback*)SceneEditor::callbackMenus},
   {"&Propriedades", FL_ALT + 'P', (Fl_Callback*)SceneEditor::callbackMenus},
 {0},
 {"&Sobre", 0, 0, 0, FL_SUBMENU},
   {"&Help...", FL_F + 1, (Fl_Callback*)SceneEditor::callbackMenus},
   {"&About...", FL_F + 2, (Fl_Callback*)SceneEditor::callbackMenus},
 {0},
 {0}
};

Fl_Menu_Item SceneEditor::popup_itens[] = {
 {"Re&nomear", FL_CTRL + 'R', (Fl_Callback*)SceneEditor::callbackMenus},
 {"&Recortar", FL_CTRL + 'X', (Fl_Callback*)SceneEditor::callbackMenus},
 {"&Copiar", FL_CTRL + 'C', (Fl_Callback*)SceneEditor::callbackMenus},
 {"Co&lar", FL_CTRL + 'V', (Fl_Callback*)SceneEditor::callbackMenus},
 {"&Apagar...", FL_CTRL + FL_Delete, (Fl_Callback*)SceneEditor::callbackMenus},
 {"Rese&tar", FL_CTRL + 'T', (Fl_Callback*)SceneEditor::callbackMenus},
 {0}
};


// *************************************************************************************
// Constructor...: Define the SmallVR Scene Editor area (menus, panels, windows, buttons...)
// *************************************************************************************
SceneEditor::SceneEditor() : Fl_Window(0, 25, Fl::w() - 10, Fl::h() - 30, "SmallVR Scene Editor 1.0") {
  int x;
  box(FL_FLAT_BOX);
  color(FL_BACKGROUND_COLOR, FL_BACKGROUND_COLOR);
  callback((Fl_Callback*)callbackJanela);
  size_range(600, 450);
  MenuPrincipal = new Fl_Menu_Bar(0, 0, w(), 25);
  MenuPrincipal->menu(menuPrincipal_itens);
  panelEsquerdo = new Fl_Group(0, 25, 136, h() - 25);
  panelEsquerdo->box(FL_EMBOSSED_FRAME);
    listObjetos = new Fl_Hold_Browser(2, 45, 132, h() - 388, "Objetos");
    listObjetos->labeltype(FL_ENGRAVED_LABEL);
    listObjetos->callback((Fl_Callback*)listObjetosOnClick);
    listObjetos->align(FL_ALIGN_TOP);
    panelEsquerdo->resizable(listObjetos);
    popup = new Fl_Menu_Button(2, 45, 132, h() - 388);
    popup->type(Fl_Menu_Button::POPUP3);
    popup->box(FL_NO_BOX);
    popup->menu(popup_itens);

    btZoom = new Fl_Roller(5, h() - 340, 90, 20, "Zoom");
    btZoom->type(FL_HORIZONTAL);
    btZoom->box(FL_DOWN_BOX);
    btZoom->range(1, 179);
    btZoom->step(-1);
    btZoom->value(90);
    btZoom->callback((Fl_Callback*)callbackZoom);
    btZoom->labeltype(FL_ENGRAVED_LABEL);
    btZoom->align(FL_ALIGN_RIGHT);

    groupPropriedades = new Fl_Group(2, h() - 318, 132, 187, "Propriedades");
    groupPropriedades->box(FL_EMBOSSED_FRAME);
    groupPropriedades->labeltype(FL_EMBOSSED_LABEL);
    groupPropriedades->align(FL_ALIGN_TOP | FL_ALIGN_INSIDE);
      groupTipo = new Fl_Group(15, groupPropriedades->y() + 21, 108, 50);
      groupTipo->box(FL_EMBOSSED_FRAME);
        rbObjeto = new Fl_Round_Button(20, groupPropriedades->y() + 24, 15, 15, "Objeto");
        rbObjeto->type(FL_RADIO_BUTTON);
        rbObjeto->value(1);
        rbObjeto->labeltype(FL_ENGRAVED_LABEL);
        rbObjeto->callback((Fl_Callback*)callbackObjeto);

        rbObservador = new Fl_Round_Button(20, groupPropriedades->y() + 39, 15, 15, "Observador");
        rbObservador->type(FL_RADIO_BUTTON);
        rbObservador->labeltype(FL_ENGRAVED_LABEL);
        rbObservador->callback((Fl_Callback*)callbackObservador);

        rbAlvo = new Fl_Round_Button(20, groupPropriedades->y() + 54, 15, 15, "Alvo");
        rbAlvo->type(FL_RADIO_BUTTON);
        rbAlvo->labeltype(FL_ENGRAVED_LABEL);
        rbAlvo->callback((Fl_Callback*)callbackObservador);
      groupTipo->end();

      btTranslacao = new Fl_Round_Button(20, groupPropriedades->y() + 73, 15, 15, "Translao");
      btTranslacao->type(FL_RADIO_BUTTON);
      btTranslacao->value(1);
      btTranslacao->labeltype(FL_ENGRAVED_LABEL);
      btTranslacao->callback((Fl_Callback*)callbackTranslacao);

      btEscala = new Fl_Round_Button(20, groupPropriedades->y() + 88, 15, 15, "Escala");
      btEscala->type(FL_RADIO_BUTTON);
      btEscala->labeltype(FL_ENGRAVED_LABEL);
      btEscala->callback((Fl_Callback*)callbackEscala);

      btRotacao = new Fl_Round_Button(20, groupPropriedades->y() + 103, 15, 15, "Rotao");
      btRotacao->type(FL_RADIO_BUTTON);
      btRotacao->labeltype(FL_ENGRAVED_LABEL);
      btRotacao->callback((Fl_Callback*)callbackRotacao);

      (new Fl_Box(5, groupPropriedades->y() + 121, 13, 20, "X"))->labeltype(FL_ENGRAVED_LABEL);
      decXfast = new Fl_Repeat_Button(16, groupPropriedades->y() + 121, 18, 20, "@<<");
      decXfast->callback((Fl_Callback*)callbackMudaCampos);
      decX = new Fl_Repeat_Button(34, groupPropriedades->y() + 121, 15, 20, "@<");
      decX->callback((Fl_Callback*)callbackMudaCampos);
      valorX = new Fl_Value_Input(49, groupPropriedades->y() + 121, 49, 20, "");
      valorX->callback((Fl_Callback*)callbackMudaCampos2);
      valorX->when(FL_WHEN_CHANGED);
      valorX->align(FL_ALIGN_LEFT);
      incX = new Fl_Repeat_Button(98, groupPropriedades->y() + 121, 15, 20, "@>");
      incX->callback((Fl_Callback*)callbackMudaCampos);
      incXfast = new Fl_Repeat_Button(113, groupPropriedades->y() + 121, 18, 20, "@>>");
      incXfast->callback((Fl_Callback*)callbackMudaCampos);

      (new Fl_Box(5, groupPropriedades->y() + 142, 13, 20, "Y"))->labeltype(FL_ENGRAVED_LABEL);
      decYfast = new Fl_Repeat_Button(16, groupPropriedades->y() + 142, 18, 20, "@<<");
      decYfast->callback((Fl_Callback*)callbackMudaCampos);
      decY = new Fl_Repeat_Button(34, groupPropriedades->y() + 142, 15, 20, "@<");
      decY->callback((Fl_Callback*)callbackMudaCampos);
      valorY = new Fl_Value_Input(49, groupPropriedades->y() + 142, 49, 20, "");
      valorY->callback((Fl_Callback*)callbackMudaCampos2);
      valorY->when(FL_WHEN_CHANGED);
      valorY->align(FL_ALIGN_LEFT);
      incY = new Fl_Repeat_Button(98, groupPropriedades->y() + 142, 15, 20, "@>");
      incY->callback((Fl_Callback*)callbackMudaCampos);
      incYfast = new Fl_Repeat_Button(113, groupPropriedades->y() + 142, 18, 20, "@>>");
      incYfast->callback((Fl_Callback*)callbackMudaCampos);

      (new Fl_Box(5, groupPropriedades->y() + 163, 13, 20, "Z"))->labeltype(FL_ENGRAVED_LABEL);
      decZfast = new Fl_Repeat_Button(16, groupPropriedades->y() + 163, 18, 20, "@<<");
      decZfast->callback((Fl_Callback*)callbackMudaCampos);
      decZ = new Fl_Repeat_Button(34, groupPropriedades->y() + 163, 15, 20, "@<");
      decZ->callback((Fl_Callback*)callbackMudaCampos);
      valorZ = new Fl_Value_Input(49, groupPropriedades->y() + 163, 49, 20, "");
      valorZ->callback((Fl_Callback*)callbackMudaCampos2);
      valorZ->when(FL_WHEN_CHANGED);
      valorZ->align(FL_ALIGN_LEFT);
      incZ = new Fl_Repeat_Button(98, groupPropriedades->y() + 163, 15, 20, "@>");
      incZ->callback((Fl_Callback*)callbackMudaCampos);
      incZfast = new Fl_Repeat_Button(113, groupPropriedades->y() + 163, 18, 20, "@>>");
      incZfast->callback((Fl_Callback*)callbackMudaCampos);
    groupPropriedades->end();

    groupVisao = new Fl_Group(2, h() - 131, 132, 131, "Viso");
    groupVisao->box(FL_EMBOSSED_FRAME);
    groupVisao->labeltype(FL_EMBOSSED_LABEL);
    groupVisao->align(FL_ALIGN_TOP | FL_ALIGN_INSIDE);
      rbPerspectiva = new Fl_Round_Button(5, groupVisao->y() + 16, 15, 15, "Perspectiva");
      rbPerspectiva->type(FL_RADIO_BUTTON);
      rbPerspectiva->labeltype(FL_ENGRAVED_LABEL);
      rbPerspectiva->callback((Fl_Callback*)callbackVisao);

      rbFrontal = new Fl_Round_Button(5, groupVisao->y() + 32, 15, 15, "Frontal");
      rbFrontal->type(FL_RADIO_BUTTON);
      rbFrontal->labeltype(FL_ENGRAVED_LABEL);
      rbFrontal->callback((Fl_Callback*)callbackVisao);
      rbFrontal->value(1);

      rbTraseira = new Fl_Round_Button(5, groupVisao->y() + 48, 15, 15, "Traseira");
      rbTraseira->type(FL_RADIO_BUTTON);
      rbTraseira->labeltype(FL_ENGRAVED_LABEL);
      rbTraseira->callback((Fl_Callback*)callbackVisao);

      rbDireita = new Fl_Round_Button(5, groupVisao->y() + 64, 15, 15, "Lateral Direita");
      rbDireita->type(FL_RADIO_BUTTON);
      rbDireita->labeltype(FL_ENGRAVED_LABEL);
      rbDireita->callback((Fl_Callback*)callbackVisao);

      rbEsquerda = new Fl_Round_Button(5, groupVisao->y() + 80, 15, 15, "Lateral Esquerda");
      rbEsquerda->type(FL_RADIO_BUTTON);
      rbEsquerda->labeltype(FL_ENGRAVED_LABEL);
      rbEsquerda->callback((Fl_Callback*)callbackVisao);

      rbSuperior = new Fl_Round_Button(5, groupVisao->y() + 96, 15, 15, "Superior");
      rbSuperior->type(FL_RADIO_BUTTON);
      rbSuperior->labeltype(FL_ENGRAVED_LABEL);
      rbSuperior->callback((Fl_Callback*)callbackVisao);

      rbInferior = new Fl_Round_Button(5, groupVisao->y() + 112, 15, 15, "Inferior");
      rbInferior->type(FL_RADIO_BUTTON);
      rbInferior->labeltype(FL_ENGRAVED_LABEL);
      rbInferior->callback((Fl_Callback*)callbackVisao);
    groupVisao->end();
  panelEsquerdo->end();

  boxTile = new Fl_Box(w() - 287, 25, 151, h() - 25); // Para o tile
  tileJanela = new Fl_Tile(136, 25, w() - 136, h() - 25);
  tileJanela->resizable(boxTile);
  tileJanela->callback((Fl_Callback*)callbackTile);
    panelGl = new Gl_Group(136, 25, w() - 273, h() - 25);
    groupVisoes = new Fl_Group(w() - 137, 25, 137, h() - 25, "Vises");
    groupVisoes->box(FL_EMBOSSED_FRAME);
    groupVisoes->labeltype(FL_EMBOSSED_LABEL);
    groupVisoes->align(FL_ALIGN_TOP | FL_ALIGN_INSIDE);
    groupVisoes->resizable(NULL);
      vistaGL1 = new Gl_Group(w() - 136, 66, 135, 135, DIREITA);
      rbVistaGL1 = new Fl_Round_Button(w() - 137, 48, 15, 15);
      rbVistaGL1->type(FL_TOGGLE_BUTTON);
      rbVistaGL1->labeltype(FL_ENGRAVED_LABEL);
      rbVistaGL1->value(1);
      rbVistaGL1->callback((Fl_Callback*)callbackVistas);
      btZoomVistaGL1 = new Fl_Roller(w() - 122, 45, 22, 20);
      btZoomVistaGL1->type(FL_HORIZONTAL);
      btZoomVistaGL1->box(FL_DOWN_BOX);
      btZoomVistaGL1->tooltip("Zoom");
      btZoomVistaGL1->range(1, 179);
      btZoomVistaGL1->step(-1);
      btZoomVistaGL1->value(90);
      btZoomVistaGL1->callback((Fl_Callback*)callbackZoom, vistaGL1);
      choiceVistaGL1 = new Fl_Choice(w() - 100, 45, 98, 20);
      for(x = 1; x < 8; x++)
        choiceVistaGL1->add(Gl_Group::label_vista[x], 0, (Fl_Callback*)callbackChoiceVisao, (void*)x);
      choiceVistaGL1->value(DIREITA - 1);

      vistaGL2 = new Gl_Group(w() - 136, 227, 135, 135, PERSPECTIVA);
      rbVistaGL2 = new Fl_Round_Button(w() - 137, 209, 15, 15);
      rbVistaGL2->type(FL_TOGGLE_BUTTON);
      rbVistaGL2->labeltype(FL_ENGRAVED_LABEL);
      rbVistaGL2->value(1);
      rbVistaGL2->callback((Fl_Callback*)callbackVistas);
      btZoomVistaGL2 = new Fl_Roller(w() - 122, 206, 22, 20);
      btZoomVistaGL2->type(FL_HORIZONTAL);
      btZoomVistaGL2->box(FL_DOWN_BOX);
      btZoomVistaGL2->tooltip("Zoom");
      btZoomVistaGL2->range(1, 179);
      btZoomVistaGL2->step(-1);
      btZoomVistaGL2->value(90);
      btZoomVistaGL2->callback((Fl_Callback*)callbackZoom, vistaGL2);
      choiceVistaGL2 = new Fl_Choice(w() - 100, 206, 98, 20);
      for(x = 1; x < 8; x++)
        choiceVistaGL2->add(Gl_Group::label_vista[x], 0, (Fl_Callback*)callbackChoiceVisao, (void*)x);
      choiceVistaGL2->value(PERSPECTIVA - 1);

      vistaGL3 = new Gl_Group(w() - 136, 389, 135, 135, SUPERIOR);
      rbVistaGL3 = new Fl_Round_Button(w() - 137, 372, 15, 15);
      rbVistaGL3->type(FL_TOGGLE_BUTTON);
      rbVistaGL3->labeltype(FL_ENGRAVED_LABEL);
      rbVistaGL3->value(1);
      rbVistaGL3->callback((Fl_Callback*)callbackVistas);
      btZoomVistaGL3 = new Fl_Roller(w() - 122, 368, 22, 20);
      btZoomVistaGL3->type(FL_HORIZONTAL);
      btZoomVistaGL3->box(FL_DOWN_BOX);
      btZoomVistaGL3->tooltip("Zoom");
      btZoomVistaGL3->range(1, 179);
      btZoomVistaGL3->step(-1);
      btZoomVistaGL3->value(90);
      btZoomVistaGL3->callback((Fl_Callback*)callbackZoom, vistaGL3);
      choiceVistaGL3 = new Fl_Choice(w() - 100, 368, 98, 20);
      for(x = 1; x < 8; x++)
        choiceVistaGL3->add(Gl_Group::label_vista[x], 0, (Fl_Callback*)callbackChoiceVisao, (void*)x);
      choiceVistaGL3->value(SUPERIOR - 1);
    groupVisoes->end();
  tileJanela->end();
  resizable(tileJanela);
  end();
}

// *************************************************************************************
// Function...: setaEnvelope(SmVR_CGeometricObject *o)
//              Put a envelope bounding box to each object
// *************************************************************************************
void setaEnvelope(SmVR_CGeometricObject *o) {
  static SmVR_CPoint *min = new SmVR_CPoint(0, 0, 0);
  static SmVR_CPoint *max = new SmVR_CPoint(0, 0, 0);
  static SmVR_Matrix m;

  if(o == NULL || o->GetObjectFromFile() == NULL) { // NULL, User e alvo nao tem envelope
    if(envelope != NULL) {
      envelope->Remove(RootObject);
      delete envelope;
      envelope = NULL;
    }
  } else if(o->GetFirstChild() == NULL) { // Se ja tem envelope nao adiciona
    if(envelope != NULL) {
      envelope->Remove(RootObject);
      delete envelope;
    }
    envelope = new SmVR_CMyEnvelope("envelope");
    o->GetLocalTransformationMatrix(m);
    o->GetObjectFromFile()->GetBoundaries(min, max);
    envelope->SetBoundaries(min, max);
    envelope->SetLocalTransformationMatrix(m);
    o->AddChild(envelope);
  }
}

// *************************************************************************************
// Function...: PrintObject(SmVR_CGeometricObject *o, FILE *arq)
//              Print object name and object name file in SVR file
// *************************************************************************************
void PrintObject(SmVR_CGeometricObject *o, FILE *arq) {
  static SmVR_Matrix m;

  if(o == NULL || o->GetObjectFromFile() == NULL) return;
  o->GetGlobalTransformationMatrix(m);
  fprintf(arq, "NAME %s\n", o->GetName());
  fprintf(arq, "FILE %s\n", ((SmVR_COBJLoader*)o->GetObjectFromFile())->GetFileName());
  for(int i = 0; i < 4; i++) {
    for(int j = 0; j < 4; j++)
      fprintf(arq, "%10.4lf", m[i][j]);
    fprintf(arq, "\n");
  }
}

// *************************************************************************************
// Function...: SaveChildren(char *filename, SmVR_CGeometricObject *o)
//              Save children of each object in SVR file
// *************************************************************************************
void SaveChildren(char *filename, SmVR_CGeometricObject *o) {
  FILE *arq;
  if(arq = fopen(filename, "wt")) {
    for(SmVR_CGeometricObject *temp = o->GetFirstChild(); temp; temp = temp->GetBrother())
      PrintObject(temp, arq);
    fclose(arq);
  }
  else printf("Problemas ao criar o arquivo '%s'\n", filename);
}

// *************************************************************************************
// Function...: LoadChildren(char *filename, SmVR_CGeometricObject *o)
//              Read SVR file, and load children of each object
//              After that, load the OBJ files and create a new SmVR object
// *************************************************************************************
void LoadChildren(char *filename, SmVR_CGeometricObject *o) {
  FILE *arq;
  char str1[512], ObjName[512], ObjFileName[512];
  int line = 0, i;
  SmVR_CGeometricObject *temp;
  SmVR_COBJLoader *tempObj;
  SmVR_Matrix m;

  if(arq = fopen(filename, "rt")) {
    while(!feof(arq)) {
      fgets(str1, 512, arq);
      if(feof(arq)) break;
      line++;
      if(!strstr(str1, "NAME")) {
        fl_alert("Tag NAME esperada na linha %d = '%s'", line, str1);
        break;
      }
      sprintf(ObjName, "%s", &str1[5]);
      ObjName[strlen(ObjName) - 1] = '\0'; // Tira o \n
      if(!strlen(ObjName)) {
        fl_alert("Nome do objeto = '%s' invlido na linha %d", ObjName, line);
        break;
      }
      fgets(str1, 512, arq);
      line++;
      if(!strstr(str1, "FILE")) {
        fl_alert("Tag FILE esperada na linha %d='%s'", line, str1);
        break;
      }
      sprintf(ObjFileName, "%s", &str1[5]);
      ObjFileName[strlen(ObjFileName) - 1] = '\0'; // Tira o \n
      if(!strlen(ObjFileName)) {
        fl_alert("Nome de arquivo '%s' invlido na linha %d", ObjFileName, line);
        break;
      }
      for(i = 0; i < 4; i++) {
        fscanf(arq, "%f %f %f %f\n", &m[i][0], &m[i][1], &m[i][2], &m[i][3]);
        line++;
      }
      // load the OBJ file
      tempObj = new SmVR_COBJLoader();
      if(!tempObj->Load(ObjFileName)) {
        delete tempObj;
        fl_alert("Arquivo '%s' no encontrado", ObjFileName);
        break;
      }
      // Create a new SmVR object and add it to the tree
      temp = new SmVR_CGeometricObject(ObjName, tempObj);
      temp->SetLocalTransformationMatrix(m);
      o->AddChild(temp);
    }
    fclose(arq);
  }
  else fl_alert("Problemas ao abrir o arquivo '%s'", filename);
}

// *************************************************************************************
// Method.....: popula_lista(SmVR_CGeometricObject *ob)
// Function...: Fill out the list of objects
// *************************************************************************************
void SceneEditor::popula_lista(SmVR_CGeometricObject *ob) {
  if(ob == NULL) return;
  if(ob->GetObjectFromFile() != NULL)
    janela->listObjetos->add(ob->GetName(), ob);
  if(ob->GetFirstChild() != NULL) {
    SmVR_CGeometricObject *obj = ob->GetFirstChild();
    popula_lista(obj);
    while(obj != NULL) {
      if(obj->GetBrother() != NULL)
        popula_lista(obj = obj->GetBrother());
      else break;
    }
  }
}

// *************************************************************************************
// Function...: repintaTudo(int invalidate = 0)
//              Redraw the GL Windows
// *************************************************************************************
void repintaTudo(int invalidate = 0) {
  char str_label[256];

  if(invalidate) {
    sprintf(str_label, "SmallVR Scene Editor 1.0  Camera: (%0.3f,%0.3f,%0.3f),(%0.3f,%0.3f,%0.3f)",
            pt_user->X, pt_user->Y, pt_user->Z,
            pt_alvo->X, pt_alvo->Y, pt_alvo->Z);
    janela->label(str_label);
    janela->panelGl->invalidate();
  }
  janela->panelGl->redraw();

  if(janela->vistaGL1->visible()) {
    if(invalidate) janela->vistaGL1->invalidate();
    janela->vistaGL1->redraw();
  }
  if(janela->vistaGL2->visible()) {
    if(invalidate) janela->vistaGL2->invalidate();
    janela->vistaGL2->redraw();
  }
  if(janela->vistaGL3->visible()) {
    if(invalidate) janela->vistaGL3->invalidate();
    janela->vistaGL3->redraw();
  }
}

// *************************************************************************************
// Function...: limpaCenario()
//              Clean the GL Windows (clean the scene)
// *************************************************************************************
void limpaCenario() {
  SmVR_CGeometricObject *obj_aux;
  sprintf(nome_arq_cenario, "");
  Acao::limpa();
  while(janela->listObjetos->size()) {
    obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->size());
    obj_aux->Remove(RootObject);
    if(obj_aux->GetObjectFromFile() != NULL && (obj_recortado == NULL || obj_aux->GetObjectFromFile() != obj_recortado->GetObjectFromFile()))
      delete obj_aux->GetObjectFromFile();
    delete obj_aux;
    janela->listObjetos->remove(janela->listObjetos->size());
  }
  setaEnvelope(NULL);
  janela->callbackObjeto(NULL, NULL);
}

// *************************************************************************************
// Function...: existeNome(char* nome)
//              Verify if exists repeated object names
// *************************************************************************************
int existeNome(char* nome) {
  for(int x = 1; x <= janela->listObjetos->size(); x++)
    if(!strcmp(janela->listObjetos->text(x), nome))
      return 1;
  return 0;
}

// *************************************************************************************
// Function...: criaNome(char* nome)
//              Create a name (label) to any object, based in the name file
// *************************************************************************************
char* criaNome(char* nome) {
  char* nome_novo = new char[128];
  int cont = 1;

  sprintf(nome_novo, "%s", nome);
  while(existeNome(nome_novo))
    sprintf(nome_novo, "%s%d", nome, cont++);
  return nome_novo;
}

// *************************************************************************************
// Method.....: adicionaObjeto(SmVR_CGeometricObject *obj_aux)
// Function...: Add object in SmallVR RootObject and list of objects
// *************************************************************************************
void SceneEditor::adicionaObjeto(SmVR_CGeometricObject *obj_aux) {
  RootObject->AddChild(obj_aux);
  listObjetos->add(obj_aux->GetName(), obj_aux);
  listObjetos->select(listObjetos->size());
  callbackObjeto(NULL, NULL);
}

// *************************************************************************************
// Method.....: selecionaObjeto(char *nome)
// Function...: Select an object
// *************************************************************************************
void SceneEditor::selecionaObjeto(char *nome) {
  if(strcmp(listObjetos->text(listObjetos->value()), nome)) {
    for(int x = 1; x <= listObjetos->size(); x++) {
      if(!strcmp(listObjetos->text(x), nome)) {
        listObjetos->select(x);
        break;
      }
    }
  }
}

// *************************************************************************************
// Method.....: apagaObjetoAtual()
// Function...: Delete the selected object
// *************************************************************************************
void SceneEditor::apagaObjetoAtual() {
  int linha = listObjetos->value();
  ((SmVR_CGeometricObject*)listObjetos->data(linha))->Remove(RootObject);
  listObjetos->remove(linha);
  if(listObjetos->size())
    listObjetos->select(linha <= listObjetos->size() ? linha : listObjetos->size());
  callbackObjeto(NULL, NULL);
}

// *************************************************************************************
// Method.....: salvaCenario()
// Function...: Save a 3D scene
// *************************************************************************************
void SceneEditor::salvaCenario() {
  char *newfile;
  if(listObjetos->size() == 0) {
    fl_alert("O cenrio est vazio e no pode ser salvo.");
  } else if(strlen(nome_arq_cenario)) {
    SaveChildren(nome_arq_cenario, RootObject);
  } else if((newfile = fl_file_chooser("Salvar Cenrio...", "*.svr", "")) != NULL) {
    fl_filename_relative(nome_arq_cenario, 512, newfile);
    SaveChildren(nome_arq_cenario, RootObject);
  }
}

// *************************************************************************************
// Method.....: callbackMenus(Fl_Menu_ *menu, void *val)
// Function...: Control menus of the SceneEditor
// *************************************************************************************
void SceneEditor::callbackMenus(Fl_Menu_ *menu, void *val) {
  static int px, py, pw, ph, is_full_screen = 0;
  SmVR_CGeometricObject *obj_aux;
  SmVR_COBJLoader *loader;
  char *newfile;
  const char* str;
  int x, linha;

  if(!strcmp(menu->text(), "&Novo...")) {
    if(janela->listObjetos->size() && Acao::podeDesfazer() && fl_ask("Deseja salvar o cenrio atual ?"))
      janela->salvaCenario();
    limpaCenario();
  } else if(!strcmp(menu->text(), "Abrir &Cenrio...")) {
    if(janela->listObjetos->size() && Acao::podeDesfazer() && fl_ask("Deseja salvar o cenrio atual ?"))
      janela->salvaCenario();
    if((newfile = fl_file_chooser("Abrir Cenrio", "*.svr", "")) != NULL) {
      limpaCenario();
      fl_filename_relative(nome_arq_cenario, 512, newfile);
      LoadChildren(nome_arq_cenario, RootObject);
      SceneEditor::popula_lista(RootObject);
      janela->listObjetos->select(janela->listObjetos->size());
      callbackObjeto(NULL, NULL);
    }
  } else if(!strcmp(menu->text(), "Abrir &Objeto...")) {
    if((newfile = fl_file_chooser("Abrir Objeto...", "*.obj", "")) != NULL) {
      fl_filename_relative(nome_arq_obj, 512, newfile);
      loader = new SmVR_COBJLoader();
      loader->Load(nome_arq_obj);
      janela->adicionaObjeto(obj_aux = new SmVR_CGeometricObject(criaNome(nome_arq_obj), loader));
      Acao::adicionaAcao(obj_aux, INSERIR);
    }
  } else if(!strcmp(menu->text(), "&Salvar...")) {
    janela->salvaCenario();
  } else if(!strcmp(menu->text(), "S&alvar Como...")) {
    if(janela->listObjetos->size() == 0) {
      fl_alert("O cenrio est vazio e no pode ser salvo.");
    } else if((newfile = fl_file_chooser("Salvar Cenrio...", "*.svr", "")) != NULL) {
      fl_filename_relative(nome_arq_cenario, 512, newfile);
      SaveChildren(nome_arq_cenario, RootObject);
    }
  } else if(!strcmp(menu->text(), "Sai&r...")) {
    callbackJanela(janela, NULL);
  } else if(!strcmp(menu->text(), "&Desfazer")) {
    Acao::desfazer();
  } else if(!strcmp(menu->text(), "R&efazer")) {
    Acao::refazer();
  } else if(!strcmp(menu->text(), "Re&nomear")) {
    if(janela->listObjetos->value() == 0) {
      fl_alert("Selecione um objeto para renomear.");
    } else {
      obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
      while((str = fl_input("Digite o nome:", obj_aux->GetName())) != NULL && strlen(str)) {
        if(!strcmp(obj_aux->GetName(), str)) break; // Ficou o mesmo nome
        if(existeNome((char*)str)) {
          fl_alert("Este nome j existe, tente outro.");
        } else {
          Acao::adicionaAcao(obj_aux, RENOMEAR, 0, (char*)str);
          obj_aux->SetName((char*)str);
          janela->listObjetos->text(janela->listObjetos->value(), obj_aux->GetName());
          break;
        }
      }
    }
  } else if(!strcmp(menu->text(), "&Recortar")) {
    if(janela->listObjetos->value() == 0) {
      fl_alert("Selecione um objeto para recortar.");
    } else {
      obj_recortado = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
      Acao::adicionaAcao(obj_recortado, REMOVER);
      janela->apagaObjetoAtual();
    }
  } else if(!strcmp(menu->text(), "&Copiar")) {
    if(janela->listObjetos->value() == 0) {
      fl_alert("Selecione um objeto para copiar.");
    } else {
      if(obj_recortado != NULL) {
        if(!Acao::existeLoader(obj_recortado->GetObjectFromFile())) {
          for(x = 1; x <= janela->listObjetos->size(); x++)
            if(((SmVR_CGeometricObject*)janela->listObjetos->data(x))->GetObjectFromFile() == obj_recortado->GetObjectFromFile())
              break;
          if(x > janela->listObjetos->size()) // Nao achou outro com mesmo objFromFile
            delete obj_recortado->GetObjectFromFile();
        }
        delete obj_recortado;
      }
      obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
      obj_recortado = new SmVR_CGeometricObject(obj_aux->GetName(), obj_aux->GetObjectFromFile());
    }
  } else if(!strcmp(menu->text(), "Co&lar")) {
    if(obj_recortado != NULL) {
      obj_aux = new SmVR_CGeometricObject(criaNome(obj_recortado->GetName()), obj_recortado->GetObjectFromFile());
      obj_aux->TranslateBy(pt_alvo->X, pt_alvo->Y, pt_alvo->Z); // Posiciona na cara do usuario
      janela->adicionaObjeto(obj_aux);
      Acao::adicionaAcao(obj_aux, INSERIR);
    }
  } else if(!strcmp(menu->text(), "&Apagar...")) {
    if(janela->listObjetos->value() == 0) {
      fl_alert("Selecione um objeto para apagar.");
    } else if(fl_ask("Deseja realmente apagar o objeto '%s' ?", janela->listObjetos->text(janela->listObjetos->value()))) {
      Acao::adicionaAcao((SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value()), REMOVER);
      janela->apagaObjetoAtual();
    }
  } else if(!strcmp(menu->text(), "Rese&tar")) {
    if(janela->listObjetos->value() == 0) {
      fl_alert("Selecione um objeto para resetar.");
    } else if(fl_ask("Deseja realmente limpar as transformaes do objeto '%s' ?", janela->listObjetos->text(janela->listObjetos->value()))) {
      obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
      Acao::adicionaAcao(obj_aux, RESETAR);
      obj_aux->SetLocalTransformationMatrixToIdentity();
      callbackObjeto(NULL, NULL);
    }
  } else if(!strcmp(menu->text(), "&Preferncias")) {
    janelaPref->show();
  } else if(!strcmp(menu->text(), "Selec Pr&ximo")) {
    if(janela->listObjetos->size()) {
      linha = (janela->listObjetos->value() < janela->listObjetos->size() ? janela->listObjetos->value() + 1 : 1);
      janela->listObjetos->select(linha);
      setaEnvelope((SmVR_CGeometricObject*)janela->listObjetos->data(linha));
      callbackObjeto(NULL, NULL);
    }
  } else if(!strcmp(menu->text(), "Selec Anteri&or")) {
    if(janela->listObjetos->size()) {
      linha = (janela->listObjetos->value() > 1 ? janela->listObjetos->value() - 1 : janela->listObjetos->size());
      janela->listObjetos->select(linha);
      setaEnvelope((SmVR_CGeometricObject*)janela->listObjetos->data(linha));
      callbackObjeto(NULL, NULL);
    }
  } else if(!strcmp(menu->text(), "&Tela Cheia")) {
    if(is_full_screen) {
      janela->fullscreen_off(px, py, pw, ph);
    } else {
      px = janela->x();
      py = janela->y();
      pw = janela->w();
      ph = janela->h();
      janela->fullscreen();
    }
    is_full_screen = !is_full_screen;
  } else if(!strcmp(menu->text(), "&Vises")) {
    if(janela->groupVisoes->visible()) {
      janela->groupVisoes->hide();
      janela->panelGl->resize((janela->panelEsquerdo->visible() ? janela->panelEsquerdo->w() : 0), 25, janela->w() - (janela->panelEsquerdo->visible() ? janela->panelEsquerdo->w() : 0), janela->h() - 25);
      janela->boxTile->resize(janela->groupVisoes->x(), 25, 0, janela->h() - 25); // Nao deixa o tile ativo
    } else {
      janela->panelGl->resize((janela->panelEsquerdo->visible() ? janela->panelEsquerdo->w() : 0), 25, janela->w() - (janela->panelEsquerdo->visible() ? janela->panelEsquerdo->w() : 0) - janela->groupVisoes->w(), janela->h() - 25);
      janela->groupVisoes->show();
      janela->boxTile->resize(janela->w() - 287, 25, 151, janela->h() - 25); // Ativa o tile
    }
    janela->rbVistaGL1->value(janela->groupVisoes->visible());
    janela->rbVistaGL2->value(janela->groupVisoes->visible());
    janela->rbVistaGL3->value(janela->groupVisoes->visible());
    SceneEditor::callbackVistas(NULL, NULL);
  } else if(!strcmp(menu->text(), "&Propriedades")) {
    if(janela->panelEsquerdo->visible()) {
      janela->panelEsquerdo->hide();
      janela->panelGl->resize(0, 25, janela->w() - (janela->groupVisoes->visible() ? janela->groupVisoes->w() : 0), janela->h() - 25);
    } else {
      janela->panelGl->resize(janela->panelEsquerdo->w(), 25, janela->w() - janela->panelEsquerdo->w() - (janela->groupVisoes->visible() ? janela->groupVisoes->w() : 0), janela->h() - 25);
      janela->panelEsquerdo->show();
    }
  } else if(!strcmp(menu->text(), "&About...")) {
    janelaAbout->show();
  } else if(!strcmp(menu->text(), "&Help...")) {
    janelaHelp->show();
  }
}

// *************************************************************************************
// Method.....: callbackJanela(Fl_Window *obj, void *val)
// Function...: Application.terminate :-D
// *************************************************************************************
void SceneEditor::callbackJanela(Fl_Window *obj, void *val) {
  janela->hide();
  exit(0);
}

// *************************************************************************************
// Method.....: callbackTile(Fl_Widget *obj, void *val)
// Function...: Resize group tiles
// *************************************************************************************
void SceneEditor::callbackTile(Fl_Widget *obj, void *val) {
  if(janela->groupVisoes->visible()) {
    janela->rbVistaGL1->position(janela->groupVisoes->x(), 48);
    janela->btZoomVistaGL1->resize(janela->groupVisoes->x() + 15, 45, janela->groupVisoes->w() - 115, 20);
    janela->choiceVistaGL1->position(janela->w() - 100, 45);
    janela->vistaGL1->resize(janela->groupVisoes->x() + 1, 66, janela->groupVisoes->w() - 2, janela->groupVisoes->w() - 2);

    janela->rbVistaGL2->position(janela->groupVisoes->x(), janela->vistaGL1->y() + janela->vistaGL1->h() + 5);
    janela->btZoomVistaGL2->resize(janela->groupVisoes->x() + 15, janela->vistaGL1->y() + janela->vistaGL1->h() + 2, janela->groupVisoes->w() - 115, 20);
    janela->choiceVistaGL2->position(janela->w() - 100, janela->vistaGL1->y() + janela->vistaGL1->h() + 2);
    janela->vistaGL2->resize(janela->groupVisoes->x() + 1, janela->vistaGL1->y() + janela->vistaGL1->h() + 23, janela->groupVisoes->w() - 2, janela->groupVisoes->w() - 2);

    janela->rbVistaGL3->position(janela->groupVisoes->x(), janela->vistaGL2->y() + janela->vistaGL2->h() + 5);
    janela->btZoomVistaGL3->resize(janela->groupVisoes->x() + 15, janela->vistaGL2->y() + janela->vistaGL2->h() + 2, janela->groupVisoes->w() - 115, 20);
    janela->choiceVistaGL3->position(janela->w() - 100, janela->vistaGL2->y() + janela->vistaGL2->h() + 2);
    janela->vistaGL3->resize(janela->groupVisoes->x() + 1, janela->vistaGL2->y() + janela->vistaGL2->h() + 23, janela->groupVisoes->w() - 2, janela->groupVisoes->w() - 2);
  } else {
    janela->panelGl->resize((janela->panelEsquerdo->visible() ? janela->panelEsquerdo->w() : 0), 25, janela->w() - (janela->panelEsquerdo->visible() ? janela->panelEsquerdo->w() : 0), janela->h() - 25);
  }
  janela->tileJanela->redraw();
  repintaTudo(1);
}

// *************************************************************************************
// Method.....: callbackTranslacao(Fl_Round_Button *obj, void *val)
// Function...: Apply values of translation in editboxes
// *************************************************************************************
void SceneEditor::callbackTranslacao(Fl_Round_Button *obj, void *val) {
  janela->valorX->range(janelaPref->cenarioMinX, janelaPref->cenarioMaxX);
  janela->valorY->range(janelaPref->cenarioMinY, janelaPref->cenarioMaxY);
  janela->valorZ->range(janelaPref->cenarioMinZ, janelaPref->cenarioMaxZ);

  janela->valorX->step(1);
  janela->valorY->step(1);
  janela->valorZ->step(1);

  janela->valorX->value(translateX);
  janela->valorY->value(translateY);
  janela->valorZ->value(translateZ);
}

// *************************************************************************************
// Method.....: callbackEscala(Fl_Round_Button *obj, void *val)
// Function...: Apply values of scale in editboxes
// *************************************************************************************
void SceneEditor::callbackEscala(Fl_Round_Button *obj, void *val) {
  janela->valorX->range(.1, 1000);
  janela->valorY->range(.1, 1000);
  janela->valorZ->range(.1, 1000);

  janela->valorX->step(.1);
  janela->valorY->step(.1);
  janela->valorZ->step(.1);

  janela->valorX->value(scaleX);
  janela->valorY->value(scaleY);
  janela->valorZ->value(scaleZ);
}

// *************************************************************************************
// Method.....: callbackRotacao(Fl_Round_Button *obj, void *val)
// Function...: Apply values of rotation in editboxes
// *************************************************************************************
void SceneEditor::callbackRotacao(Fl_Round_Button *obj, void *val) {
  janela->valorX->range(-359.9, 359.9);
  janela->valorY->range(-359.9, 359.9);
  janela->valorZ->range(-359.9, 359.9);

  janela->valorX->step(.1);
  janela->valorY->step(.1);
  janela->valorZ->step(.1);

  janela->valorX->value(rotateX);
  janela->valorY->value(rotateY);
  janela->valorZ->value(rotateZ);
}

// *************************************************************************************
// Method.....: listObjetosOnClick(Fl_Hold_Browser *obj, void *val)
// Function...: Choose an object in the list
// *************************************************************************************
void SceneEditor::listObjetosOnClick(Fl_Hold_Browser *obj, void *val) {
  if(!janela->listObjetos->value()) // Se clicou fora da area
    janela->listObjetos->select(janela->listObjetos->size());
  if(janela->rbObjeto->value()) callbackObjeto(NULL, NULL);
  else callbackObservador(NULL, NULL);
}

// *************************************************************************************
// Method.....: callbackObjeto(Fl_Round_Button *obj, void *val)
// Function...: Enable updates in object properties
// *************************************************************************************
void SceneEditor::callbackObjeto(Fl_Round_Button *obj, void *val) {
  static SmVR_Matrix m;
  SmVR_CGeometricObject *obj_aux;

  if(janela->listObjetos->size()) {
    if(!janela->listObjetos->value() || janela->listObjetos->value() > janela->listObjetos->size())
      janela->listObjetos->value(1);
    obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
    setaEnvelope(obj_aux);
    obj_aux->GetLocalTransformationMatrix(m);
    translateX = m[3][0];
    translateY = m[3][1];
    translateZ = m[3][2];
    scaleX = (m[0][0] >= 0.0001 ? m[0][0] : 1);
    scaleY = (m[1][1] >= 0.0001 ? m[1][1] : 1);
    scaleZ = (m[2][2] >= 0.0001 ? m[2][2] : 1);
  } else {
    setaEnvelope(NULL);
    translateX = translateY = translateZ = 0;
    scaleX = scaleY = scaleZ = 1;
  }
  rotateX = rotateY = rotateZ = 0;
  if(janela->btTranslacao->value()) {
    janela->valorX->value(translateX);
    janela->valorY->value(translateY);
    janela->valorZ->value(translateZ);
  } else if(janela->btEscala->value()) {
    janela->valorX->value(scaleX);
    janela->valorY->value(scaleY);
    janela->valorZ->value(scaleZ);
  } else if(janela->btRotacao->value()) {
    janela->valorX->value(rotateX);
    janela->valorY->value(rotateY);
    janela->valorZ->value(rotateZ);
  }
  repintaTudo();
}

// *************************************************************************************
// Method.....: callbackObservador(Fl_Round_Button *obj, void *val)
// Function...: Enable updates in Observer properties
// *************************************************************************************
void SceneEditor::callbackObservador(Fl_Round_Button *obj, void *val) {
  static int foi = 0; // Nao me pergunte porque !!!
  if(!foi) {          // Mas se nao chamar a callback na 1ra vez nao funciona
    foi = 1;          // E se chama-la no final do main da pau !!!
    SceneEditor::callbackVisao(janela->rbFrontal, NULL); // Ajusta a visao
  }
  setaEnvelope(user);
  user->GetPointInOCS(Zero, pt_user, RootObject);
  alvo->GetPointInOCS(Zero, pt_alvo, RootObject);
  if(janela->rbObservador->value()) {
    translateX = pt_user->X;
    translateY = pt_user->Y;
    translateZ = pt_user->Z;
  } else {
    translateX = pt_alvo->X;
    translateY = pt_alvo->Y;
    translateZ = pt_alvo->Z;
  }
  scaleX = scaleY = scaleZ = 1;
  rotateX = rotateY = rotateZ = 0;
  if(janela->btTranslacao->value()) {
    janela->valorX->value(translateX);
    janela->valorY->value(translateY);
    janela->valorZ->value(translateZ);
  } else if(janela->btEscala->value()) {
    janela->valorX->value(scaleX);
    janela->valorY->value(scaleY);
    janela->valorZ->value(scaleZ);
  } else if(janela->btRotacao->value()) {
    janela->valorX->value(rotateX);
    janela->valorY->value(rotateY);
    janela->valorZ->value(rotateZ);
  }
  repintaTudo(1);
}

// *************************************************************************************
// Method.....: callbackMudaCampos2(Fl_Value_Input *obj, void *val)
// Function...: Update coordinate fields directly (editing fields)
// *************************************************************************************
void SceneEditor::callbackMudaCampos2(Fl_Value_Input *obj, void *val) {
  SmVR_CGeometricObject *obj_aux;
  if(janela->listObjetos->value() == 0) {
    if(janela->listObjetos->size()) {
      janela->listObjetos->select(janela->listObjetos->size());
      callbackObjeto(NULL, NULL);
    }
    else return;
  }
  if(janela->rbObjeto->value()) {
    obj_aux = (SmVR_CGeometricObject*)janela->listObjetos->data(janela->listObjetos->value());
    if(janela->btTranslacao->value()) {
      if(obj == janela->valorX) {
        Acao::adicionaAcao(obj_aux, TRANS_X, janela->valorX->value() - translateX);
        obj_aux->TranslateBy(janela->valorX->value() - translateX, 0, 0);
        translateX = janela->valorX->value();
      } else if(obj == janela->valorY) {
        Acao::adicionaAcao(obj_aux, TRANS_Y, janela->valorY->value() - translateY);
        obj_aux->TranslateBy(0, janela->valorY->value() - translateY, 0);
        translateY = janela->valorY->value();
      } else if(obj == janela->valorZ) {
        Acao::adicionaAcao(obj_aux, TRANS_Z, janela->valorZ->value() - translateZ);
        obj_aux->TranslateBy(0, 0, janela->valorZ->value() - translateZ);
        translateZ = janela->valorZ->value();
      }
    } else if(janela->btEscala->value()) {
      if(obj == janela->valorX) {
        if(janela->valorX->value() > 0.0) {
          Acao::adicionaAcao(obj_aux, ESC_X, janela->valorX->value() - scaleX);
          obj_aux->ScaleBy(janela->valorX->value() / scaleX, 1, 1);
          scaleX = janela->valorX->value();
        }
      } else if(obj == janela->valorY) {
        if(janela->valorY->value() > 0.0) {
          Acao::adicionaAcao(obj_aux, ESC_Y, janela->valorY->value() - scaleY);
          obj_aux->ScaleBy(1, janela->valorY->value() / scaleY, 1);
          scaleY = janela->valorY->value();
        }
      } else if(obj == janela->valorZ) {
        if(janela->valorZ->value() > 0.0) {
          Acao::adicionaAcao(obj_aux, ESC_Z, janela->valorZ->value() - scaleZ);
          obj_aux->ScaleBy(1, 1, janela->valorZ->value() / scaleZ);
          scaleZ = janela->valorZ->value();
        }
      }
    } else if(janela->btRotacao->value()) {
      if(obj == janela->valorX) {
        if(janela->valorX->value()) {
          Acao::adicionaAcao(obj_aux, ROT_X, janela->valorX->value() - rotateX);
          obj_aux->RotateBy(janela->valorX->value() - rotateX, 1, 0, 0);
          rotateX = janela->valorX->value();
        }
      } else if(obj == janela->valorY) {
        if(janela->valorY->value()) {
          Acao::adicionaAcao(obj_aux, ROT_Y, janela->valorY->value() - rotateY);
          obj_aux->RotateBy(janela->valorY->value() - rotateY, 0, 1, 0);
          rotateY = janela->valorY->value();
        }
      } else if(obj == janela->valorZ) {
        if(janela->valorZ->value()) {
          Acao::adicionaAcao(obj_aux, ROT_Z, janela->valorZ->value() - rotateZ);
          obj_aux->RotateBy(janela->valorZ->value() - rotateZ, 0, 0, 1);
          rotateZ = janela->valorZ->value();
        }
      }
    }
    repintaTudo();
  } else { // Manipula o observador ou alvo
    obj_aux = (janela->rbObservador->value() ? user : alvo);
    if(janela->btTranslacao->value()) {
      if(obj == janela->valorX) {
        obj_aux->TranslateByOnOBJCS(janela->valorX->value() - translateX, 0, 0, alvo);
        translateX = janela->valorX->value();
      } else if(obj == janela->valorY) {
        obj_aux->TranslateByOnOBJCS(0, janela->valorY->value() - translateY, 0, alvo);
        translateY = janela->valorY->value();
      } else if(obj == janela->valorZ) {
        obj_aux->TranslateByOnOBJCS(0, 0, janela->valorZ->value() - translateZ, alvo);
        translateZ = janela->valorZ->value();
      }
    } else if(janela->btRotacao->value() && janela->rbObservador->value()) {
      if(obj == janela->valorX) {
        if(janela->valorX->value()) {
          obj_aux->RotateBy(janela->valorX->value() - rotateX, 1, 0, 0);
          rotateX = janela->valorX->value();
        }
      } else if(obj == janela->valorY) {
        if(janela->valorY->value()) {
          obj_aux->RotateBy(janela->valorY->value() - rotateY, 0, 1, 0);
          rotateY = janela->valorY->value();
        }
      } else if(obj == janela->valorZ) {
        if(janela->valorZ->value()) {
          obj_aux->RotateBy(janela->valorZ->value() - rotateZ, 0, 0, 1);
          rotateZ = janela->valorZ->value();
        }
      }
    }
    user->GetPointInOCS(Zero, pt_user, RootObject);
    alvo->GetPointInOCS(Zero, pt_alvo, RootObject);
    repintaTudo(1);
  }
}

// *************************************************************************************
// Method.....: callbackMudaCampos(Fl_Repeat_Button *obj, void *val)
// Function...: Update coordinate fields using repeat buttons
// *************************************************************************************
void SceneEditor::callbackMudaCampos(Fl_Repeat_Button *obj, void *val) {
  if(obj == janela->decXfast) {
    janela->valorX->value(janela->valorX->clamp(janela->valorX->value() - 4 * janela->valorX->step()));
    callbackMudaCampos2(janela->valorX, NULL);
  } else if(obj == janela->decX) {
    janela->valorX->value(janela->valorX->clamp(janela->valorX->value() - janela->valorX->step()));
    callbackMudaCampos2(janela->valorX, NULL);
  } else if(obj == janela->incX) {
    janela->valorX->value(janela->valorX->clamp(janela->valorX->value() + janela->valorX->step()));
    callbackMudaCampos2(janela->valorX, NULL);
  } else if(obj == janela->incXfast) {
    janela->valorX->value(janela->valorX->clamp(janela->valorX->value() + 4 * janela->valorX->step()));
    callbackMudaCampos2(janela->valorX, NULL);
  } else if(obj == janela->decYfast) {
    janela->valorY->value(janela->valorY->clamp(janela->valorY->value() - 4 * janela->valorY->step()));
    callbackMudaCampos2(janela->valorY, NULL);
  } else if(obj == janela->decY) {
    janela->valorY->value(janela->valorY->clamp(janela->valorY->value() - janela->valorY->step()));
    callbackMudaCampos2(janela->valorY, NULL);
  } else if(obj == janela->incY) {
    janela->valorY->value(janela->valorY->clamp(janela->valorY->value() + janela->valorY->step()));
    callbackMudaCampos2(janela->valorY, NULL);
  } else if(obj == janela->incYfast) {
    janela->valorY->value(janela->valorY->clamp(janela->valorY->value() + 4 * janela->valorY->step()));
    callbackMudaCampos2(janela->valorY, NULL);
  } else if(obj == janela->decZfast) {
    janela->valorZ->value(janela->valorZ->clamp(janela->valorZ->value() - 4 * janela->valorZ->step()));
    callbackMudaCampos2(janela->valorZ, NULL);
  } else if(obj == janela->decZ) {
    janela->valorZ->value(janela->valorZ->clamp(janela->valorZ->value() - janela->valorZ->step()));
    callbackMudaCampos2(janela->valorZ, NULL);
  } else if(obj == janela->incZ) {
    janela->valorZ->value(janela->valorZ->clamp(janela->valorZ->value() + janela->valorZ->step()));
    callbackMudaCampos2(janela->valorZ, NULL);
  } else if(obj == janela->incZfast) {
    janela->valorZ->value(janela->valorZ->clamp(janela->valorZ->value() + 4 * janela->valorZ->step()));
    callbackMudaCampos2(janela->valorZ, NULL);
  }
}

// *************************************************************************************
// Method.....: callbackZoom(Fl_Roller *obj, void *val)
// Function...: Update the view angle, using a roller button
// *************************************************************************************
void SceneEditor::callbackZoom(Fl_Roller *obj, void *val) {
  if(obj == janela->btZoom) {
    janela->panelGl->anguloVisao = (int)obj->value();
  } else { // Vistas
    ((Gl_Group*)val)->anguloVisao = (int)obj->value();
  }
  repintaTudo(1);
}

// *************************************************************************************
// Method.....: callbackVisao(Fl_Widget *obj, void *val)
// Function...: Enable a principal window view
// *************************************************************************************
void SceneEditor::callbackVisao(Fl_Widget *obj, void *val) {
  RootObject->AddChild(alvo); // Desliga a hierarquia
  user->SetLocalTransformationMatrixToIdentity();
  alvo->SetLocalTransformationMatrixToIdentity();
  alvo->TranslateToOnOBJCS(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                           janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                           janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, RootObject);
  if(janela->rbPerspectiva->value()) {
    user->TranslateToOnOBJCS(janelaPref->cenarioMinX, janelaPref->cenarioMaxY,
                             janelaPref->cenarioMaxZ, RootObject);
  } else if(janela->rbFrontal->value()) {
    user->TranslateToOnOBJCS(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                             janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                             janelaPref->cenarioMaxZ, RootObject);
  } else if(janela->rbTraseira->value()) {
    user->TranslateToOnOBJCS(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                             janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                             janelaPref->cenarioMinZ, RootObject);
  } else if(janela->rbDireita->value()) {
    user->TranslateToOnOBJCS(janelaPref->cenarioMaxX,
                             janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                             janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, RootObject);
  } else if(janela->rbEsquerda->value()) {
    user->TranslateToOnOBJCS(janelaPref->cenarioMinX,
                             janelaPref->cenarioMinY + (janelaPref->cenarioMaxY - janelaPref->cenarioMinY) / 2.0,
                             janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, RootObject);
  } else if(janela->rbSuperior->value()) {
    user->TranslateToOnOBJCS(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                             janelaPref->cenarioMaxY,
                             janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, RootObject);
  } else if(janela->rbInferior->value()) {
    user->TranslateToOnOBJCS(janelaPref->cenarioMinX + (janelaPref->cenarioMaxX - janelaPref->cenarioMinX) / 2.0,
                             janelaPref->cenarioMinY,
                             janelaPref->cenarioMinZ + (janelaPref->cenarioMaxZ - janelaPref->cenarioMinZ) / 2.0, RootObject);
  }
  user->AddChild(alvo); // Volta a hierarquia
  user->GetPointInOCS(Zero, pt_user, RootObject);
  alvo->GetPointInOCS(Zero, pt_alvo, RootObject);
  repintaTudo(1);
}

// *************************************************************************************
// Method.....: callbackVistas(Fl_Round_Button *obj, void *val)
// Function...: Enable a view (right side)
// *************************************************************************************
void SceneEditor::callbackVistas(Fl_Round_Button *obj, void *val) {
  if(janela->rbVistaGL1->value()) janela->vistaGL1->show();
  else janela->vistaGL1->hide();
  if(janela->rbVistaGL2->value()) janela->vistaGL2->show();
  else janela->vistaGL2->hide();
  if(janela->rbVistaGL3->value()) janela->vistaGL3->show();
  else janela->vistaGL3->hide();
  repintaTudo(1);
}

// *************************************************************************************
// Method.....: callbackChoiceVisao(Fl_Choice *obj, void *val)
// Function...: Choose a view using a "combobox"
// *************************************************************************************
void SceneEditor::callbackChoiceVisao(Fl_Choice *obj, void *val) {
  if(obj == janela->choiceVistaGL1) {
    janela->vistaGL1->SetTipo((tipo_vista)(janela->choiceVistaGL1->value() + 1));
  } else if(obj == janela->choiceVistaGL2) {
    janela->vistaGL2->SetTipo((tipo_vista)(janela->choiceVistaGL2->value() + 1));
  } else if(obj == janela->choiceVistaGL3) {
    janela->vistaGL3->SetTipo((tipo_vista)(janela->choiceVistaGL3->value() + 1));
  }
}

// *************************************************************************************
// Method.....: inicializaJanela()
// Function...: Initialize windows
// *************************************************************************************
void SceneEditor::inicializaJanela() {
  show();
  SceneEditor::callbackTranslacao(NULL, NULL); // Ajusta os botoes para translacao
  repintaTudo(1); // Seta o titulo inicial da janela
}

// *************************************************************************************
// Function...: int main(int argc, char **argv)
//              Show the application
// *************************************************************************************
int main(int argc, char **argv) {
  janelaSplash = new Splash();    // Carrega e exibe
  janelaSplash->show(argc, argv); // a splash screen
  // Inicializa a SmallVR
  RootObject = SmVR_Init(NULL); // Inicializa a Small VR
  user = new SmVR_CGeometricObject("user");
  user->TranslateBy(pt_user->X, pt_user->Y, pt_user->Z);
  alvo = new SmVR_CGeometricObject("alvo");
  alvo->TranslateBy(pt_alvo->X, pt_alvo->Y, pt_alvo->Z);
  user->AddChild(alvo);
  RootObject->AddChild(user);
  // Cria as outras janelas
  janela = new SceneEditor();
  janelaPref = new Preferencias();
  janelaAbout = new About();
  janelaHelp = new Fl_Help_Dialog();
  janelaHelp->resize((Fl::w() - 650) / 2, 25, 650, Fl::h() - 30);
  janelaHelp->load("help.html");
  return Fl::run();
}
