//
// Copyright 2005 by Mauro Charo, Rafael Rieder.
// Porto Alegre, Brazil.
//
// This library is free software; you can redistribute it and/or modify it under the
// terms of the GNU Library General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the GNU Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public License along with
// this library; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA 02111-1307 USA.
//
// Please report bugs & problems to maurocharao@yahoo.com.br or rafaelrieder@gmail.com
//
#ifndef _Scene_Editor_H_
#define _Scene_Editor_H_

#include "Gl_Group.h"
#include "Preferencias.h"
#include "SmVR.h"
#include <FL/Fl_Box.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Hold_Browser.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Roller.H>
#include <FL/Fl_Repeat_Button.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Tile.H>
#include <FL/Fl_Value_Input.H>


class SceneEditor : public Fl_Window {
public:
  SceneEditor();
  void inicializaJanela();
  void adicionaObjeto(SmVR_CGeometricObject *obj_aux);
  void selecionaObjeto(char *nome);
  void apagaObjetoAtual();
  void salvaCenario();

  static void popula_lista(SmVR_CGeometricObject *ob);
  static void callbackMenus(Fl_Menu_ *menu, void *val);
  static void callbackJanela(Fl_Window *obj, void *val);
  static void callbackTile(Fl_Widget *obj, void *val);
  static void callbackMudaCampos(Fl_Repeat_Button *obj, void *val);
  static void callbackMudaCampos2(Fl_Value_Input *obj, void *val);
  static void callbackZoom(Fl_Roller *obj, void *val);
  static void callbackTranslacao(Fl_Round_Button *obj, void *val);
  static void callbackEscala(Fl_Round_Button *obj, void *val);
  static void callbackRotacao(Fl_Round_Button *obj, void *val);
  static void listObjetosOnClick(Fl_Hold_Browser *obj, void *val);
  static void callbackObjeto(Fl_Round_Button *obj, void *val);
  static void callbackObservador(Fl_Round_Button *obj, void *val);
  static void callbackVisao(Fl_Widget *obj, void *val);
  static void callbackVistas(Fl_Round_Button *obj, void *val);
  static void callbackChoiceVisao(Fl_Choice *obj, void *val);

  Fl_Hold_Browser *listObjetos;
  Fl_Value_Input *valorX, *valorY, *valorZ;
  Gl_Group *panelGl;
  Gl_Group *vistaGL1, *vistaGL2, *vistaGL3;
  Fl_Round_Button *btRotacao;

protected:
  static Fl_Menu_Item menuPrincipal_itens[];
  static Fl_Menu_Item popup_itens[];
  Fl_Menu_Bar *MenuPrincipal;
  Fl_Menu_Button *popup;
  Fl_Box *boxTile;
  Fl_Tile *tileJanela;
  Fl_Group *panelEsquerdo;
  Fl_Roller *btZoom;
  Fl_Group *groupPropriedades;
  Fl_Group *groupTipo;
  Fl_Round_Button *rbObjeto, *rbObservador, *rbAlvo;
  Fl_Round_Button *btTranslacao;
  Fl_Round_Button *btEscala;
  Fl_Repeat_Button *decXfast, *decX, *incX, *incXfast;
  Fl_Repeat_Button *decYfast, *decY, *incY, *incYfast;
  Fl_Repeat_Button *decZfast, *decZ, *incZ, *incZfast;
  Fl_Group *groupVisao;
  Fl_Round_Button *rbPerspectiva;
  Fl_Round_Button *rbSuperior, *rbInferior;
  Fl_Round_Button *rbFrontal, *rbTraseira;
  Fl_Round_Button *rbDireita;
  Fl_Round_Button *rbEsquerda;
  Fl_Group *groupVisoes;
  Fl_Round_Button *rbVistaGL1, *rbVistaGL2, *rbVistaGL3;
  Fl_Roller *btZoomVistaGL1, *btZoomVistaGL2, *btZoomVistaGL3;
  Fl_Choice *choiceVistaGL1, *choiceVistaGL2, *choiceVistaGL3;
};

extern SmVR_CGeometricObject *RootObject;
extern SmVR_CPoint *pt_user, *pt_alvo;

extern SceneEditor *janela;
extern Preferencias *janelaPref;
#endif
