// ************************************************
//  SmVR_CMyEnvelope.cpp
//
//  Authors: Marcio Serolli Pinho
//           Regis Augusto Poli Kopper
//           Mauro Cesar Charao dos Santos
//
//  Emails: pinho@inf.pucrs.br
//          regis@webstuff.com.br
//          maurocharao@yahoo.com
//
//  Porto Alegre, Brazil, 2003-2006.
// ************************************************
#include "SmVR_CMyEnvelope.h"

extern SmVR_CGeometricObject *RootObject;
extern int HaInterseccao(float V0[3], float V1[3], float V2[3],
                         float U0[3], float U1[3], float U2[3]);

// *************************************************************************************
// Set boundaries of the envelope
// *************************************************************************************
void SmVR_CMyEnvelope::SetBoundaries(SmVR_CPoint *min, SmVR_CPoint *max) {
  vertices[0][0] = max->X; vertices[0][1] = max->Y; vertices[0][2] = max->Z;
  vertices[1][0] = min->X; vertices[1][1] = max->Y; vertices[1][2] = max->Z;
  vertices[2][0] = min->X; vertices[2][1] = min->Y; vertices[2][2] = max->Z;
  vertices[3][0] = max->X; vertices[3][1] = min->Y; vertices[3][2] = max->Z;
  vertices[4][0] = max->X; vertices[4][1] = max->Y; vertices[4][2] = min->Z;
  vertices[5][0] = min->X; vertices[5][1] = max->Y; vertices[5][2] = min->Z;
  vertices[6][0] = min->X; vertices[6][1] = min->Y; vertices[6][2] = min->Z;
  vertices[7][0] = max->X; vertices[7][1] = min->Y; vertices[7][2] = min->Z;
}

// *************************************************************************************
// Eight cube points // size = 1 // Centered at the origin
// *************************************************************************************
float SmVR_CMyEnvelope::vertices[8][3] = {
  {  .5,  .5,  .5 }, //0 - right front top
  { -.5,  .5,  .5 }, //1 - left  front top
  { -.5, -.5,  .5 }, //2 - left  front bottom
  {  .5, -.5,  .5 }, //3 - right front bottom
  {  .5,  .5, -.5 }, //4 - right back  top
  { -.5,  .5, -.5 }, //5 - left  back  top
  { -.5, -.5, -.5 }, //6 - left  back  bottom
  {  .5, -.5, -.5 }  //7 - right back  bottom
};

// *************************************************************************************
// Twelve cube triangular faces
// *************************************************************************************
int SmVR_CMyEnvelope::faces[12][3] = {
  { 0, 1, 2 }, //  0 - front
  { 2, 3, 0 }, //  1 - front
  { 1, 2, 6 }, //  2 - left
  { 6, 5, 1 }, //  3 - left
  { 4, 7, 6 }, //  4 - back
  { 6, 5, 4 }, //  5 - back
  { 4, 7, 3 }, //  6 - right
  { 3, 0, 4 }, //  7 - right
  { 4, 0, 1 }, //  8 - top
  { 1, 5, 4 }, //  9 - top
  { 2, 3, 7 }, // 10 - bottom
  { 7, 6, 2 }  // 11 - bottom
};

// *************************************************************************************
// Twelve cube triangular normals
// *************************************************************************************
float SmVR_CMyEnvelope::normais[12][3] = {
  {  0,  0,  1 },
  {  0,  0,  1 },
  { -1,  0,  0 },
  { -1,  0,  0 },
  {  0,  0, -1 },
  {  0,  0, -1 },
  {  1,  0,  0 },
  {  1,  0,  0 },
  {  0,  1,  0 },
  {  0,  1,  0 },
  {  0, -1,  0 },
  {  0, -1,  0 }
};


// *************************************************************************************
// Constructor
// *************************************************************************************
SmVR_CMyEnvelope::SmVR_CMyEnvelope(char *name) : SmVR_CGeometricObject(name, desenha) {}

GLfloat SmVR_CMyEnvelope::color[4] = { 1.0, 0.0, 0.0, 1 };  // Default Color = red


// *************************************************************************************
// int SmVR_CMyEnvelope::desenha(void *p)
// Function...: Draw a bounding box around the object
// *************************************************************************************
int SmVR_CMyEnvelope::desenha(void *p) {
  GLfloat tempColor[4];

  glGetFloatv(GL_CURRENT_COLOR, tempColor);	
  glColor4fv(p ? (GLfloat*)p : color);
  glBegin(GL_LINE_STRIP);
  for(int x = 0; x < 12; x++) {
    glNormal3fv(normais[x]);
    glVertex3fv(vertices[faces[x][0]]);
    glVertex3fv(vertices[faces[x][1]]);
    glVertex3fv(vertices[faces[x][2]]);
  }
  glEnd();
  glColor4fv(tempColor);

  return 1;
}

// *************************************************************************************
// int SmVR_CMyEnvelope::colided(float u0[3], float u1[3], float u2[3])
// Function...: Collision test between the object and the triangle
//              Each parameter is a triangle vertice
// *************************************************************************************
int SmVR_CMyEnvelope::colided(float u0[3], float u1[3], float u2[3]) {
  SmVR_CPoint *pt_aux = new SmVR_CPoint(0, 0, 0);
  SmVR_CPoint pt_in(0, 0, 0);
  float v0[3], v1[3], v2[3];

  for(int x = 0; x < 12; x++) {
    pt_in.X = vertices[faces[x][0]][0];
    pt_in.Y = vertices[faces[x][0]][1];
    pt_in.Z = vertices[faces[x][0]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v0[0] = pt_aux->X;
    v0[1] = pt_aux->Y;
    v0[2] = pt_aux->Z;

    pt_in.X = vertices[faces[x][1]][0];
    pt_in.Y = vertices[faces[x][1]][1];
    pt_in.Z = vertices[faces[x][1]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v1[0] = pt_aux->X;
    v1[1] = pt_aux->Y;
    v1[2] = pt_aux->Z;

    pt_in.X = vertices[faces[x][2]][0];
    pt_in.Y = vertices[faces[x][2]][1];
    pt_in.Z = vertices[faces[x][2]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v2[0] = pt_aux->X;
    v2[1] = pt_aux->Y;
    v2[2] = pt_aux->Z;

    if(HaInterseccao(v0, v1, v2, u0, u1, u2))
      return 1;
  }
  return 0;
}

// *************************************************************************************
// int SmVR_CMyEnvelope::colided(SmVR_CMyEnvelope *obj)
// Function...: Collision test between objects
// *************************************************************************************
int SmVR_CMyEnvelope::colided(SmVR_CMyEnvelope *obj) {
  SmVR_CPoint *pt_aux = new SmVR_CPoint(0, 0, 0);
  SmVR_CPoint pt_in(0, 0, 0);
  float v0[3], v1[3], v2[3];

  for(int x = 0; x < 12; x++) {
    pt_in.X = vertices[faces[x][0]][0];
    pt_in.Y = vertices[faces[x][0]][1];
    pt_in.Z = vertices[faces[x][0]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v0[0] = pt_aux->X;
    v0[1] = pt_aux->Y;
    v0[2] = pt_aux->Z;

    pt_in.X = vertices[faces[x][1]][0];
    pt_in.Y = vertices[faces[x][1]][1];
    pt_in.Z = vertices[faces[x][1]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v1[0] = pt_aux->X;
    v1[1] = pt_aux->Y;
    v1[2] = pt_aux->Z;

    pt_in.X = vertices[faces[x][2]][0];
    pt_in.Y = vertices[faces[x][2]][1];
    pt_in.Z = vertices[faces[x][2]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v2[0] = pt_aux->X;
    v2[1] = pt_aux->Y;
    v2[2] = pt_aux->Z;

    if(obj->colided(v0, v1, v2))
      return 1;
  }
  return 0;
}
