// *******************************************************
//
// OBJ_Texture_LIB.cpp
//
//      Authors: Marcio Serolli Pinho
//               Felipe Bacim de Araujo e Silva
//      Emails: pinho@inf.pucrs.br
//              blah37@gmail.com
//
//   This library reads a subset set of Alias Wavefront OBJ
//       files.
//   It just parse vertices and faces with 3 vertices.
//
//       This code was based on the file OBJ2ASC converter found
//   at ftp://avalon.viewpoint.com/pub/utils/converters/
//
//
// *******************************************************
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>

#ifdef WIN32  // only for Visual C
#include <windows.h>
#endif

#include <gl/gl.h>
#include <gl/glut.h>

#include "Obj_Texture_lib.h"

#include <list>
using namespace std;

//
// Type definitions for LISTs
// This is necessary because some versions of G++
// have problems with the default list allocators
//
typedef list<int> LIST_INT;
typedef list<char *> LIST_CHAR_PTR;
typedef list<float> LIST_FLOAT;
/*
#if ((defined(__MINGW32_VERSION) && (__MINGW32_MAJOR_VERSION >= 3 && __MINGW32_MINOR_VERSION >= 7)) || !defined(__GNUC__))
typedef list<int> LIST_INT;
typedef list<char *> LIST_CHAR_PTR;
typedef list<float> LIST_FLOAT;

#else
typedef list<int, __malloc_alloc_template<0> > LIST_INT;
typedef list<char *, __malloc_alloc_template<0> > LIST_CHAR_PTR;
typedef list<float, __malloc_alloc_template<0> > LIST_FLOAT;
#endif
*/
FILE *infile;

char buff[2000];

// *******************************************************
//
//
// *******************************************************

TVertices CalculateNormal (TVertices *v1,
                                                  TVertices *v2,
                                                  TVertices *v3)
{
       TVertices va, vb, n;
       GLfloat modulo;


 // cálculo do vetor Normal N=(v2-v1)x(v3-v1)
 va.x = v2->x - v1->x;  // vetor a va = (v2-v1)
 va.y = v2->y - v1->y;
 va.z = v2->z - v1->z;

 vb.x = v3->x - v1->x;  // vetor b vb = (v3-v1)
 vb.y = v3->y - v1->y;
 vb.z = v3->z - v1->z;

 n.x = (va.y * vb.z) - (va.z * vb.y);
 n.y = (va.z * vb.x) - (va.x * vb.z);
 n.z = (va.x * vb.y) - (va.y * vb.x);

 modulo = sqrt (n.x*n.x + n.y*n.y + n.z*n.z);

 n.x = n.x / modulo;
 n.y = n.y / modulo;
 n.z = n.z / modulo;
 return n;
}



// *******************************************************
//  ReadALine
//    this function gets a line and clean it by removing
//    leading white space, and other unusefull
//    The line is in 'char buff[2000]' vector.
//
// *******************************************************

void ReadALine(void)
{
       int posi;

       fgets(buff,1999,infile);

       // Kill nulls, tabs anything strange
       posi=0;
       while ((buff[posi] != 0x0d) && (buff[posi] != 0x0a))
               {
                 if (buff[posi] < ' ')   // anything instead of
                       buff[posi]=' ';       // printable things ?
                 posi++;
               }
       posi=0;
       //Kill all leading white space
       while ((buff[0] == ' ') && (buff[0] != 0x0d) && (buff[0] != 0x0a))
               memmove(&buff[0],&buff[1],199);
       // Kill any space padding
       while ((buff[posi] != 0x0d) && (buff[posi] != 0x0a))
               {
               // space padding
               if ((buff[posi]==' ') && (buff[posi+1]==' '))
                       memmove(&buff[posi],&buff[posi+1],1998-posi);
               else
                       posi++;
               }

}



// *******************************************************
//void SearchForMaterial(char *InputLine, int &nMat,
//                                              LIST_INT &facesList,
//                                              LIST_CHAR_PTR &mtllibList,
//                                              LIST_CHAR_PTR &usemtlList)
//              check to see if the input is a material line
//
// *******************************************************

void SearchForMaterial(char *InputLine, int &nMat,
                                          LIST_INT &facesList,
                                          LIST_CHAR_PTR &mtllibList,
                                          LIST_CHAR_PTR &usemtlList)
{
//       Check to see if there is a material
       char part1[200], part2[200];

       sscanf(buff,"%s%s",part1, part2);
       if (!strcmp(part1,"usemtl"))
       {
               char *aux = (char*)calloc((strlen(part2) + 1), sizeof(char));
               strcat(aux,part2);
               usemtlList.push_back(aux);
               facesList.push_back(-4);
               part1[0] = part2[0] = '\0';
               nMat++;
       }
       if (!strcmp(part1,"mtllib"))
       {
               char *aux = (char*)calloc((strlen(part2) + 1), sizeof(char));
               strcat(aux,part2);
               mtllibList.push_back(aux);
               facesList.push_back(-5);
               part1[0] = part2[0] = '\0';
       }

}
// *******************************************************
// int SeparateObjectParts(TObject *o,LIST_INT &facesList,
//                                              LIST_FLOAT &verticesList,
//                                              LIST_FLOAT &textureVerticesList,
//                                              LIST_FLOAT &normalVerticesList,
//                                              LIST_CHAR_PTR &mtllibList,
//                                              LIST_CHAR_PTR &usemtlList)
//              This function reads an Obj File an creates 2 new
//      files. On with the faces and other with the vertices.
//
// *******************************************************
int SeparateObjectParts(TObject *o,LIST_INT &facesList,
                                               LIST_FLOAT &verticesList,
                                               LIST_FLOAT &textureVerticesList,
                                               LIST_FLOAT &normalVerticesList,
                                               LIST_CHAR_PTR &mtllibList,
                                               LIST_CHAR_PTR &usemtlList)
{
       char type[199];
       float n[7];

       char tmpbuf[120];
       char *tmpptr;
       char *buffptr;
       int arrcount,
               nv=0, nf=0, nvnormal=0, nvtexture=0, nMat=0,
               firstface=1;

       int facenum=0;

       //printf("Separating Vertices, Faces, Normals and Textures...\n");

       // Creates temporary names for vertices and faces files
       infile=fopen(o->Name,"rb");

       if (infile == NULL)
       {
               printf("No INPUT file!\n");
               return 0;
       }

       while (!feof(infile))
       {
               ReadALine();  /* read control line */
               //printf("%s\n", buff);

               // Check to see if there is a
               // material specification in the current line
               SearchForMaterial(buff, nMat,facesList,mtllibList,usemtlList);

               type[0]=0;
               sscanf(buff, "%s %f %f %f", &type, &n[0], &n[1], &n[2]);

               // Read the faces
               if (toupper(type[0])=='F')   /* we have a face */
               {
                       int coord, text, norm;

                       facenum++;
                       if(!(facenum % 1000))
                               //printf("Reading (line: %d)...\n", facenum);
                               printf(".");

                       buffptr=strchr(buff,' ');  // skip over the "f " or "fo "
                       arrcount=0;
                       tmpptr=tmpbuf;

                       char v[3][100];  // nnn/nnn/nnn   nnn/nnn/nnn  nnn/nnn/nnn

                       sscanf (buffptr, "%s %s %s", v[0], v[1], v[2]);

                       int cont;
                       for (int i=0; i<3;i++)
                       {
                               cont=0;
                               while (1)  // count how many '/' we have in v[i]
                               {
                                       buffptr = strchr(v[i],'/');
                                       if (buffptr)
                                          *buffptr = ' ';
                                       else break;
                                       cont++;
                               }

                               coord=text=norm=-1;
                               switch (cont)
                               {
                                       case 0: // just the vertex
                                               coord = atoi (v[i]);
                                               break;
                                       case 1: // the vertex and the texture
                                               sscanf(v[i], "%d %d", &coord, &text);
                                               break;
                                       case 2: // the vertex, the texture and normal vector
                                               sscanf(v[i], "%d %d %d", &coord, &text, &norm);
                                               // Test is the face is like v//t
                                               if (norm < 0)    // This face has no texture info,
                                               {
                                                       norm = text; // but it has normals.
                                                       text = -1;
                                               }
                                               break;
                                       default: printf("Wrong number of '/' (%d) in face %d\n", cont, nf);
                               }
                               facesList.push_back(coord-1);
                               facesList.push_back(text-1);
                               facesList.push_back(norm-1);
                       }
                       nf++;
               }

               if (toupper(type[0])=='V') // can be VT, VN or just v
               {

                       if (toupper(type[1])=='N') // Is this a VERTEX NORMAL ? (vn)
                       {
                               nvnormal++;
                               normalVerticesList.push_back(n[0]);
                               normalVerticesList.push_back(n[1]);
                               normalVerticesList.push_back(n[2]);
                       }
                       else
                               if (toupper(type[1])=='T')        // Is this a VERTEX TEXTURE? (vt)
                               {
                                       nvtexture++;
                                       textureVerticesList.push_back(n[0]);
                                       textureVerticesList.push_back(n[1]);
                                       textureVerticesList.push_back(n[2]);
                               }
                               else
                                       if (toupper(type[0])=='V') // it is just a regular vertice !
                                       {
                                               nv++;
                                               verticesList.push_back(n[0]);
                                               verticesList.push_back(n[1]);
                                               verticesList.push_back(n[2]);
                                       }
               }
       }
       fclose(infile);
       printf("\n");

       o->nFaces = nf;
       o->nVertices = nv;
       o->nVertexNormals = nvnormal;
       o->nVertexTextures = nvtexture;
       o->nMaterialsUsed = nMat;
//      printf("Separation finished...\n");
       return 1;
}
// *******************************************************
// void LoadTexturesAndNormals(TObject *o,LIST_FLOAT &textureVerticesList,
//                                                      LIST_FLOAT &normalVerticesList)
//
// *******************************************************
void LoadTexturesAndNormals(TObject *o,LIST_FLOAT &textureVerticesList,
                                                       LIST_FLOAT &normalVerticesList)
{
       int i;

       //if (textureVerticesList.empty())
       //      printf("No Texture Vertices!!\n");

       LIST_FLOAT::iterator verticesIterator;
       verticesIterator = textureVerticesList.begin();

       for (i=0; i< o->nVertexTextures; i++)
       {
               o->VertexTextures[i].x = *verticesIterator;
               verticesIterator++;
               o->VertexTextures[i].y = *verticesIterator;
               verticesIterator++;
               o->VertexTextures[i].z = *verticesIterator;
               verticesIterator++;
       }

       verticesIterator = normalVerticesList.begin();

       for (i=0; i< o->nVertexNormals; i++)
       {
               o->VertexNormals[i].x = *verticesIterator;
               verticesIterator++;
               o->VertexNormals[i].y = *verticesIterator;
               verticesIterator++;
               o->VertexNormals[i].z = *verticesIterator;
               verticesIterator++;
       }
}
// *******************************************************
// void LoadFaces(TObject *o,LIST_INT &facesList,
//                         LIST_CHAR_PTR &mtllibList,
//                         LIST_CHAR_PTR &usemtlList)
//
//
//
// *******************************************************
void LoadFaces(TObject *o,LIST_INT &facesList,
                          LIST_CHAR_PTR &mtllibList,
                          LIST_CHAR_PTR &usemtlList)
{
       int i,itMax1=0,itMax2=0;
       LIST_INT::iterator facesListIterator;
       facesListIterator = facesList.begin();
       LIST_CHAR_PTR::iterator mtllibListIterator;
       mtllibListIterator = mtllibList.begin();
       LIST_CHAR_PTR::iterator usemtlListIterator;
       usemtlListIterator = usemtlList.begin();

       //printf("\nLoading the faces...\n");

       if (facesList.empty())
       {
               printf("No FACES list! Press ENTER to finish\n");
               getchar();
               return;
       }

       int text, norm,
               v1, nMat=0;

       char MaterialName[100], MaterialLibraryName[100];

       for (i=0; i<o->nMaterialsUsed; i++)
       {
               o->FacesWithMaterials[i].FirstFace = -1;
               o->FacesWithMaterials[i].LastFace = -1;
       }

       for (i=0; i<o->nFaces; i++)
       {
               for (int nv=0; nv <3; nv++)  // each face has only 3 vertices
               {
                       // vertex nv
                       if (*facesListIterator == -4) // is there a material especification ?
                       {
                               // Remove the '#' from the first position of the string and
                               // remove the '\n' from the last position of the string
                               strcpy(MaterialName,*usemtlListIterator);
                               itMax1++;
                               if(itMax1<usemtlList.size()) usemtlListIterator++;

                               if (o->FacesWithMaterials[nMat].FirstFace == -1)
                               {
                                       o->FacesWithMaterials[nMat].FirstFace = i;
                                       strcpy(o->FacesWithMaterials[nMat].MaterialName, MaterialName);
                               }
                               else
                                       if (o->FacesWithMaterials[nMat].LastFace == -1)
                                       {
                                               o->FacesWithMaterials[nMat].LastFace = i-1;
                                               nMat++;
                                               o->FacesWithMaterials[nMat].FirstFace = i;
                                               strcpy(o->FacesWithMaterials[nMat].MaterialName, MaterialName);
                                       }
                                       else printf ("\n>>>>Error reading object material....\n");
                               facesListIterator++;
                               nv = -1;  // restart the vertex loop
                       }
                       else if (*facesListIterator == -5) // is there a material library ?
                       {
                               // Remove the '!' from the first position of the string and
                               // remove the '\n' from the last position of the string
                               strcpy(MaterialLibraryName,*mtllibListIterator);
                               itMax2++;
                               if(itMax2<mtllibList.size()) mtllibListIterator++;
                               facesListIterator++;
                               nv = -1;  // restart the vertex loop
                       }
                       else// line with a regular vertice
                       {
                               v1 = *facesListIterator;
                               facesListIterator++;
                               text = *facesListIterator;
                               facesListIterator++;
                               norm = *facesListIterator;
                               facesListIterator++;
                               o->Faces[i][nv].v = v1;   // vertex index
                               o->Faces[i][nv].texture = text;// texture index
                               o->Faces[i][nv].normal = norm; // normal index
                       }
               }

               // calculate the face normal
               o->FaceNormals[i] = CalculateNormal (&o->Vertices[o->Faces[i][0].v],
                                                                                       &o->Vertices[o->Faces[i][1].v],
                                                                                       &o->Vertices[o->Faces[i][2].v]);
       }

       if (o->nMaterialsUsed)
       {
               // Set the last face for the last material
               o->FacesWithMaterials[nMat].LastFace = o->nFaces-1;
       }

       ReadMaterialFile(MaterialLibraryName, o);

       // Search for material numbers
       for (int m=0; m<o->nMaterialsUsed; m++)
       {
               o->FacesWithMaterials[m].Mat = SearchForMaterialNumber(o, o->FacesWithMaterials[m].MaterialName);
       }

       //printf("Done Reading Faces!\n");
}
// *******************************************************
// void LoadVertices(TObject *o,LIST_FLOAT &verticesList)
//
//
//
// *******************************************************
void LoadVertices(TObject *o,LIST_FLOAT &verticesList)
{
       int i;

       if (verticesList.empty())
       {
               printf("No VERTICES! Press ENTER to finish\n");
               getchar();
               return;
       }

       LIST_FLOAT::iterator verticesIterator;
       verticesIterator = verticesList.begin();

       for (i=0; i< o->nVertices; i++)
       {
               o->Vertices[i].x = *verticesIterator;
               verticesIterator++;
               o->Vertices[i].y = *verticesIterator;
               verticesIterator++;
               o->Vertices[i].z = *verticesIterator;
               verticesIterator++;

               if (i == 0)
               {
                       o->Min.x = o->Max.x =  o->Vertices[i].x;
                       o->Min.y = o->Max.y =  o->Vertices[i].y;
                       o->Min.z = o->Max.z =  o->Vertices[i].z;
               }

               if (o->Vertices[i].x > o->Max.x)
                       o->Max.x = o->Vertices[i].x;
               if (o->Vertices[i].x < o->Min.x)
                       o->Min.x = o->Vertices[i].x;
               if (o->Vertices[i].y > o->Max.y)
                       o->Max.y = o->Vertices[i].y;
               if (o->Vertices[i].y < o->Min.y)
                       o->Min.y = o->Vertices[i].y;
               if (o->Vertices[i].z > o->Max.z)
                       o->Max.z = o->Vertices[i].z;
               if (o->Vertices[i].z < o->Min.z)
                       o->Min.z = o->Vertices[i].z;
       }
}

// *******************************************************
// void LoadOBJ(TObject *o,LIST_FLOAT &verticesList,
//                       LIST_FLOAT &textureVerticesList,
//                       LIST_FLOAT &normalVerticesList,
//                       LIST_INT &facesList,
//                       LIST_CHAR_PTR &mtllibList,
//                       LIST_CHAR_PTR &usemtlList)
//              Carrega o objetocom as listas geradas anteriormente
// na rotina 'SeparateVerticesAndFaces' para a memória.
// *******************************************************
void LoadOBJ(TObject *o,LIST_FLOAT &verticesList,
                        LIST_FLOAT &textureVerticesList,
                        LIST_FLOAT &normalVerticesList,
                        LIST_INT &facesList,
                        LIST_CHAR_PTR &mtllibList,
                        LIST_CHAR_PTR &usemtlList)
{
       LoadVertices(o,verticesList);
       LoadFaces (o,facesList,mtllibList,usemtlList);
       LoadTexturesAndNormals(o,textureVerticesList,normalVerticesList);
}


// *******************************************************
// OBJReader
//              Gerencia a carga de um OBJ.
//              Passe o nome do arquivo em o->Name
// *******************************************************
int OBJ_Texture_Reader(TObject *o)
{
       printf("\nReading \"%s\"...", o->Name);

       LIST_INT facesList;
       LIST_FLOAT verticesList;
       LIST_FLOAT textureVerticesList;
       LIST_FLOAT normalVerticesList;
       LIST_CHAR_PTR mtllibList;
       LIST_CHAR_PTR usemtlList;

       // Separates diferent object parts into diferent files
       if (!SeparateObjectParts(o,facesList,verticesList,textureVerticesList,normalVerticesList,mtllibList,usemtlList))
               return 0;

       //printf("%d faces\n", o->nFaces);

       o->NormalOrientation = 0;

       o->Vertices     = NULL;
       o->Faces = NULL;
       o->FaceNormals  = NULL;
       o->VertexTextures       = NULL;
       o->VertexNormals        = NULL;
       o->FacesWithMaterials  = NULL;

// Allocates memory for Vertices, Faces, Normals and Texture Vertices
       o->Vertices           = (TVertices *) malloc (o->nVertices* sizeof(TVertices));
       o->Faces              =    (TFaces *) malloc (o->nFaces * sizeof(TFaces));
       if (o->nFaces != 0)
       {
               o->FaceNormals = (TVertices *) malloc (o->nFaces * sizeof(TVertices));
               if (o->FaceNormals == NULL)
               {
                       printf("Not enough memory to load the object!\n");
                       return 0;
               }
       }
       if (o->nVertexTextures != 0)
       {
               o->VertexTextures = (TVertices *) malloc (o->nVertexTextures * sizeof(TVertices));
               if (o->VertexTextures == NULL)
               {
                       printf("Not enough memory to load the object!\n");
                       return 0;
               }
       }
       if (o->nVertexNormals != 0)
       {
               o->VertexNormals = (TVertices *) malloc (o->nVertexNormals * sizeof(TVertices));
               if (o->VertexNormals == NULL)
               {
                       printf("Not enough memory to load the object!\n");
                       return 0;
               }
       }
       if (o->nMaterialsUsed != 0)
       {
               o->FacesWithMaterials = (TFacesMat *) malloc (o->nMaterialsUsed * sizeof (TFacesMat));
               if (o->FacesWithMaterials == NULL)
               {
                       printf("Not enough memory to load the object!\n");
                       return 0;
               }
       }
   /*
   printf("nVertices          = %x\n",o->nVertices);
   printf("Vertices           = %x\n",o->Vertices);
   printf("nFaces             = %x\n",o->nFaces);
   printf("Faces              = %x\n",o->Faces);
   printf("nFaces             = %x\n",o->nFaces);
   printf("FaceNormals        = %x\n",o->FaceNormals);
   printf("nVertexTextures    = %x\n",o->nVertexTextures);
   printf("VertexTextures     = %x\n",o->VertexTextures);
   printf("nVertexNormals     = %x\n",o->nVertexNormals);
   printf("VertexNormals      = %x\n",o->VertexNormals);
   printf("nMaterialsUsed     = %x\n",o->nMaterialsUsed);
   printf("FacesWithMaterials = %x\n",o->FacesWithMaterials);
   */
       //o->Materials = // the material list is allocated in fucntion "ReadMaterialFile" of "Material.cpp"
       //printf("\tLoading them into memory...\n");

// the material list is allocated in fucntion "ReadMaterialFile" of "Material.cpp"

       LoadOBJ(o,verticesList,textureVerticesList,normalVerticesList,facesList,mtllibList,usemtlList);

       verticesList.clear();
       textureVerticesList.clear();
       normalVerticesList.clear();
       facesList.clear();
       mtllibList.clear();
       usemtlList.clear();

       printf("Done!\n");
       return 1;
}

// *******************************************************
// OBJ_DisplayWithMaterials
//              Display the object using its material.
//              The method takes each material and draw the faces
//      that are associated to it
//
// *******************************************************
void OBJ_DisplayWithMaterials (TObject *o)
{
       int v, vt, tx, vn;
       int f, m;

// Save the current rendering status
       glPushAttrib(GL_ALL_ATTRIB_BITS);

       if (o->NormalOrientation)
               glFrontFace(GL_CW);
       else glFrontFace(GL_CCW);

       // Ativa o "Color Tracking"
       glEnable(GL_COLOR_MATERIAL);

       // Disable the use o texture
//      if (o->nVertexTextures)
//              glDisable ( GL_TEXTURE_2D );

       for (m=0; m < o->nMaterialsUsed; m++)
       {
//printf("          Face INICIAL %d\n", o->FacesWithMaterials[m].FirstFace);
//printf("          Face FINAL %d\n\n", o->FacesWithMaterials[m].LastFace);

               int Mat;
               Mat = o->FacesWithMaterials[m].Mat;
//printf("Material: (%d) - %s\t", Mat, o->FacesWithMaterials[m].MaterialName);
//printf("%s\n", o->Materials[Mat].Name);
//printf("Cor: %6.3f, %6.3f, %6.3f\n", o->Materials[Mat].dif.r, o->Materials[Mat].dif.g, o->Materials[Mat].dif.b);

// To do: It still needs to use the following parameters:
//
//d 1                   --- Material Alpha - Done, 04/mar/2005 by Regis Kopper
//illum 2               ---
//Ns 10.000000  --- Shininess
//
//

               if (Mat >= 0) // maybe there is a missing material
               {
                       // If there is a texture for this material...
                       GLfloat AmbienteColor[]   = {o->Materials[Mat].amb.r, o->Materials[Mat].amb.g, o->Materials[Mat].amb.b} ;
                       //GLfloat DifuseColor[]   = {o->Materials[Mat].dif.r-.8, o->Materials[Mat].dif.g-.8, o->Materials[Mat].dif.b-.8, 1.0f };
                       //GLfloat DifuseColor[]   = {o->Materials[Mat].dif.r, o->Materials[Mat].dif.g, o->Materials[Mat].dif.b, 1.0f };
                       GLfloat DifuseColor[]   = {o->Materials[Mat].dif.r, o->Materials[Mat].dif.g, o->Materials[Mat].dif.b, o->Materials[Mat].d };
                       //GLfloat EspecularColor[] = {o->Materials[Mat].spe.r, o->Materials[Mat].spe.g, o->Materials[Mat].spe.b, 1.0f };
                       GLfloat EspecularColor[] = {o->Materials[Mat].spe.r, o->Materials[Mat].spe.g, o->Materials[Mat].spe.b, o->Materials[Mat].d };
//                      GLfloat White[] = {1,1,1,1};
                       GLfloat White[] = {1,1,1,o->Materials[Mat].d};
                       //GLenum e = glGetError();
                       //printf("%f %f %f  %f %f %f  %f %f %f\n",DifuseColor[0],DifuseColor[1],DifuseColor[2],
                       //                                       AmbienteColor[0],AmbienteColor[1],AmbienteColor[2],
                       //                                       EspecularColor[0],EspecularColor[1],EspecularColor[2]);
                       if (o->Materials[Mat].TextureName[0] && o->Materials[Mat].flag)
                       {
                               o->Materials[Mat].TextObj.Use();
                               glDisable(GL_COLOR_MATERIAL);
                               glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,White);
                               glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,White);
                               glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,White);
                       }
                       else
                       {
                               glDisable(GL_TEXTURE_2D);
                               glDisable(GL_COLOR_MATERIAL);
                               glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,AmbienteColor);
                               glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,DifuseColor);
                               glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,EspecularColor);
                       }
               }

               glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);              // Set The Blending Function For Translucency
               glEnable(GL_BLEND);                                                                             // Turn Blending On

               glBegin(GL_TRIANGLES);
               double p[3][3];

               for (f = o->FacesWithMaterials[m].FirstFace; f <= o->FacesWithMaterials[m].LastFace; f++)
               {
                       if (!o->nVertexNormals)
                               glNormal3f(o->FaceNormals[f].x, o->FaceNormals[f].y, o->FaceNormals[f].z);
                       for (v=0; v< 3; v++)
                       {
                               // get the vertices
                               vt = o->Faces[f][v].v; // get the number of the vertice
                               // If there is a texture for this material...
                               if (Mat >= 0)
                                       if (o->Materials[Mat].TextureName[0])
                                       {
                                               tx = o->Faces[f][v].texture;
                                               glTexCoord2f(o->VertexTextures[tx].x, o->VertexTextures[tx].y);
                                       }
                               //
                               if (o->nVertexNormals)
                               {
                                       vn = o->Faces[f][v].normal;
                                       glNormal3f(o->VertexNormals[vn].x, o->VertexNormals[vn].y, o->VertexNormals[vn].z);
                               }
                               glVertex3f(o->Vertices[vt].x, o->Vertices[vt].y, o->Vertices[vt].z);
                               p[v][0] = o->Vertices[vt].x;
                               p[v][1] = o->Vertices[vt].y;
                               p[v][2] = o->Vertices[vt].z;

                       }
               }
               glEnd();

       }

// Restore the old rendering status
       glPopAttrib();


}


// *******************************************************
// DisplayOBJ
//              Exibe o objeto na tela
// *******************************************************
void OBJ_Display (TObject *o)
{

       if (o->nMaterialsUsed) // Is there any material for this object ?
       {
               OBJ_DisplayWithMaterials(o);
               return;
       }

       glBegin(GL_TRIANGLES);

       int f, v, vt, tx, vn;

       for (f=0; f< o->nFaces; f++)
       {
               if (!o->nVertexNormals)
                       glNormal3f(o->FaceNormals[f].x,
                       o->FaceNormals[f].y,
                       o->FaceNormals[f].z);
               for (v = 0; v< 3; v++)
               {
                       // get the vertices
                       vt = o->Faces[f][v].v; // get the number of the vertice
                       if (o->nVertexTextures)
                       {
                               tx = o->Faces[f][v].texture;
                               glTexCoord2f(o->VertexTextures[tx].x, o->VertexTextures[tx].y);
                       }
                       //
                       if (o->nVertexNormals)
                       {
                               vn = o->Faces[f][v].normal;
                               glNormal3f(o->VertexNormals[vn].x, o->VertexNormals[vn].y, o->VertexNormals[vn].z);
                       }
                       glVertex3f(o->Vertices[vt].x,
                               o->Vertices[vt].y,
                               o->Vertices[vt].z);
               }
       }
       glEnd();
}

// *******************************************************
// void FreeObjectData(TObject *o)
//              Clear all the memory allocated for the object
// *******************************************************
void FreeObjectData(TObject *o)
{
       free(o->Vertices);
       free(o->Faces);
       free(o->FaceNormals);
       free(o->VertexTextures);
       free(o->VertexNormals);
       free(o->FacesWithMaterials);
       free(o->Materials);
}

