// *******************************************************
//
// OBJ_Texture_LIB.h
//	Author: Marcio Serolli Pinho
//	Email: pinho@inf.pucrs.br
//
//   This library reads a subset set of Alias Wavefront OBJ
//	 files.
//   It just parse vertices and faces with 3 vertices.
//
//	 This code is based on the file OBJ2ASC converter found
//   at ftp://avalon.viewpoint.com/pub/utils/converters/
//
//
// *******************************************************

#ifndef _OBJ_Texture_Lib_
#define _OBJ_Texture_Lib_

#include <ImageLoaders/GLTexture/GLTexture.h>

typedef struct
{
	float x,y,z;
} TVertices;

//typedef int TFaces[3];
typedef struct
{
   int v,       // vertex index
	   normal,  // normal index for each vertex
	   texture; // texture index for each vertex
}TFaces[3];

typedef struct
{
	float r,g,b;
} TRGB;

typedef struct
{
	char Name[200];
	char TextureName[200];

	GLTexture TextObj;

	TRGB dif, spe, amb;
	float Ns, d, illum;
	bool flag;
} TMaterial;

typedef struct
{
	int FirstFace, LastFace;
	int Mat;
	char MaterialName[200];
	//TMaterial Mat;
} TFacesMat;

typedef struct
{
	TVertices *Vertices;
	TFaces *Faces;
	TFacesMat *FacesWithMaterials;
	TVertices *FaceNormals;
	TVertices *VertexTextures;
	TVertices *VertexNormals;
	TMaterial *Materials;

	int nVertices, nFaces, nVertexTextures, nVertexNormals, nMaterialsUsed, nMaterials;

	char Name[200];
	TVertices Min, Max;
	char TextureName[200];
	int NormalOrientation;

} TObject;



int OBJ_Texture_Reader(TObject *o);
void OBJ_Display(TObject *o);
void FreeObjectData(TObject *o);


void ReadMaterialFile(char *MaterialLibraryName, TObject *o);
int SearchForMaterialNumber(TObject *o, char *MaterialName);

#endif
