// ***************************************************
// SmVR_COBJLoader.cpp
//
//  Author: Marcio Serolli Pinho
//	Email: pinho@inf.pucrs.br
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2002.
// ************************************************

#include <stdio.h>
#include <string.h>


#ifdef WIN32
#include <windows.h>
#endif

#include "../SmVR_CObjFromFile.h" // virtual base class

#include "SmVR_COBJLoader.h"
#include "SmVR_Matrix.h"
//#include <conio.h>



// ***************************************************
//	SmVR_COBJLoader
//
//
// ***************************************************
SmVR_COBJLoader::SmVR_COBJLoader()
{
	strcpy(obj.Name,"Choose a name for your object !!");
	useRAPID = 0;
}

// ***************************************************
//	SmVR_COBJLoader
//
//
// ***************************************************
SmVR_COBJLoader::SmVR_COBJLoader(int rapid)
{
	strcpy(obj.Name,"Choose a name for your object !!");
	useRAPID = rapid;
}
// ************************************************
//  GetFileName()
//
// ************************************************
char *SmVR_COBJLoader::GetFileName()
{
    return obj.Name;
}


// ***************************************************
//	SmVR_COBJLoader
//
//
// ***************************************************
int SmVR_COBJLoader::Load(char const *filename)
{
	strcpy(obj.Name, filename);
	if (OBJ_Texture_Reader(&obj)) {
		if(useRAPID)
        	RAPID_model_Read(&obj,&collisionModel);
		return 1;
	}
	else {
		return 0;
	}

}

// ************************************************
// Method: Collided(SmVR_CObjFromFile *obj)
// Description: returns the number of colliding points
// for this object.
//
// ************************************************
int SmVR_COBJLoader::Collided(SmVR_CObjFromFile *obj,SmVR_Matrix mtx1,SmVR_Matrix mtx2, int mode)
{
	//implementacao
	double r1[3][3], r2[3][3], t1[3], t2[3];

  SmVR_RAPIDMatrixConversion(mtx1,r1,t1);
  SmVR_RAPIDMatrixConversion(mtx2,r2,t2);

  #ifdef TRACE2
  printf("MTX 1\t\t%.2f\t%.2f\t%.2f\t%.2f\n",mtx1[0][0],mtx1[0][1],mtx1[0][2],mtx1[0][3]);
  printf("     \t\t%.2f\t%.2f\t%.2f\t%.2f\n",mtx1[1][0],mtx1[1][1],mtx1[1][2],mtx1[1][3]);
  printf("     \t\t%.2f\t%.2f\t%.2f\t%.2f\n",mtx1[2][0],mtx1[2][1],mtx1[2][2],mtx1[2][3]);
  printf("     \t\t%.2f\t%.2f\t%.2f\t%.2f\n\n",mtx1[3][0],mtx1[3][1],mtx1[3][2],mtx1[3][3]);
  printf(" R1  \t\t%.2f\t%.2f\t%.2f\n",r1[0][0],r1[0][1],r1[0][2]);
  printf("     \t\t%.2f\t%.2f\t%.2f\n",r1[1][0],r1[1][1],r1[1][2]);
  printf("     \t\t%.2f\t%.2f\t%.2f\n",r1[2][0],r1[2][1],r1[2][2]);
  printf(" T1  \t\t%.2f\t%.2f\t%.2f\n\n",t1[0],t1[1],t1[2]);

  printf("MTX 2\t\t%.2f\t%.2f\t%.2f\t%.2f\n",mtx2[0][0],mtx2[0][1],mtx2[0][2],mtx2[0][3]);
  printf("     \t\t%.2f\t%.2f\t%.2f\t%.2f\n",mtx2[1][0],mtx2[1][1],mtx2[1][2],mtx2[1][3]);
  printf("     \t\t%.2f\t%.2f\t%.2f\t%.2f\n",mtx2[2][0],mtx2[2][1],mtx2[2][2],mtx2[2][3]);
  printf("     \t\t%.2f\t%.2f\t%.2f\t%.2f\n\n",mtx2[3][0],mtx2[3][1],mtx2[3][2],mtx2[3][3]);
  printf(" R2  \t\t%.2f\t%.2f\t%.2f\n",r2[0][0],r2[0][1],r2[0][2]);
  printf("     \t\t%.2f\t%.2f\t%.2f\n",r2[1][0],r2[1][1],r2[1][2]);
  printf("     \t\t%.2f\t%.2f\t%.2f\n",r2[2][0],r2[2][1],r2[2][2]);
  printf(" T2  \t\t%.2f\t%.2f\t%.2f\n\n",t2[0],t2[1],t2[2]);
	#endif

  if(!mode)
  {
    //printf("%d - FIRST CONTACT\n",mode);
    Collide(r1, t1, GetCollisionModel(), r2, t2, ((SmVR_COBJLoader*)obj)->GetCollisionModel(), FIRST_CONTACT);
  }

  else
  {
    //printf("%d - ALL CONTACTS\n",mode);
    Collide(r1, t1, GetCollisionModel(), r2, t2, ((SmVR_COBJLoader*)obj)->GetCollisionModel(), ALL_CONTACTS);
  }

	/*

  // looking at the report, we can see where all the contacts were, and
  // also how many tests were necessary:

  printf("All contacts between overlapping tori:\n");

  printf("Num box tests: %d\n", RAPID_num_box_tests);
  printf("Num contact pairs: %d\n", RAPID_num_contacts);

  */

  return RAPID_num_contacts;
}

// ***************************************************
//	SmVR_COBJLoader
//
//
// ***************************************************
void SmVR_COBJLoader::Render()
{
	OBJ_Display(&obj);
}

// ***************************************************
//	GetBoundaries
//
//
// ***************************************************
void SmVR_COBJLoader::GetBoundaries(SmVR_CPoint *min, SmVR_CPoint *max)
{
	min->X = obj.Min.x;
	min->Y = obj.Min.y;
	min->Z = obj.Min.z;

	max->X = obj.Max.x;
	max->Y = obj.Max.y;
	max->Z = obj.Max.z;

	//printf("Minimo: %f %f %f\n",obj.Min.x, obj.Min.y, obj.Min.z);
}

// ***************************************************
//	SmVR_COBJLoader::InvertNormalOrientation()
//
//
// ***************************************************
void SmVR_COBJLoader::InvertNormalOrientation()
{
	obj.NormalOrientation = !obj.NormalOrientation;
}

// ***************************************************
//	SmVR_COBJLoader::PrintEnvelope()
//
//
// ***************************************************
void SmVR_COBJLoader::PrintEnvelope()
{
	printf("Min: %f %f %f\n",obj.Min.x, obj.Min.y, obj.Min.z);
	printf("Max: %f %f %f\n",obj.Max.x, obj.Max.y, obj.Max.z);
}
// ***************************************************
//	SmVR_COBJLoader
//
//
// ***************************************************
void SmVR_COBJLoader::GetFace(int n, int v[3])
{
	v[0] = obj.Faces[n][0].v;
	v[1] = obj.Faces[n][1].v;
	v[2] = obj.Faces[n][2].v;
}
// ***************************************************
//	SmVR_COBJLoader
//
//
// ***************************************************
SmVR_COBJLoader::~SmVR_COBJLoader()
{
	//printf("'Destructor' from 'SmVR_COBJLoader' class.\n");
	FreeObjectData(&obj);
}
