// **********************************************************************
//	Leitura.cpp   --   Exemplo de um programa em OpenGL para ler 
//	obejtos 3D
//		 
//
// **********************************************************************

#include <windows.h>
#include <stdio.h>
#include <math.h>


#include <gl\gl.h>
#include <gl\glu.h>
#ifdef __GNU__
#include "glut.h" 
#endif
#ifndef __GNU__
#include <gl\glut.h>
#endif

typedef struct
{
	float X,Y,Z;
} TPoint;


typedef struct
{
	TPoint P1, P2, P3;
	float R,G,B;
	TPoint Normal;
} TTriangle;


// **********************************************************************
//  void Normalise(TPoint *p)
//
//
// **********************************************************************
void Normalise(TPoint *p)
{
	double length;
	
	length = sqrt(p->X * p->X + p->Y * p->Y + p->Z * p->Z);
	if (length != 0) {
		p->X /= length ;
		p->Y /= length ;
		p->Z /= length ;
	} else {
		p->X = 0;
		p->Y = 0;
		p->Z = 0;
	}
}

// **********************************************************************
//  void Normalise(TPoint *p)
//
//
// **********************************************************************
TPoint CalcNormal(TPoint p,TPoint p1,TPoint p2)
{
	TPoint n,pa,pb;
	
	pa.X = p1.X - p.X;
	pa.Y = p1.Y - p.Y;
	pa.Z = p1.Z - p.Z;
	pb.X = p2.X - p.X;
	pb.Y = p2.Y - p.Y;
	pb.Z = p2.Z - p.Z;
	Normalise(&pa);
	Normalise(&pb);
	
	n.X = pa.Y * pb.Z - pa.Z * pb.Y;
	n.Y = pa.Z * pb.X - pa.X * pb.Z;
	n.Z = pa.X * pb.Y - pa.Y * pb.X;
	Normalise(&n);
	
	return(n);
}


// **********************************************************************
// void LeObjeto(char *Nome)
//
//
// **********************************************************************
TTriangle *LeObjeto (char *Nome, int &NFaces)
{
	FILE *arq;
	int i, aux;
	char str[300];
	TTriangle *ptr;
	
	arq = fopen(Nome,"rt");
	if (!arq)
	{
		printf("Problemas com a  leitura do objeto '%s'\n", Nome);
		getchar();			
		exit(0);
	}
	NFaces=0;
	while(!feof(arq))
	{
		fgets(str,300,arq);
		NFaces++;
	}
	NFaces--;
	rewind(arq);

	printf("Allocating memory to hold the faces. (%f Mb)...\n", sizeof(TTriangle) * NFaces / 1000000.0f);
	ptr = (TTriangle *) malloc(sizeof(TTriangle) * NFaces);
	if (!ptr)
	{
		printf("Problemas com a alocao de memria para as %d faces do objeto\n", NFaces);
		getchar();			
		exit(0);
	}
	printf("Loading the faces...\n");

//	arq = fopen(Nome,"rt");

	for(i=0;i<NFaces;i++)
	{		
		fscanf(arq, "%f%f%f%f%f%f%f%f%f%i", 
				&ptr[i].P1.X, &ptr[i].P1.Y, &ptr[i].P1.Z,
				&ptr[i].P2.X, &ptr[i].P2.Y, &ptr[i].P2.Z,
				&ptr[i].P3.X, &ptr[i].P3.Y, &ptr[i].P3.Z, &aux);	
				ptr[i].B = (aux & 0x0000FF)/255.0;
				ptr[i].G = (aux & 0x00FF00)/255.0;
				ptr[i].R = (aux & 0xFF0000)/255.0;
		ptr[i].Normal = CalcNormal(ptr[i].P1, ptr[i].P2, ptr[i].P3);

	}
	fclose(arq);
	return ptr;

}

// **********************************************************************
// void ExibeObjeto (TTriangle **Objeto)
//
//
// **********************************************************************
void ExibeObjeto (TTriangle *Objeto, int NFaces)
{
	int i;
	glBegin(GL_TRIANGLES);
		for(i=0;i<NFaces;i++)
		{
			glColor3f(Objeto[i].R, Objeto[i].G, Objeto[i].B);
			glNormal3f(Objeto[i].Normal.X,Objeto[i].Normal.Y, Objeto[i].Normal.Z);
			glVertex3f(Objeto[i].P1.X, Objeto[i].P1.Y, Objeto[i].P1.Z);
			glVertex3f(Objeto[i].P2.X, Objeto[i].P2.Y, Objeto[i].P2.Z);
			glVertex3f(Objeto[i].P3.X, Objeto[i].P3.Y, Objeto[i].P3.Z);
		}
	glEnd();	  
}


