// Just to supress sscanf warning in the compiler
#define _CRT_SECURE_NO_DEPRECATE

#include "OBJLoader.h"

OBJLoader OBJLoader::single;

OBJLoader::OBJLoader ()
{
	
	matManager = OBJMaterialManager::getInstance ();

}

void OBJLoader::calcSizes ()
{

	long ini = (long) myfile->tellg ();

	std::string line = "a";

	int size = 0;

	while (!myfile->eof ())
	{

		getline (*myfile, line);

		if (line.size ())
		{

			if (line[0] == 'v')
			{

				if (line[1] == ' ')
				{

					while (line[0] != '#') getline (*myfile, line);

					sscanf (line.c_str (), "# %d", &size);

					auxVv = new float[auxVc = (size * 3)];

				}

				else if (line[1] == 'n')
				{

					while (line[0] != '#') getline (*myfile, line);

					sscanf (line.c_str (), "# %d", &size);

					auxNv = new float[auxNc = (size * 3)];

				}

				else if (line[1] == 't')
				{

					while (line[0] != '#') getline (*myfile, line);

					sscanf (line.c_str (), "# %d", &size);

					auxTv = new float[auxTc = (size * 3)];

					auxOBJ->texCoordCount = size;

				}

			}

			else if (line[0] == 'f')
			{

				while (line[0] != '#') getline (*myfile, line);

				sscanf (line.c_str (), "# %d", &size);

				auxOBJ->vertices = new float[auxOBJ->verticesCount = (size * 9)];

				auxOBJ->normals = new float[auxOBJ->verticesCount];

				if (auxOBJ->texCoordCount) 
				{

					auxOBJ->texCoordCount = size * 9;

					auxOBJ->texCoords = new float[auxOBJ->verticesCount];

				}

			}

		}

	}

	myfile->clear ();

	myfile->seekg (ini);

}

void OBJLoader::getVertices ()
{

	int index = 0;

	std::string line = "v";

	getline (*myfile, line);

	while (line[0] != '#')
	{

		while (!line.size ()) getline (*myfile, line);

		sscanf (line.substr (2).c_str (), "%f %f %f", &(auxVv[index]), &(auxVv[index + 1]), &(auxVv[index + 2]));

		index += 3;

		getline (*myfile, line);

	}

	getline (*myfile, line);

}

void OBJLoader::getVerticeNormals ()
{

	int index = 0;

	std::string line = "vn";

	getline (*myfile, line);

	while (line[0] != '#')
	{

		while (!line.size ()) getline (*myfile, line);

		sscanf (line.substr (2).c_str (), "%f %f %f", &(auxNv[index]), &(auxNv[index + 1]), &(auxNv[index + 2]));

		index += 3;

		getline (*myfile, line);

	}

	getline (*myfile, line);

}

void OBJLoader::getTextureCoords ()
{

	int index = 0;
	
	std::string line = "vt";

	getline (*myfile, line);

	if (!(line.size ()) || line[0] != 'v') return;

	while (line[0] != '#')
	{

		while (!line.size ()) getline (*myfile, line);

		sscanf (line.substr (2).c_str (), "%f %f %f", &(auxTv[index]), &(auxTv[index + 1]), &(auxTv[index + 2]));

		index += 3;

		getline (*myfile, line);

	}

	getline (*myfile, line);

}

void OBJLoader::getFaces ()
{

	bool lastOpaque = true;

	int index = 0;

	std::string line = "f";

	getline (*myfile, line);

	while (line[0] != '#')
	{

		while (!line.size ()) getline (*myfile, line);

		if (line[0] == 'u')
		{

			if (lastOpaque)
			{

				if (auxOBJ->opaqueCount)
					auxOBJ->opaque[auxOBJ->opaqueCount - 1].indCount = (index - auxOBJ->opaque[auxOBJ->opaqueCount - 1].begin);

			}

			else
			{

				if (auxOBJ->translucentCount)
					auxOBJ->translucent[auxOBJ->translucentCount - 1].indCount = (index - auxOBJ->translucent[auxOBJ->translucentCount - 1].begin);

			}

			char matName[32];

			sscanf (line.substr (7).c_str (), "%s", matName);

			if (matManager->getMaterial (matName)->translucent)
			{

				if (auxOBJ->translucentCount != MAX_MESHES)
				{

					auxOBJ->translucent[auxOBJ->translucentCount].material = matManager->getMaterial (matName);

					auxOBJ->translucent[auxOBJ->translucentCount].begin = index;

					auxOBJ->translucentCount++;

					lastOpaque = false;

				}

			}
			
			else if (auxOBJ->opaqueCount != MAX_MESHES)
			{

				auxOBJ->opaque[auxOBJ->opaqueCount].material = matManager->getMaterial (matName);

				auxOBJ->opaque[auxOBJ->opaqueCount].begin = index;

				auxOBJ->opaqueCount++;

				lastOpaque = true;

			}

		}

		else if (line[0] == 'f')
		{

			int v1, v2, v3;

			int t1, t2, t3;

			int n1, n2, n3;

			if (auxOBJ->texCoordCount) sscanf (line.substr (2).c_str (), "%d/%d/%d %d/%d/%d %d/%d/%d", 
				&v1, &t1, &n1, &v2, &t2, &n2, &v3, &t3, &n3);

			else sscanf (line.substr (2).c_str (), "%d//%d %d//%d %d//%d", 
				&v1, &n1, &v2, &n2, &v3, &n3);

			auxOBJ->normals [(3 * index)] = auxNv[(3 * (n1 - 1))];

			auxOBJ->normals [(3 * index) + 1] = auxNv[(3 * (n1 - 1)) + 1];

			auxOBJ->normals [(3 * index) + 2] = auxNv[(3 * (n1 - 1)) + 2];

			auxOBJ->normals [(3 * index) + 3] = auxNv[(3 * (n2 - 1))];

			auxOBJ->normals [(3 * index) + 4] = auxNv[(3 * (n2 - 1)) + 1];

			auxOBJ->normals [(3 * index) + 5] = auxNv[(3 * (n2 - 1)) + 2];

			auxOBJ->normals [(3 * index) + 6] = auxNv[(3 * (n3 - 1))];

			auxOBJ->normals [(3 * index) + 7] = auxNv[(3 * (n3 - 1)) + 1];

			auxOBJ->normals [(3 * index) + 8] = auxNv[(3 * (n3 - 1)) + 2];


			if (auxOBJ->texCoordCount)
			{

				auxOBJ->texCoords [(3 * index)] = auxTv[(3 * (t1 - 1))];

				auxOBJ->texCoords [(3 * index) + 1] = auxTv[(3 * (t1 - 1)) + 1];

				auxOBJ->texCoords [(3 * index) + 2] = auxTv[(3 * (t1 - 1)) + 2];

				auxOBJ->texCoords [(3 * index) + 3] = auxTv[(3 * (t2 - 1))];

				auxOBJ->texCoords [(3 * index) + 4] = auxTv[(3 * (t2 - 1)) + 1];

				auxOBJ->texCoords [(3 * index) + 5] = auxTv[(3 * (t2 - 1)) + 2];

				auxOBJ->texCoords [(3 * index) + 6] = auxTv[(3 * (t3 - 1))];

				auxOBJ->texCoords [(3 * index) + 7] = auxTv[(3 * (t3 - 1)) + 1];

				auxOBJ->texCoords [(3 * index) + 8] = auxTv[(3 * (t3 - 1)) + 2];

			}

			auxOBJ->vertices [(3 * index)] = auxVv[(3 * (v1 - 1))];

			auxOBJ->vertices [(3 * index) + 1] = auxVv[(3 * (v1 - 1)) + 1];

			auxOBJ->vertices [(3 * index) + 2] = auxVv[(3 * (v1 - 1)) + 2];

			auxOBJ->vertices [(3 * index) + 3] = auxVv[(3 * (v2 - 1))];

			auxOBJ->vertices [(3 * index) + 4] = auxVv[(3 * (v2 - 1)) + 1];

			auxOBJ->vertices [(3 * index) + 5] = auxVv[(3 * (v2 - 1)) + 2];

			auxOBJ->vertices [(3 * index) + 6] = auxVv[(3 * (v3 - 1))];

			auxOBJ->vertices [(3 * index) + 7] = auxVv[(3 * (v3 - 1)) + 1];

			auxOBJ->vertices [(3 * index) + 8] = auxVv[(3 * (v3 - 1)) + 2];

			index += 3;

		}

		getline (*myfile, line);

	}

	if (lastOpaque)
	{

		if (auxOBJ->opaqueCount)
			auxOBJ->opaque[auxOBJ->opaqueCount - 1].indCount = (index - auxOBJ->opaque[auxOBJ->opaqueCount - 1].begin);

	}

	else
	{

		if (auxOBJ->translucentCount)
			auxOBJ->translucent[auxOBJ->translucentCount - 1].indCount = (index - auxOBJ->translucent[auxOBJ->translucentCount - 1].begin);

	}

	delete auxNv;

	delete auxTv;

	delete auxVv;

}

void OBJLoader::passHeader (char * fileName)
{

	long seek = (long) myfile->tellg ();

	std::string line = "#";

	while (line[0] != 'v')
	{

		seek = (long) myfile->tellg ();

		getline (*myfile, line);

		while (!line.size ())
		{

			seek = (long) myfile->tellg ();

			getline (*myfile, line);

		}

		if (line[0] == 'm')
		{

			char matName[32];

			sscanf (line.substr (7).c_str (), "%s", &matName);

			std::string pathName = fileName;

			int pathIndex = pathName.find_last_of ('/');

			if (pathIndex < 0) matManager->loadMaterials (matName);

			else
			{

				pathName = pathName.substr (0, pathIndex + 1) + matName;

				matManager->loadMaterials ((char *)pathName.c_str ());

			}

		}

	}

	myfile->seekg (seek);

}

OBJLoader * OBJLoader::getInstance ()
{ return & single; }

OBJ * OBJLoader::loadOBJ (char * fileName)
{

	objsIt = objs.find (fileName);

	if (objsIt != objs.end ()) return objsIt->second;

	myfile = new std::ifstream (fileName);

	if (!myfile->is_open ()) return 0;

	auxOBJ = new OBJ ();

	passHeader (fileName);

	calcSizes ();

	getVertices ();

	getVerticeNormals ();

	getTextureCoords ();

	getFaces ();

	myfile->close ();

	delete myfile;

	objs[fileName] = auxOBJ;

	return auxOBJ;

}

void OBJLoader::clear ()
{

	objsIt = objs.begin ();

	while (objsIt != objs.end ())
	{
		
		delete objsIt->second;

		objsIt++;

	}

	objs.clear ();

}

OBJLoader::~OBJLoader ()
{

	clear ();

}