// Just to supress sscanf warning in the compiler
#ifndef _CRT_SECURE_NO_DEPRECATE
#define _CRT_SECURE_NO_DEPRECATE
#endif

#include "OBJMaterialManager.h"

#include "OBJTextureManager.h"

OBJMaterialManager OBJMaterialManager::single;

OBJMaterialManager::OBJMaterialManager ()
{}

void OBJMaterialManager::loadNewMaterial (std::ifstream * myfile, std::string materialName, std::string pathName)
{

	materialIt = materials.find (materialName);

	if (materialIt != materials.end ()) return;

	std::string line = "bazinga";

	OBJMaterial * newMat = new OBJMaterial ();

	while (!myfile->eof ())
	{

		getline (*myfile, line);

		if (!line.size ()) break;

		std::string type = line.substr (1, 2);

		if (!type.compare ("Ns"))
			sscanf (line.c_str (), "%*s %f", &(newMat->shininess));

		else if (!type.compare ("Tf"))
		{

			float alpha = 1.0f;

			sscanf (line.c_str (), "%*s %f", &alpha);

			if (alpha < 1.0f)
			{

				newMat->translucent = true;

				newMat->ambient[3] = alpha;

				newMat->diffuse[3] = alpha;

				newMat->emission[3] = alpha;

				newMat->specular[3] = alpha;

			}

		}

		else if (!type.compare ("Ka"))
			sscanf (line.c_str (), "%*s %f %f %f", 
			&(newMat->ambient[0]), &(newMat->ambient[1]), &(newMat->ambient[2]));

		else if (!type.compare ("Kd"))
			sscanf (line.c_str (), "%*s %f %f %f", 
			&(newMat->diffuse[0]), &(newMat->diffuse[1]), &(newMat->diffuse[2]));

		else if (!type.compare ("Ke"))
			sscanf (line.c_str (), "%*s %f %f %f", 
			&(newMat->emission[0]), &(newMat->emission[1]), &(newMat->emission[2]));

		else if (!type.compare ("Ks"))
			sscanf (line.c_str (), "%*s %f %f %f", 
			&(newMat->specular[0]), &(newMat->specular[1]), &(newMat->specular[2]));

		else if (!type.compare ("ma"))
		{

			char texName[256];

			sscanf (line.c_str (), "%*s %s", &texName);

			pathName += texName;

			newMat->textureID = OBJTextureManager::getInstance ()->loadTexture ((char *)pathName.c_str ());

			if (pathName.find (".p"))
			{

				newMat->translucent = true;

			}

			materials[materialName] = newMat;
			
			return;

		}

	}

	materials[materialName] = newMat;

}

OBJMaterialManager * OBJMaterialManager::getInstance ()
{ return & single; }

OBJMaterial * OBJMaterialManager::getMaterial (char * matName)
{ return materials[matName]; }

void OBJMaterialManager::loadMaterials (char * fileName)
{

	std::ifstream myfile (fileName);

	if (!myfile.is_open ()) return;

	std::string pathName = fileName;

	int pathIndex = pathName.find_last_of ('/');

	if (pathIndex >= 0)
		pathName = pathName.substr (0, pathIndex + 1);

	std::string line = "";

	while (!myfile.eof ())
	{

		getline (myfile, line);

		while (!myfile.eof () && (line.size () == 0)) getline (myfile, line);

		if (!line.substr (0, 3).compare ("new"))
		{

			loadNewMaterial (& myfile, line.substr (7), pathName);

		}

	}

	myfile.close ();

}

void OBJMaterialManager::clear ()
{

	materialIt = materials.begin ();

	while (materialIt != materials.end ())
	{

		delete materialIt->second;

		materialIt++;

	}

	materials.clear ();

}

OBJMaterialManager::~OBJMaterialManager ()
{

	clear ();

}