// ******************************************************
// SVE2RAPID.cpp
//		This module is responsible for make the comunication
// between SVE an RAPID.
//		It has 4 main functions that can be used by the
// one who wants to use RAPID with SVE:
//
//	1) void Insert_SVE_Object_into_RAPID (SVE_object o)
//		This function inserts an SVE object into RAPID 
//      data structure
//
//  2) int CheckColisionSVE_RAPID (SVE_object o1, SVE_object o2)
//		This function checks if there is collision between 2
//      SVE objects
//
//  3) void PutAllSVEObjectsOnRAPID(void)
//      This function inserts all the SVE current objects 
//	    into RAPID. It calls 'Insert_SVE_Object_into_RAPID' 
//
//  4) int CheckCollisionONEtoALL(SVE_object o)
//      This function verifies if any object previously 
//      inserted on RAPID collide with the given 'o' object.  
//		It calls 'CheckColisionSVE_RAPID'
//	
// ******************************************************

#include <conio.h>


#include <sve.h>
#include <rapid.h>
#include <UtilitiesSVE.h>
#include <sve2rapid.h>


// ******************************************************
//  Prints a SVE matrix
//		Just for debug....
// ******************************************************
void printM_SVE(M_matrix M)
{
  printf("%g  %g  %g  %g\n%g  %g  %g  %g\n%g  %g  %g  %g\n%g  %g  %g  %g\n",
	 M[0][0], M[0][1], M[0][2], M[0][3],
	 M[1][0], M[1][1], M[1][2], M[1][3],
	 M[2][0], M[2][1], M[2][2], M[2][3],
	 M[3][0], M[3][1], M[3][2], M[3][3]);
}
// ******************************************************
//  SVE_RAPIDMatrixConversion
//
//     Converts the SVE transformation matrix to 
//	RAPID Translation and Rotation Matrices
//     SVE uses a 4x4 transformation matrix
//     RAPID uses 
//			- a 1x3 Translation matrix
//			- a 3x3 Rotation matrix
//
// ******************************************************
void SVE_RAPIDMatrixConversion(	M_matrix M_SVE, // SVE matrix
							   double MR[3][3], // rotation matrix
							   double MT[3])    // translation matrix
{
	int i, j;

// copy the rotation part of the matrix
	// note that yhe matrix is transposed
	for (i=0;i<3;i++)
		for (j=0;j<3;j++)
			MR[i][j] = M_SVE[j][i];

// copy the translation part of the matrix
	for (i=0;i<3;i++)
		MT[i] = M_SVE[3][i];

}

// ******************************************************
//   CheckColisionSVE_RAPID 
//        Checks if there is a collision between 2 SVE
//    objects.
//	      Returns the number of collisions
//    OBS: the objetc identiifcation for SVE is in 
//		   'UserPtr' field of SVE_object data structure
//
// ******************************************************
int CheckColisionSVE_RAPID (SVE_object o1, SVE_object o2)
{
	double MR1[3][3], MT1[3];
	double MR2[3][3], MT2[3];
	M_matrix Matrix_o1, Matrix_o2;
	
//get the "o1" data from SVE
	SVE_getWorldMatrix(o1, Matrix_o1);
	SVE_RAPIDMatrixConversion(Matrix_o1,MR1, MT1);

//get the "o2" data from SVE
//PrintPositionFromMatrix("Shared Object (in CD - 1): ", o2->worldPosition);
    SVE_getWorldMatrix(o2, Matrix_o2);
//PrintPositionFromMatrix("Shared Object (in CD - 2): ", o2->worldPosition);
    SVE_RAPIDMatrixConversion(Matrix_o2,MR2, MT2);

// now we can perform a collision query:
	RAPID_Collide(MR1, MT1, (RAPID_model *) o1->UserPtr, 
				  MR2, MT2, (RAPID_model *) o2->UserPtr, 
				  RAPID_FIRST_CONTACT);

//    printf("Num box tests: %d\n", RAPID_num_box_tests);
//    printf("Num contact pairs: %d\n", RAPID_num_contacts);

//	if (RAPID_num_contacts)
//		printf("Contatos: %d e %d\n", RAPID_contact[0].id1, RAPID_contact[0].id2);

	return RAPID_num_contacts;
}

// ******************************************************
//    Insert_SVE_Object_into_RAPID 
//			Inserts the object faces into RAPID data
//		structure.		
// 			One important thing to notice is that 
//		the 'UserPtr' field of the SVE_object data 
//		strutures receices a reference of the RAPID object
//		that represents it.
//
// ******************************************************
void Insert_SVE_Object_into_RAPID (SVE_object o)
{

    SVE_primitive primitive;
	SVE_facePtr face;
	SVE_vertexPtr OneVertice;
	L_list faceNode, vertexNode;
    double Points[3][3];
	int count = 0;
	RAPID_model *aux; // temporary area for RAPID object

	if (o == NULL)
	{
	    printf("SVE-RAPID: NULL Object.\n");
		exit(1);
	}

	// check for valid objects
    if (IsORIGIN(o))  // Origin ?
	  if (IsLight(o)) // a Light ?
		{
		    printf("SVE-RAPID: Trying to insert a Light or the ORING on RAPID.\n");
			getch();
			exit(1);
		}

	printf("SVE-RAPID: Inserting %s on RAPID data structure.\n", 
		o->geometry->primitivesFilename);

// Initiates the RAPID data structure
	aux = new RAPID_model;

	if (aux == NULL)
	{
	    printf("SVE-RAPID: Memory alocation problem in RAPID (1).\n");
		getch();
		exit(1);
	}

// Browsing the SVE structure .....
    primitive = (SVE_primitive) o->geometry->primitives->data;

	
// for each face...... 
    for (faceNode = primitive->faceList; !L_listEmpty(faceNode);
         faceNode = L_getNext(faceNode)) 
	{
        face = (SVE_facePtr) L_getData(faceNode);
     // for each vertex ...... 
		int i; // number os vertex per face
        for (i=0, vertexNode = face->vertexList;
             !L_listEmpty(vertexNode);
             vertexNode = L_getNext(vertexNode), i++) 
		{
            if (i>2)
			{
				printf("SVE-RAPID: RAPID excepts only triangles.\n");
			    printf("SVE-RAPID: Face %d has more than 3 vertices", count);
				exit(1);
			}
            // put the vertex in a tmp vector
            OneVertice = (SVE_vertexPtr) L_getData(vertexNode);
			Points[i][0] = OneVertice->point->point[0];
			Points[i][1] = OneVertice->point->point[1];
			Points[i][2] = OneVertice->point->point[2];
        }
        // insert the point in RAPID
		if (aux->AddTri(Points[0], Points[1],Points[2], count) != RAPID_OK)
		{
		    printf("SVE-RAPID: Memory alocation problem in RAPID (1).\n");
			exit(1);
		}
		count++;
		if (!(count%1000))  // just to output some messages
			printf("SVE-RAPID: Inserting faces(%d)....\n", count);
    }
	printf("SVE-RAPID: Creating RAPID structure for the object...\n");
	if (aux->EndModel() != RAPID_OK)
	{
	    printf("SVE-RAPID: Unable to create the object in RAPID.\n");
		exit(1);
	}
	printf("SVE-RAPID: Object %s added in RAPID. %d triangles.\n", 
		o->geometry->primitivesFilename, count);
	
// Puts, in SVE_object structure, a reference for RAPID object
// This will be used in 'CheckColisionSVE_RAPID' function
	o->UserPtr = aux;
}

static int Num_objects =0; // number of objects inserted in RAPID
static SVE_object *ObjectList ; // LOCAL copy of the SVE object list

// ******************************************************
// void PutAllSVEObjectsOnRAPID(void)
//     This function inserts "almost" all the objects in 
//   SVE data structure into RAPID 
//   In fact, it does not perform the insertion by itself.
//	 It just browses the SVE object tree and calls the  
//   Insert_SVE_Object_into_RAPID function to insert each
//   object on the tree.
//		This function also keeps a copy of the SVE objects
//   inserted on RAPID. <<<<<< THIS should be removed in the
//                      <<<<<<  future ! It not essential 
// ******************************************************
//void PutAllSVEObjectsOnRAPID(SVE_object Pointer, SVE_object PartnerPointer)
void PutAllSVEObjectsOnRAPID(void)
{


// Get all the objects from SVE data structure
// and insert them into RAPID structure

	L_list objNode, objectTree;
    SVE_object object;

	printf("******************* Objects in the scene\n");
	printf("******************* Putting them on RAPID\n");

    objectTree = SVE_worldState->worldObject->children;

// Count how many objects we have on the scene
	
	for (Num_objects=0, objNode = objectTree; !L_listEmpty(objNode); objNode = L_getNext(objNode)) 
	{
        // Get the object from SVE list
		object = (SVE_object) L_getData(objNode);

		if (! IsORIGIN(object))
			if (object->geometry == NULL)
			{
				printf("SVE-RAPID: Object '%s' has NO geometry.\n", 
				object->name);
				getch();
				exit(1);
			}
			else if (!IsLight(object))
					Num_objects ++;
	}
    Num_objects -= 2; // The list contains also the 
					  // the "Pointer" and "PartnerPointer" objects. 

    // create a LOCAL copy of the  object list
	
	ObjectList = (SVE_object *) malloc(Num_objects*sizeof(SVE_object));

	int i;
	for (i=0,objNode = objectTree; !L_listEmpty(objNode); objNode = L_getNext(objNode)) 
	{
        // Get the object from SVE list
		object = (SVE_object) L_getData(objNode);

		printf("Object: %s.\n", object->name);
		// Check if it is the ORIGIN, the Pointer or a Light 
		// We don't need any of them.
		if (! IsORIGIN(object))
		  if (! IsLight(object))
			if (strcmp(object->name, "Pointer"))
				if (strcmp(object->name, "PartnerPointer"))
			//if ((object != Pointer) && (object != PartnerPointer))
				{
					Insert_SVE_Object_into_RAPID (object);  //send to RAPID
					ObjectList[i] = object;
					i++;
				}
    }
	printf("*******************\n");
}
// ******************************************************
//  int CheckCollisionONEtoALL(SVE_object o)
//     This function checks one object against all the 
//    others that were inserted on RAPID 
// ******************************************************
SVE_object CheckCollisionONEtoALL(SVE_object o)
{

	for (int i=0; i < Num_objects; i++)
	{
       if  (ObjectList[i] == o) // skip the self intersection
		   continue;
	   if (CheckColisionSVE_RAPID (ObjectList[i],o))
	   {
		   //printf("CheckCollisionONEtoALL:I have found a collision between %s and %s.\n",
		   //   ObjectList[i]->name, o->name);
		   return ObjectList[i];
	   } 
	}
    return NULL;
}

// ******************************************************
//	FreeMemorySVE_RAPID
//
// ******************************************************
void FreeMemorySVE_RAPID()
{
	int i;
	// free the RAPID memory
	for (i=0; i < Num_objects; i++)
	{
       free(ObjectList[i]->UserPtr);
	}
	
	free (ObjectList);

}
// ------ End of file SVE2RAPID -----