// ************************************************
//  SmVR_CEnvelope.cpp
//
//
//  Authors: Marcio Serolli Pinho
//           Regis Augusto Poli Kopper
//           Mauro Cesar Charao dos Santos
//
//  Emails: pinho@inf.pucrs.br
//          regis@webstuff.com.br
//          maurocharao@yahoo.com
//
//  Porto Alegre, Brazil, 2003.
// ************************************************
#include "SmVR_CEnvelope.h"

extern SmVR_CGeometricObject *RootObject;
extern int HaInterseccao(float V0[3], float V1[3], float V2[3],
                         float U0[3], float U1[3], float U2[3]);
//====================================================================================//
//                          Variaveis e funcoes estaticas                             //
//====================================================================================//

// Os 8 pontos do cubo de tamanho 1 centrado na origem
float SmVR_CEnvelope::vertices[8][3] = {
  {  .5,  .5,  .5 }, //0 - superior dianteiro direito
  { -.5,  .5,  .5 }, //1 - superior dianteiro esquerdo
  { -.5, -.5,  .5 }, //2 - inferior dianteiro esquerdo
  {  .5, -.5,  .5 }, //3 - inferior dianteiro direito
  {  .5,  .5, -.5 }, //4 - superior trazeiro direito
  { -.5,  .5, -.5 }, //5 - superior trazeiro esquerdo
  { -.5, -.5, -.5 }, //6 - inferior trazeiro esquerdo
  {  .5, -.5, -.5 }  //7 - inferior trazeiro direito
};

// As 12 faces triangulares do cubo
int SmVR_CEnvelope::faces[12][3] = {
  { 0, 1, 2 }, //  0 - frontal
  { 2, 3, 0 }, //  1 - frontal
  { 1, 2, 6 }, //  2 - esquerda
  { 6, 5, 1 }, //  3 - esquerda
  { 4, 7, 6 }, //  4 - traseira
  { 6, 5, 4 }, //  5 - traseira
  { 4, 7, 3 }, //  6 - direita
  { 3, 0, 4 }, //  7 - direita
  { 4, 0, 1 }, //  8 - superior
  { 1, 5, 4 }, //  9 - superior
  { 2, 3, 7 }, // 10 - inferior
  { 7, 6, 2 }  // 11 - inferior
};

// As 12 faces triangulares do cubo
float SmVR_CEnvelope::normais[12][3] = {
  {  0,  0,  1 },
  {  0,  0,  1 },
  { -1,  0,  0 },
  { -1,  0,  0 },
  {  0,  0, -1 },
  {  0,  0, -1 },
  {  1,  0,  0 },
  {  1,  0,  0 },
  {  0,  1,  0 },
  {  0,  1,  0 },
  {  0, -1,  0 },
  {  0, -1,  0 }
};
// ************************************************
//
//
//
//
// ************************************************

//construtor
SmVR_CEnvelope::SmVR_CEnvelope(char *name) : SmVR_CGeometricObject(name, desenha) {}
// cores
GLfloat SmVR_CEnvelope::color[4] = { 1.0, 0.0, 0.0, 1 };

// ************************************************
//
//
//
//
// ************************************************
int SmVR_CEnvelope::desenha(void *p) {
GLfloat tempColor[4];

  glGetFloatv(GL_CURRENT_COLOR, tempColor);	
  glColor4fv((p ? (GLfloat*)p : color));
  glBegin(GL_LINE_STRIP);
  for(int x = 0; x < 12; x++) {
    glNormal3fv(normais[x]);
    glVertex3fv(vertices[faces[x][0]]);
    glVertex3fv(vertices[faces[x][1]]);
    glVertex3fv(vertices[faces[x][2]]);
  }
  glEnd();
  glColor4fv(tempColor);

  return 1;
}

// ************************************************
//
//
//
//
// ************************************************
int SmVR_CEnvelope::colided(float u0[3], float u1[3], float u2[3]) {
  SmVR_CPoint *pt_aux = new SmVR_CPoint(0, 0, 0);
  SmVR_CPoint pt_in(0, 0, 0);
  float v0[3], v1[3], v2[3];

  for(int x = 0; x < 12; x++) {
    pt_in.X = vertices[faces[x][0]][0];
    pt_in.Y = vertices[faces[x][0]][1];
    pt_in.Z = vertices[faces[x][0]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v0[0] = pt_aux->X;
    v0[1] = pt_aux->Y;
    v0[2] = pt_aux->Z;

    pt_in.X = vertices[faces[x][1]][0];
    pt_in.Y = vertices[faces[x][1]][1];
    pt_in.Z = vertices[faces[x][1]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v1[0] = pt_aux->X;
    v1[1] = pt_aux->Y;
    v1[2] = pt_aux->Z;

    pt_in.X = vertices[faces[x][2]][0];
    pt_in.Y = vertices[faces[x][2]][1];
    pt_in.Z = vertices[faces[x][2]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v2[0] = pt_aux->X;
    v2[1] = pt_aux->Y;
    v2[2] = pt_aux->Z;

    if(HaInterseccao(v0, v1, v2, u0, u1, u2))
      return 1;
  }
  return 0;
}

// ************************************************
//
//
//
//
// ************************************************
int SmVR_CEnvelope::colided(SmVR_CEnvelope *obj) {
  SmVR_CPoint *pt_aux = new SmVR_CPoint(0, 0, 0);
  SmVR_CPoint pt_in(0, 0, 0);
  float v0[3], v1[3], v2[3];

  for(int x = 0; x < 12; x++) {
    pt_in.X = vertices[faces[x][0]][0];
    pt_in.Y = vertices[faces[x][0]][1];
    pt_in.Z = vertices[faces[x][0]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v0[0] = pt_aux->X;
    v0[1] = pt_aux->Y;
    v0[2] = pt_aux->Z;

    pt_in.X = vertices[faces[x][1]][0];
    pt_in.Y = vertices[faces[x][1]][1];
    pt_in.Z = vertices[faces[x][1]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v1[0] = pt_aux->X;
    v1[1] = pt_aux->Y;
    v1[2] = pt_aux->Z;

    pt_in.X = vertices[faces[x][2]][0];
    pt_in.Y = vertices[faces[x][2]][1];
    pt_in.Z = vertices[faces[x][2]][2];
    GetPointInOCS(pt_in, pt_aux, RootObject);
    v2[0] = pt_aux->X;
    v2[1] = pt_aux->Y;
    v2[2] = pt_aux->Z;

    if(obj->colided(v0, v1, v2))
      return 1;
  }
  return 0;
}
