// ************************************************
//  SmVR_CPath.cpp
//
//
//  Authors: Marcio Serolli Pinho
//           Regis Augusto Poli Kopper
//           Mauro Cesar Charao dos Santos
//
//  Emails: pinho@inf.pucrs.br
//          regis@webstuff.com.br
//          maurocharao@yahoo.com
//
//  Porto Alegre, Brazil, 2003.
// ************************************************
#include <malloc.h>
#include "SmVR_CPath.h"

extern SmVR_CGeometricObject *RootObject;
extern int HaInterseccao(float V0[3], float V1[3], float V2[3],
                         float U0[3], float U1[3], float U2[3]);

int SmVR_CPath::static_draw(void* p) {
  ((SmVR_CPath *)p)->draw(NULL);
  return 0;
}

SmVR_CPath::SmVR_CPath(char *scene, char *name) : SmVR_CGeometricObject(name, static_draw) {
  load(scene);
  SetRenderFunctionData(this);
}

float color[3] = { 1, 0, 0};
// **********************************************************************
// int draw(void *p)
// **********************************************************************
int SmVR_CPath::draw(void *p) {
  glColor3fv(color);
  glLineWidth(10);
  glBegin(GL_LINES);
  for(int x = 0; x < no_lines; x++) {
    glVertex3f(lines[x].x0, 0.0f, lines[x].y0);
    glVertex3f(lines[x].x1, 0.0f, lines[x].y1);
  }
  glEnd();
  return 1;
}

// **********************************************************************
// int SmVR_CPath::colided(float u0[3], float u1[3], float u2[3])
// **********************************************************************
int SmVR_CPath::colided(float u0[3], float u1[3], float u2[3]) {
  SmVR_CPoint pt_aux(0, 0, 0);
  SmVR_CPoint pt_in(0, 0, 0);
  float v0[3], v1[3];

  for(int x = 0; x < no_lines; x++) {
    pt_in.X = lines[x].x0;
    pt_in.Y = 0;
    pt_in.Z = lines[x].y0;
    GetPointInOCS(pt_in, &pt_aux, RootObject);
    v0[0] = pt_aux.X;
    v0[1] = pt_aux.Y;
    v0[2] = pt_aux.Z;

    pt_in.X = lines[x].x1;
    pt_in.Y = 0;
    pt_in.Z = lines[x].y1;
    GetPointInOCS(pt_in, &pt_aux, RootObject);
    v1[0] = pt_aux.X;
    v1[1] = pt_aux.Y;
    v1[2] = pt_aux.Z;

    if(HaInterseccao(v0, v1, v1, u0, u1, u2))
      return 1;
  }
  return 0;
}

// **********************************************************************
// int SmVR_CPath::colided(SmVR_CEnvelope *obj)
// **********************************************************************
int SmVR_CPath::colided(SmVR_CEnvelope *obj) {
  SmVR_CPoint pt_aux(0, 0, 0);
  SmVR_CPoint pt_in(0, 0, 0);
  float v0[3], v1[3], v2[3];

  for(int x = 0; x < 12; x++) {
    pt_in.X = obj->GetVertices(obj->GetFaces(x,0),0);
    pt_in.Y = obj->GetVertices(obj->GetFaces(x,0),1);
    pt_in.Z = obj->GetVertices(obj->GetFaces(x,0),2);
    obj->GetPointInOCS(pt_in, &pt_aux, RootObject);
    v0[0] = pt_aux.X;
    v0[1] = pt_aux.Y;
    v0[2] = pt_aux.Z;

    pt_in.X = obj->GetVertices(obj->GetFaces(x,1),0);
    pt_in.Y = obj->GetVertices(obj->GetFaces(x,1),1);
    pt_in.Z = obj->GetVertices(obj->GetFaces(x,1),2);
    obj->GetPointInOCS(pt_in, &pt_aux, RootObject);
    v1[0] = pt_aux.X;
    v1[1] = pt_aux.Y;
    v1[2] = pt_aux.Z;

    pt_in.X = obj->GetVertices(obj->GetFaces(x,2),0);
    pt_in.Y = obj->GetVertices(obj->GetFaces(x,2),1);
    pt_in.Z = obj->GetVertices(obj->GetFaces(x,2),2);
    obj->GetPointInOCS(pt_in, &pt_aux, RootObject);
    v2[0] = pt_aux.X;
    v2[1] = pt_aux.Y;
    v2[2] = pt_aux.Z;

    /*printf("%d: ((%4.2f,%4.2f,%4.2f),(%4.2f,%4.2f,%4.2f),(%4.2f,%4.2f,%4.2f))\n"
           ,x
           ,v0[0],v0[1],v0[2]
           ,v1[0],v1[1],v1[2]
           ,v2[0],v2[1],v2[2]);*/

    if(this->colided(v0, v1, v2))
      return 1;
  }
  return 0;
}

// **********************************************************************
// int load(char *name)
// **********************************************************************
int SmVR_CPath::load(char *name) {
  FILE *arq = fopen(name, "r");
  int x;

  if(arq == NULL) {
    printf("Arquivo de cenario %s nao encontrado!!!\n", name);
    return 0;
  }
  if(!fscanf(arq, "%d", &no_lines)) {
    printf("O arquivo deve conter a quantidade de linhas seguida de 4 pontos para cada linha!!!\n", name);
    return 0;
  }
  if(no_lines < 0) {
    printf("Quantidade de linhas (%d) invalida!!!\n", no_lines);
    return 0;
  }
  lines = (line*)malloc(sizeof(line) * no_lines);
  for(x = 0; x < no_lines && !feof(arq); x++) {
    while(!fscanf(arq, "%f", &lines[x].x0)) printf("Erro ao ler x0 da linha %d\n", x);
    while(!fscanf(arq, "%f", &lines[x].y0)) printf("Erro ao ler y0 da linha %d\n", x);
    while(!fscanf(arq, "%f", &lines[x].x1)) printf("Erro ao ler x1 da linha %d\n", x);
    while(!fscanf(arq, "%f", &lines[x].y1)) printf("Erro ao ler y1 da linha %d\n", x);
  }
  return 1;
}
