// ***************************************************
// SmVR_CCyberTrackII.cpp
//		This module implements the class that handle
//	the CybertrackII (CTII) device.
//
//
//  Author: Marcio Serolli Pinho
//	Emial: pinho@inf.pucrs.br		 
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2002.
// ***************************************************

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pthread.h"

pthread_t thread1;
pthread_mutex_t barrier1;

#include "../SerialLib.h"


// ***************************************************
// ***************************************************
// ***************************************************

//  CybertrackII Driver functions

// ***************************************************
// void CTII_ReadTrackerOrientation(char *data)
//
//
// ***************************************************
void *CTII_ReadTrackerOrientation(void *p)
{
	char aux[100], *data = (char*)p;
	int i;

	while(1)
	{
		WriteSerialString("S"); // Request a tracker data

		i=0;
		pthread_mutex_lock(&barrier1);
		do{
			ReadStrFromSerial(aux,1);
			data[i] = aux[0];
			if (aux[0] == '\0') // read until the end of 
				break;			// the stream
			i++;
		} while (1);
		pthread_mutex_unlock(&barrier1);
	}
}

// ***************************************************
// int CTII_CyberTrackOK()
//		Check if the tracker is working properly
//
// ***************************************************
int CTII_CyberTrackOK()
{
	char Nome[10];

	ReadStrFromSerial(Nome, 3);
	if (strcmp(Nome, "CTM"))
	{
		return 0;
		exit(1);
	}
	else return 1;
}

// ***************************************************
// void CTII_InitCyberTrack(char *port)
//		Initialie the CTII
//
// ***************************************************
void CTII_InitCyberTrack(char *port, char *trackerData)
{

    // Open the serial port.... (port, speed, timeout)
	// Sometimes it is necessary to change the timeout
	if (OpenSerialPort(port, 9600, 50) == -1) 
	{
        printf("Failed to open the serial port: '%s'", port);
        exit(1);
    }

    printf("Serial port openned.\n");

	// Inicializa the CTII
	WriteSerialString("R");
	WriteSerialString("X");

	if (CTII_CyberTrackOK())
	{
		printf("Cybertrack II successfully initialized!\n");
		WriteSerialString("E"); // Set the CTII to ASCII mode
		return;
	}
	pthread_create(&thread1, NULL, CTII_ReadTrackerOrientation, trackerData);
	pthread_mutex_init(&barrier1,NULL);

	printf("Failed to initialize Cyber Track II.\n");
	exit(1);

}


// ***************************************************
// ***************************************************
// ***************************************************

// Implementation of SmVR_CCyberTrackII class

#include "SmVR_CCyberTrackII.h"

SmVR_CCyberTrackII::SmVR_CCyberTrackII()
{
	//printf("'CONSTRUCTOR' method from NEW class....\n"); 
	BindObject=NULL;
	NumberOfSensors = 1;

}
// ***************************************************
// int SmVR_CCyberTrackII::Init(void *p)
//	
//
// ***************************************************
int SmVR_CCyberTrackII::Init(void *p)
{
	char aux[10];

	//printf("'Init' method from NEW class....\n"); 
	strncpy(aux, (char *)p, 3);
	aux[3] = '\0';
	if (strcmp(aux,"COM"))
	{
		printf("The parameter passed to 'Init' method must be \"COM1\", \"COM2\", ...\n"); 
	}
	// for CTII 'p' must be "COM1", "COM2", ... 
	CTII_InitCyberTrack((char *)p, StrRead); 
	
	return 0;
}

// ***************************************************
//	int SmVR_CCyberTrackII::Read(void *data)
//
//
// ***************************************************
int SmVR_CCyberTrackII::Read(void *data)
{
	char yaw, pitch, roll;
	int i;

	//printf("'Read' method from NEW class....\n"); 

	//CTII_ReadTrackerOrientation(aux);

	//printf("Read from CTII: %s\n", aux);

	// Save the read data in class member
	for(i=0;i<10;i++)
		if(pthread_mutex_trylock(&barrier1)!=EBUSY)
		{
			strcpy(aux, StrRead);
			pthread_mutex_unlock(&barrier1);
			break;
		}

	// The CTII string data is allways like this
	//     Y304.3P-00.9R-00.9
	// So we use 'sscanf' to parse it

	sscanf(StrRead, "%c%f%c%f%c%f", &yaw, &y, &pitch, &x, &roll, &z);

	if ((yaw != 'Y') || (pitch != 'P') || (roll != 'R'))
		printf("Problems reading the tracker\n");
	//printf("x = %f  y= %f  z = %f\n",x,y,z);

	return 0;
}

// ***************************************************
// void SmVR_CCyberTrackII::Bind(int nSensor, SmVR_CGeometricObject *o )
//
//
// ***************************************************
void SmVR_CCyberTrackII::Bind(int nSensor, SmVR_CGeometricObject *o )
{

	BindObject = o;
	//printf("'Bind' method from NEW class....\n"); 
}


// ***************************************************
//
// ***************************************************
void SmVR_CCyberTrackII::UpdateAllSensors (void)
{
	char Temp[100];
	SmVR_CPoint Zero(0,0,0), Pos;

	// read the tracker
	Read(Temp);

	// Update the object linked to the tracker 
	if (BindObject)
	{	
		// Reset the rotation part of the object matrix
		BindObject->SetRotationMatrixToIdent();
		// Set the the object with data that has come from the tracker
		BindObject->RotateBy(x,1,0,0);
		BindObject->RotateBy(-y,0,1,0);
		BindObject->RotateBy(z,0,0,1);
	}
}

// ***************************************************
//
// ***************************************************
void SmVR_CCyberTrackII::SetExtraData(void *data)
{
	printf("'SetExtraData' method from NEW class....\n"); 
}


// ***************************************************
//
// ***************************************************
SmVR_CCyberTrackII::~SmVR_CCyberTrackII(void)
{
	CloseSerialPort();
}
