// ***************************************************
// SmVR_CGlasses.cpp
//		This module implements the class that handle
//	the IGlasses Tracker device.
//
//
//  Author: Marcio Serolli Pinho
//	Emial: pinho@inf.pucrs.br		 
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2002.
// ***************************************************

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pthread.h"
#include "SmVR.h"

pthread_t thread;
pthread_mutex_t barrier;

#include "../SerialLib.h"

float x0, y0, z0, LastX, LastY, LastZ/*, x, y, z*/;


// ***************************************************
// ***************************************************
// ***************************************************

//  IGlasses Driver functions

// **************************************************
//  int IglassesREAD(float *x, float *y, float *z)
//
//
// **************************************************
void* IglassesREAD(void *p)
{
	int byteRead;
	int i, countError=0;
	short intRead;
	int msB, lsB;
	float data[3];
	SmVR_CPoint *xyz = (SmVR_CPoint*)p;
	while(1)
	{
		do
		{
			// Send the reading command to the tracker....
			WriteSerialOneByte('S');
			// Read data from the serail port  
			byteRead = ReadSerialOneByte();

			if (byteRead == -1) // There is and error 
			{
				SerialClearBuffer();  // Clear the serial port
				return 0; // fim.
			}

			if (byteRead == 0) // Something has been read...
			{
				SerialClearBuffer();  // Clear the serial port
				printf("Waiting for a new data from Iglasses...\n");
				countError++;
				if (countError > 3)
				{
					printf("Iglasses is problably disconnected from the serial port.\n");
					exit(1);
				}
			}	
		} while (byteRead == 0); // try again
		
		// Some data has been read
		for (i=0; i<3; i++) 
		{
			msB = ReadSerialOneByte();
			lsB = ReadSerialOneByte();
			if (msB > 127) 
				intRead = (msB-255) * 255 + (lsB-255);
			else
				intRead = msB * 255 + lsB;

			// Save the X, Y and Z in [0], [1] and [2]        
			data[i] = (float) intRead * (180.0f/16384.0f);
		}
		// setup the output...
		//while(pthread_mutex_trylock(&barrier) == EBUSY);
		pthread_mutex_lock(&barrier);
		//printf("barrier thread 1\n");
		xyz->Y = data[0];
		xyz->X = data[1];
		xyz->Z = data[2];
		//pthread_cond_signal(&cond);
		//pthread_mutex_unlock(&barrier);

		// Clear the serial port 
		SerialClearBuffer();
		// Discard the unused data
		WriteSerialString("!R");
		
		//while(pthread_mutex_trylock(&barrier) == EBUSY);
		//pthread_mutex_lock(&barrier);
		//printf("barrier thread 2\n");
		LastX = xyz->X;
		LastY = xyz->Y;
		LastZ = xyz->Z;
		printf("x0 = %6.2f\t", x0);	
		xyz->X = xyz->X - x0;
		while (xyz->X > 180) 
			xyz->X = xyz->X - 360;
		while (xyz->X < -180) 
			xyz->X = xyz->X + 360;
		printf("y0 = %6.2f\t", y0);	
		xyz->Y = xyz->Y - y0;
		while (xyz->Y > 180) 
			xyz->Y = xyz->Y - 360;
		while (xyz->Y < -180) 
			xyz->Y = xyz->Y + 360;

		printf("z0 = %6.2f\n", z0);	
		xyz->Z = xyz->Z - z0;
		while (xyz->Z > 180) 
			xyz->Z = xyz->Z - 360;
		while (xyz->Z < -180) 
			xyz->Z = xyz->Z + 360;
		//pthread_cond_signal(&cond);
		//pthread_cond_signal(&cond1);
		pthread_mutex_unlock(&barrier);
	}
	return p;
}

// **************************************************
//  void IglassesEND()
//
//
// **************************************************
void IglassesEND()
{
	CloseSerialPort();
}

// **************************************************
//  void IglassesRESET()
//
//
// **************************************************
void IglassesRESET_AXIS()
{
	//while(pthread_mutex_trylock(&barrier) == EBUSY) /*printf("busy!!!")*/;
	pthread_mutex_lock(&barrier);
	printf("barrier reset\n");
	//pthread_cond_wait(&cond1,&barrier);
	//printf("barrier reset cond ok\n");
	x0 = LastX;
	y0 = LastY;
	z0 = LastZ;
	pthread_mutex_unlock(&barrier);
	printf ("Reseting Iglasses...\n");

}

// **************************************************
//  int IglassesINIT(char *port )
//		'port' can be "COM1", "COM2", "COM3",....  
//
// **************************************************
int IglassesINIT(char *port, SmVR_CPoint *xyz)
{
	x0 = y0 = z0 = 0.0f;

    // Open the serial port.... (port, speed, timeout)
	// Sometimes it is necessary to change the timeout
	if (OpenSerialPort(port,19200,500) == -1)
	{
        printf("Problems opening serial port (%s)...\n", port);
        return 0;
    }
	// initialize the tracker
	WriteSerialString("!M2,P,B,5,3\r");
	// WriteSerialString("!M2,P,B,7,7\r");
    printf("Serial port (%s) initialized.\n", port);
	pthread_create(&thread, NULL, IglassesREAD, xyz);
	pthread_mutex_init(&barrier,NULL);
	//pthread_mutex_init(&barrier1,NULL);
    //pthread_cond_init(&cond, NULL);
    //pthread_cond_init(&cond1, NULL);
	//pthread_cond_init(&cond2, NULL);
	//pthread_cond_signal(&cond1);
	return 1;
}

// ***************************************************
// ***************************************************
// ***************************************************

// Implementation of SmVR_CIGlasses class

#include "SmVR_CIGlasses.h"

SmVR_CIGlasses::SmVR_CIGlasses()
{
	//printf("'CONSTRUCTOR' method from NEW class....\n"); 
	BindObject=NULL;
	NumberOfSensors = 1;

}

// ***************************************************
// int SmVR_CIGlasses::Reset(void)
//
//
// ***************************************************
int SmVR_CIGlasses::Reset(void *p)
{
	IglassesRESET_AXIS();
	return 0;
}


// ***************************************************
// int SmVR_CIGlasses::Init(void *p)
//	
//
// ***************************************************
int SmVR_CIGlasses::Init(void *p)
{
	char aux[10];

	//printf("'Init' method from NEW class....\n"); 
	strncpy(aux, (char *)p, 3);
	aux[3] = '\0';
	if (strcmp(aux,"COM"))
	{
		printf("The parameter passed to 'Init' method must be \"COM1\", \"COM2\", ...\n"); 
	}
	// for Iglasses 'p' must be "COM1", "COM2", ... 
	xyz = new SmVR_CPoint(0, 0, 0);
	IglassesINIT((char *)p, xyz); 
	
	return 0;
}

// ***************************************************
//	int SmVR_CIGlasses::Read(void *data)
//
//
// ***************************************************
int SmVR_CIGlasses::Read(void *data)
{

	//printf("'Read' method from NEW class....\n"); 

	//IglassesREAD(&x, &y, &z);

	//printf("Read from Iglasses: %s\n", aux);

	return 0;
}

// ***************************************************
// void SmVR_CIGlasses::Bind(int nSensor, SmVR_CGeometricObject *o )
//
//
// ***************************************************
void SmVR_CIGlasses::Bind(int nSensor, SmVR_CGeometricObject *o )
{

	BindObject = o;
	//printf("'Bind' method from NEW class....\n"); 
}


// ***************************************************
//
// ***************************************************
void SmVR_CIGlasses::UpdateAllSensors (int rx, int ry, int rz)
{
	//char Temp[100];
	SmVR_CPoint Zero(0,0,0), Pos;

	// read the tracker
	//Read(Temp);

	// Update the object linked to the tracker 
	if (BindObject)
	{	
		// Reset the rotation part of the object matrix
		BindObject->SetRotationMatrixToIdent();
		// Set the the object with data that has come from the tracker
		//pthread_cond_wait(&cond,&barrier);
		pthread_mutex_lock(&barrier);
		printf("x = %6.2f\ty = %6.2f\tz = %6.2f\n",xyz->X,xyz->Y,xyz->Z);
		if(ry) BindObject->RotateBy(xyz->Y,0,1,0);
		if(rx) BindObject->RotateBy(-xyz->X,1,0,0);
		if(rz) BindObject->RotateBy(xyz->Z,0,0,1);
		pthread_mutex_unlock(&barrier);
	}
}

// ***************************************************
//
// ***************************************************
void SmVR_CIGlasses::UpdateAllSensors (void)
{
    UpdateAllSensors (1,1,1);
}

// ***************************************************
//
// ***************************************************
void SmVR_CIGlasses::SetExtraData(void *data)
{
	printf("'SetExtraData' method from NEW class....\n"); 
}


// ***************************************************
//
// ***************************************************
SmVR_CIGlasses::~SmVR_CIGlasses(void)
{
	CloseSerialPort();
}
