// ***************************************************
// SmVR_CGlasses.cpp
//		This module implements the class that handle
//	the Isotrack Tracker device.
//
//
//  Author: Marcio Serolli Pinho
//	Emial: pinho@inf.pucrs.br		 
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2002.
// ***************************************************

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pthread.h"



//#include "../SerialLib.h"



#include "polhemus.h"
CPolhemus *frak;

pthread_t thread3;
pthread_mutex_t barrier666, barrier777;

static double POS[2][3], ROT[2][3], LastPos[2][3], pos[2][3], rot[2][3];;

// ***************************************************
// ***************************************************
// ***************************************************

//  Isotrack Driver functions

// **************************************************
//  int IsotrackREAD(double Pos1[3], double Rot1[3], 
//				     double Pos2[3], double Rot2[3])
//
//
// **************************************************
void * IsotrackREAD( void *p)
{
	int i;
	while(1)
	{
		
		printf("\b");
		pthread_mutex_lock(&barrier777);
		frak->GetPosition(1, pos[0]);
		frak->GetOrientation(1, rot[0]);
		frak->GetPosition(2, pos[1]);
		frak->GetOrientation(2, rot[1]);
		pthread_mutex_unlock(&barrier777);

		pthread_mutex_lock(&barrier666);
		for(i = 0; i < 3; i++)
		{
			POS[0][i] = pos[0][i];
			POS[1][i] = pos[1][i];
			ROT[0][i] = rot[0][i];
			ROT[1][i] = rot[1][i];
		}
		pthread_mutex_unlock(&barrier666);
		
	}


		//printf("Pos1= %7.2f, %7.2f, %7.2f\n", Pos1[0],Pos1[1], Pos1[2]);
		//printf("Pos2= %7.2f, %7.2f, %7.2f\n\n", Pos2[0],Pos2[1], Pos2[2]);

	return p;
}

// **************************************************
//  void IsotrackEND()
//
//
// **************************************************
void IsotrackEND()
{
//	CloseSerialPort();

}

// **************************************************
//  void IsotrackRESET()
//
//
// **************************************************
void IsotrackRESET_AXIS()
{
	
	pthread_mutex_lock(&barrier777);
	frak->GetPosition(1, LastPos[0]);
	frak->GetPosition(2, LastPos[1]);
	pthread_mutex_unlock(&barrier777);
	printf ("Reseting Isotrack...\n");

	

}

// **************************************************
//  int IsotrackINIT(char *port )
//		'port' can be "COM1", "COM2", "COM3",....  
//
// **************************************************
int IsotrackINIT(char *port )
{
	frak = new CPolhemus(port, B9600);


	LastPos[0][0] = LastPos[0][1] = LastPos[0][2] = 0;
	LastPos[1][0] = LastPos[1][1] = LastPos[1][2] = 0;

	return 1;
}

// ***************************************************
// ***************************************************
// ***************************************************

// Implementation of SmVR_CIsotrack class

#include "SmVR_CIsotrack.h"

SmVR_CIsotrack::SmVR_CIsotrack()
{

	int i;

	//printf("'CONSTRUCTOR' method from NEW class....\n"); 
	NumberOfSensors = 2;

	for (i=0; i< NumberOfSensors;i++)
		BindObject[i]=NULL;
}



// ***************************************************
// int SmVR_CIsotrack::Init(void *p)
//	
//
// ***************************************************
int SmVR_CIsotrack::Init(void *p)
{
	char aux[10];

	//printf("'Init' method from NEW class....\n"); 
	strncpy(aux, (char *)p, 3);
	aux[3] = '\0';
	if (strcmp(aux,"COM"))
	{
		printf("The parameter passed to 'Init' method must be \"COM1\", \"COM2\", ...\n"); 
	}
	// for Isotrack 'p' must be "COM1", "COM2", ... 
	IsotrackINIT((char *)p); 
	pthread_create(&thread3, NULL, IsotrackREAD, NULL);
	pthread_mutex_init(&barrier666,NULL);
	pthread_mutex_init(&barrier777,NULL);
	
	return 0;
}

// ***************************************************
//	int SmVR_CIsotrack::Read(void *data)
//
//
// ***************************************************
int SmVR_CIsotrack::Read(void *data)
{

	//printf("'Read' method from NEW class....\n"); 

	//IsotrackREAD (POS[0], ROT[0], POS[1], ROT[1]);

	//printf("Read from Isotrack: %s\n", aux);

	return 0;
}

// ***************************************************
// void SmVR_CIsotrack::Bind(int nSensor, SmVR_CGeometricObject *o )
//
//
// ***************************************************
void SmVR_CIsotrack::Bind(int nSensor, SmVR_CGeometricObject *o )
{

	BindObject[nSensor] = o;
	//printf("'Bind' method from NEW class....\n"); 
}

// ***************************************************
//
// ***************************************************
void SmVR_CIsotrack::UpdateAllSensors (int tx, int ty, int tz, int rx, int ry, int rz)
{
	int i;
	SmVR_CPoint Zero(0,0,0), Pos;

	// read the tracker
	//Read(NULL);

	// Update the object linked to the tracker 
	for (i=0; i< NumberOfSensors;i++)
		if (BindObject[i])
		{	
			// Reset the rotation part of the object matrix
			//BindObject[i]->SetRotationMatrixToIdent();
			BindObject[i]->SetLocalTransformationMatrixToIdentity();

			// Set the the object with data that has come from the tracker
			// reset the tracker position 
			
			BindObject[i]->TranslateBy((tx ? (float)((int)( LastPos[i][1]*2.54))/1 : 0),
                                       (ty ? (float)((int)( LastPos[i][2]*2.54))/1 : 0),
                                       (tz ? (float)((int)(-LastPos[i][0]*2.54))/1 : 0));
			// Move the tracker object
			//printf("antes update\n");
			pthread_mutex_lock(&barrier666);
			BindObject[i]->TranslateBy((tx ? (float)((int)(-POS[i][1]*2.54))/1 : 0),
                                       (ty ? (float)((int)(-POS[i][2]*2.54))/1 : 0),
                                       (tz ? (float)((int)( POS[i][0]*2.54))/1 : 0));

			if(ry) BindObject[i]->RotateBy(((int)(180-ROT[i][0])/1),0,1,0);
			if(rx) BindObject[i]->RotateBy(((int)ROT[i][1]/1),1,0,0);
			if(rz) BindObject[i]->RotateBy(((int)(-ROT[i][2])/1),0,0,1); 
			//printf("fim update\n");
			pthread_mutex_unlock(&barrier666);


		}
	//printf("x = %6.2f\ty = %6.2f\tz = %6.2f\n",x,y,z);
	//printf("ROT x = %6.2f\n", ROT[0][1]);
}

// ***************************************************
//
// ***************************************************
void SmVR_CIsotrack::UpdateAllSensors (void)
{
    UpdateAllSensors (1,1,1,1,1,1);
}



// ***************************************************
// int SmVR_CIsotrack::Reset(void *p)
//
//
// ***************************************************
int SmVR_CIsotrack::Reset(void *p)
{
	IsotrackRESET_AXIS();
	return 0;
}
////////////////////////////////////////////////////////////////////////////////
/*
extern SmVR_CGeometricObject *RootObject;
// ***************************************************
//
// ***************************************************
void SmVR_CIsotrack::UpdateBind(int i)
{
	// Reset all local the transformations for the object
	BindObject[i]->SetLocalTransformationMatrixToIdentity();				
	
	// Apply the translations and rotations
	BindObject[i]->RotateBy(ROT[i][1],1,0,0);
	BindObject[i]->RotateBy(180-ROT[i][0],0,1,0);
	BindObject[i]->RotateBy(-ROT[i][2],0,0,1); 
	
	BindObject[i]->TranslateBy(-POS[i][1],-POS[i][2], POS[i][0]);
	
}

// ***************************************************
// void SmVR_CIsotrack::UpdateAllSensors (void)
//
//
// ***************************************************
void SmVR_CIsotrack::UpdateAllSensors (void)
{
	int i;
	SmVR_CPoint Zero(0,0,0), Pos;
	SmVR_Matrix MLocal;
	SmVR_Matrix MResult;

	// read the tracker
	Read(NULL);

	// Update each object linked to the tracker 
	for (i=0; i< NumberOfSensors;i++)
		if (BindObject[i])
		{	
			//BindObject[i]->GetLocalTransformationMatrix(MOriginal);
			//SmVR_PrintMatrix("UPDATE: Matriz Local ANTES\n",MOriginal);

			UpdateBind(i);

			// Get the updated transformation matrix
			BindObject[i]->GetLocalTransformationMatrix(MLocal);

			// Multiply the current transformation by the inverse matrix 
			// obtained when the last RESET was done
			// MResult = MLocal * Inverse[i]
			SmVR_MultMatrix(MLocal, Inverse[i], MResult);

			// Update the object transformation matrix
			BindObject[i]->SetLocalTransformationMatrix(MResult);
		}
}
// ***************************************************
// int SmVR_CIsotrack::Reset(void *p)
//
//
// ***************************************************
int SmVR_CIsotrack::Reset(void *p)
{
	SmVR_Matrix MLocal;
	int i;

	Read(NULL);

	// Update the object linked to the tracker 
	for (i=0; i< NumberOfSensors;i++)
		if (BindObject[i])
		{	

			UpdateBind(i);

			// Get the updated transformation matrix
			BindObject[i]->GetLocalTransformationMatrix(MLocal);

			// Save the inverse of this transformation for future use
			// Check the "UpdateAllSensors" to see how it it used
			SmVR_InvertMatrix3 (MLocal, Inverse[i]);

			// Reset all local the transformations for the object
			BindObject[i]->SetLocalTransformationMatrixToIdentity();
		}
	return 0;
}
*//////////////////////////////////////////////////////////////////////////////////////////
// ***************************************************
//  void SmVR_CIsotrack::SetExtraData(void *data)
//
//
// ***************************************************
void SmVR_CIsotrack::SetExtraData(void *data)
{
	printf("'SetExtraData' method from NEW class....\n"); 
}

// ***************************************************
//
// ***************************************************
SmVR_CIsotrack::~SmVR_CIsotrack(void)
{
	//CloseSerialPort();
}


