// SmVR_CGeometricObject.h


#ifndef _SmVR_CGeometricObject_
#define _SmVR_CGeometricObject_

#include "SmVR_CPoint.h"

#include "SmVR_Matrix.h"

#include "3DLoaders/SmVR_CObjFromFile.h"
#include "SmVR_CommonTypes.h"

class CGeometry
{
	public:
		SmVR_CPoint *GetVertices();
		unsigned long int nVertex;

	private:
		SmVR_CPoint *VertexList;

};

class SmVR_CGeometricObject
{
	public:
		// Constructors
		SmVR_CGeometricObject(char const *name);
		SmVR_CGeometricObject(char const *name, FunctionPointer *f);
		SmVR_CGeometricObject(char const *name, SmVR_CObjFromFile *obj);

		//Destructor
		//~SmVR_CGeometricObject();


		// Miscelaneous methods
		static void TestNullSceneGraph(SmVR_CGeometricObject *o); // class method
		void SetVisibility(int status) { visible = status; };
		void SetName(const char *name);
		char *GetName();
		char *GetFilename();
		int IsVisible(void);

		// Direct matrix manipulation methods
		static void UpdateAllGlobalTransformationMatrix(); // class method
		void UpdateAllMatrices(void);
		void CalculateGlobalTransformationMatrix(void);

		// Geometric transformation methods
		void TranslateBy(float tx, float ty, float tz);
		void TranslateBy(SmVR_CPoint p);
		void TranslateByOnOBJCS(float tx, float ty, float tz, SmVR_CGeometricObject *);
		void TranslateByOnOBJCS(SmVR_CPoint p, SmVR_CGeometricObject *OCS);

		void TranslateToOnOBJCS(float tx, float ty, float tz, SmVR_CGeometricObject *OCS);
		void TranslateToOnOBJCS(SmVR_CPoint p, SmVR_CGeometricObject *OCS);

		void RotateBy(float angle, float ax, float ay, float az);
		void RotateBy(float ang, SmVR_CPoint p);
		void RotateByOnOBJCS(float angle, float ax, float ay, float az, SmVR_CGeometricObject *);
		void RotateByOnOBJCS(float angle, SmVR_CPoint p, SmVR_CGeometricObject *);

		void ScaleBy(float ex, float ey, float ez);
		void ScaleBy(SmVR_CPoint p);
		void ScaleByOnOBJCS(float ex, float ey, float ez, SmVR_CGeometricObject *OCS);
		void ScaleByOnOBJCS(SmVR_CPoint p, SmVR_CGeometricObject *OCS);

		void TranslateTo(float tx, float ty, float tz);
		void TranslateTo(SmVR_CPoint p);

		//SmVR_CPoint GetPosition();
		//void GetPosition(float *ex, float *ey, float *ez);

		// debugging
		void Print(void);

		// Scene graph manipulation methods
		BOOL AddChild(SmVR_CGeometricObject *o);
		BOOL Remove(SmVR_CGeometricObject *NewParentForChildren);

		SmVR_CGeometricObject *GetFirstChild();
		SmVR_CGeometricObject *GetParent();
		SmVR_CGeometricObject *GetBrother();
		SmVR_CGeometricObject *GetPreviousBrother();
		SmVR_CGeometricObject *FindChildObject(char *name);

		// WCS conversion methods
		void GetPointInOCS(SmVR_CPoint in, SmVR_CPoint *out, SmVR_CGeometricObject *OCS);

		// Render related methods
		void Render(void);
		void SetRenderFunctionData(void *d);
		int GetRenderFunctionReturnValue(void);

		void SetRenderFunction(FunctionPointer *f);
		FunctionPointer *GetRenderFunction(void);
		void CopyRenderFunctionFromObject(SmVR_CGeometricObject *o);


		// Matrices methods
		void GetLocalTransformationMatrix(SmVR_Matrix m);
		void GetGlobalTransformationMatrix(SmVR_Matrix m);
		void SetLocalTransformationMatrixToIdentity(void);
		void SetLocalTransformationMatrix(SmVR_Matrix m);
		
		void SetRotationMatrixToIdent(void);

		SmVR_CObjFromFile* GetObjectFromFile(void);

		int Collided(SmVR_CGeometricObject *o, int mode);
		int Collided(SmVR_CGeometricObject *o);
		

	private:

		void Init(const char *name);
		char *name;
		SmVR_CGeometricObject *parent;
		SmVR_CGeometricObject *child;
		SmVR_CGeometricObject *brother, *prevBrother;

		SmVR_Matrix TransformationMatrix, GlobalTransformationMatrix;

		FunctionPointer *RenderFunction;	// Drawing function
		void *RenderFunctionData;
		int FunctionReturnValue;
		char *Filename;				// Object file

		CGeometry *GeometryData;
		SmVR_CObjFromFile *ObjFromFile;

		int visible;

		SmVR_CGeometricObject *FindObjectAux(char *name);

};

#endif
