// ************************************************
// SmVR_Scene.cpp
//		Scene reader
//
//  Author: Marcio Serolli Pinho
//	Email: pinho@inf.pucrs.br		 
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2002.
// ************************************************

#include "SmVR_Scene.h"
#include "3DLoaders/OBJ/SmVR_COBJLoader.h"



// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
int SmVR_Scene::OpenSceneFile(char *name)
{
	strcpy (filename, name);
	arq = fopen(name, "r");
	if (arq == NULL)
	{
		return 0;
	}
	else return 1;

}
// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
char *SmVR_Scene::ReadLine(char *l)
{

	char *aux, str[MAX_STR_SIZE];

	aux = fgets(str, MAX_STR_SIZE, arq);

	if (aux)
	{	// remove the newline token at the end of string
		str[strlen(str)-1] = '\0';

		// remove spaces and tabs from the beginning of str
		while ((*aux == ' ') || (*aux == '\t'))
			aux++;
		// update the parameter
		strcpy(l,aux);
	}

	//printf("Line [%d] read.\n", NoOfLine++);
	return aux;

}
// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
int ParseLineOfRXYZ (char *l, float *ang, SmVR_CPoint *point)
{
	char str[MAX_STR_SIZE];

	sscanf(l,"%s%f%f%f%f", str, ang, &point->X, &point->Y, &point->Z);

	if (!strcmp(str, "ROT"))
	   return ROTATION_TOKEN;

	
	return WRONG_TOKEN;

}
// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
int ParseLineOfXYZ(char *l, SmVR_CPoint *point)
{
	char str[MAX_STR_SIZE];

	sscanf(l,"%s%f%f%f", str, &point->X, &point->Y,&point->Z);
	if (!strcmp(str, "POS"))
	   return POSITION_TOKEN;

	if (!strcmp(str, "ROT"))
	   return ROTATION_TOKEN;

	if (!strcmp(str, "ESC"))
	   return SCALE_TOKEN;
	
	return WRONG_TOKEN;
}
// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
int ParseLineOfNAME(char *l, char *name)
{
	char str[MAX_STR_SIZE];

	sscanf(l,"%s%s", str, name);
	if (!strcmp(str, "NAME"))
		return NAME_TOKEN;

	if (!strcmp(str, "FILE"))
		return FILE_TOKEN;

	return WRONG_TOKEN;
}

// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
int SmVR_Scene::GetNextObject(char *name, char *file3D, SmVR_CPoint *pos, SmVR_CPoint *esc, SmVR_CPoint *ori, float *ang)
{
	char lineoftext[MAX_STR_SIZE];
	char *ret;

	//fscanf(arq, "%s%s", aux, name

	// name
	do
	{
		ret = ReadLine(lineoftext);
		if (!ret) 
   			return END_OF_SCENE;
	} while (lineoftext[0] == '#' || lineoftext[0] == '\0');



	// check to see if there is a FAMILY token
	if (lineoftext[0] == '{')
		return START_OF_FAMILY;

	if (lineoftext[0] == '}')
		return END_OF_FAMILY;

	// =============
	// Object name
	// =============
//	ret = ReadLine(lineoftext);
	if (ParseLineOfNAME(lineoftext, name) != NAME_TOKEN)
	{
		printf("Object NAME Expected !\n");
		exit(1);	
	}

	// =============
	// Object file
	// =============
	ret = ReadLine(lineoftext);
	if (ParseLineOfNAME(lineoftext, file3D) != FILE_TOKEN)
	{
		printf("Object FILE Expected !\n");
		exit(1);	
	}

	if (!ret)
		return END_OF_SCENE;

	// =============
	// translation
	// =============
	ret = ReadLine(lineoftext);

	if (!ret)
		return END_OF_SCENE;
	else {
	// save the translation
		if (ParseLineOfXYZ(lineoftext, pos) != POSITION_TOKEN)
		{
			printf("Position Expected !\n");
			exit(1);	
		}
	}

	// =============
	// rotation
	// =============
	ret = ReadLine(lineoftext);

	if (!ret)
		return END_OF_SCENE;
	else {
	// save the rotation
		if (ParseLineOfRXYZ(lineoftext, ang, ori) != ROTATION_TOKEN)
		{
			printf("Rotation Expected !\n");
			exit(1);	
		}
	}

	// =============
	// scale
	// =============
	ret = ReadLine(lineoftext);
	if (!ret)
		return END_OF_SCENE;
		else {
	// save the scale
		if (ParseLineOfXYZ(lineoftext, esc) != SCALE_TOKEN)
		{
			printf("Scale Expected !\n");
			exit(1);	
		}
	}

	return END_OF_OBJECT;
}
// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
int SmVR_Scene::ReadObject(SmVR_CGeometricObject *parent, SmVR_CPoint *pos, SmVR_CPoint *esc, SmVR_CPoint *ori, float *ang )
{
	int ret;
	char name[100], 
		 file3D[MAX_STR_SIZE];
	SmVR_CGeometricObject *obj;

	SmVR_CObjFromFile *ObjectFromFile; 


	ret = GetNextObject (name, file3D, pos, esc, ori, ang);
	
	if (ret == END_OF_OBJECT)
	{
		ObjectFromFile = new SmVR_COBJLoader( );	
		if (!ObjectFromFile->Load(file3D))
		{
			printf ("SmallVR SCENE - Error reading file '%s'\n",file3D);
			printf ("Program Aborted.\n");
			exit(1);
		}
		obj = new SmVR_CGeometricObject(name, ObjectFromFile);
		parent->AddChild (obj);
	}
	return ret;
}


// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
void SmVR_Scene::ReadFamily(SmVR_CGeometricObject *parent)
{
	SmVR_CGeometricObject *child;
	SmVR_CPoint pos, esc, ori;
	float angle;

	int ret, prev_ret=-1;

	do
	{
		ret = ReadObject(parent, &pos, &esc, &ori, &angle);

		if (ret == END_OF_SCENE) // Done with the scen file !
			return;	
		child = parent->GetFirstChild();
		if (ret == START_OF_FAMILY)
		{
			// if it starts a new family,
			// the last object inserted is its parent
			ReadFamily(child);
		}
		if ((ret == END_OF_FAMILY) || (ret == END_OF_OBJECT))
		{
			// the parent's transformations are applied 
			// AFTER all children has been added to the parent
			child->TranslateBy(pos);
			child->RotateBy(angle, ori);
			child->ScaleBy(esc);
		}
	} while (ret != END_OF_SCENE);
}

// ************************************************
//	Method: 
//	Description: 
//
// ************************************************
SmVR_Scene::SmVR_Scene(char *name, SmVR_CGeometricObject *root)
{

	NoOfLine = 0;
	filename = strdup(name); 
	arq = fopen(filename,"rt");
	if (arq == NULL)
	{
		printf("Scene file does not exists !\n");
		exit(1);
	}
	printf("Parsing '%s' scene file...\n", name);
	ReadFamily(root);
	printf("Done !\n");
}


