//*************************************************
//
// StereoIglasses.cpp
//		Testes para gerao de imagens 
//	estereoscpicas.
//      Este programa assum que se esteja usando o
//  culos Iglasses	  
// 		O mtodo de posicionamento dos oilhos do 
//	observador usado neste fonte foi criado com 
//	base no site de Paul Bourke, acessvel pela URL
//  http://astronomy.swin.edu.au/~pbourke/opengl/stereogl
//
//  Instrues de utilizao 
//
//		- Colocar o include 
//		#include <Stereo/StereoIglasses.h>
//
//		- Acertar o resoluo de vdeo 
//				void ChangeResolution(int mode);
//		O parmetro "mode" pode ser
//
//		#define VM_640x480 1
//		#define VM_800x600 2
//		#define VM_1024x768 3
//		#define VM_1400x1050 4
//
//
//		- Inicializar o Stencil Buffer
//
//			glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL); 
//
//		- Colocar na callback de reshape a chamada da 
//
// 			StereoIglasses_Init(Largura, Altura);
//
//		- Colocar na callback de display a chamada das rotinas de 
//	posicionamento do observador
//
//			StereoIglasses_CameraPos(User.X, User.Y, User.Z, Target.X, Target.Y, Target.Z);
//
//		- Colocar na callback de display a chamada das rotinas de 
//	posicionamento dos olhos seguidas da rotina de rendering
//
//
//Direito	
//	glPushMatrix();
//		StereoIglasses_PosicOlhoDireito();
//		StereoIglasses_HabilitaOlhoDireito();
//		ExibeCena(); // rotina que exibe o cenrio
//	glPopMatrix();
//
// Esquerdo
//	glPushMatrix();
//		StereoIglasses_PosicOlhoEsquerdo();
//		StereoIglasses_HabilitaOlhoEsquerdo();
//		ExibeCena(); // rotina que exibe o cenrio
//	glPopMatrix();
//
//
//  =============================================
//  Author: Mrcio Serolli Pinho
//  E-Mail: pinho@inf.pucrs.br
//  Version: 1.0
//  Date: April, 4th, 2003.
// 
// *************************************************/

#include <math.h>
#include <windows.h>
#include <stdio.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glut.h>

#include "StereoIglasses.h"

CAMERA camera;   

#define PI 3.141592653589793238462643
#define DTOR            0.0174532925
#define RTOD            57.2957795

#define CROSSPROD(p1,p2,p3) \
   p3.x = p1.y*p2.z - p1.z*p2.y; \
   p3.y = p1.z*p2.x - p1.x*p2.z; \
   p3.z = p1.x*p2.y - p1.y*p2.x


// **********************************************************************
//  void DrawLine(int x1, int y1, int x2, int y2)
//
//
// **********************************************************************
void DrawLine(int x1, int y1, int x2, int y2)
{
	glVertex2f(x1,y1);
	glVertex2f(x2,y2);
}

// **********************************************************************
//  void DesenhaFundo()
//      Inicializa o stencil com 1 nas linhas pares 
//
// **********************************************************************
void DesenhaFundo(int Largura, int Altura )
{

	glMatrixMode(GL_PROJECTION); // 
	glLoadIdentity ();
	gluOrtho2D(0, Largura, 0, Altura);

	glMatrixMode(GL_MODELVIEW);

	glBegin(GL_LINES);
		for (int y= 0; y < Altura; y += 2)
		{
			DrawLine(0,y, Largura, y);
		}
	glEnd();
	// volta  matriz de objetos
	glMatrixMode(GL_MODELVIEW);

}

// **********************************************************************
//  void InitStencil()
//     
//
// **********************************************************************
void InitStencil(int Largura, int Altura )
{
    // Habilita o uso do Stencil 
	glEnable(GL_STENCIL_TEST);

	// Define que "0" ser usado para limpar o Stencil 
	glClearStencil(0);
	// limpa o Stencil
	glClear(GL_STENCIL_BUFFER_BIT);

	glStencilFunc(GL_ALWAYS, 1, 1); 
	glStencilOp(GL_REPLACE,GL_REPLACE,GL_REPLACE);

	DesenhaFundo(Largura,Altura);

}
// **********************************************************************
//  void StereoIglasses_HabilitaOlhoEsquerdo()
//
//
// **********************************************************************
void StereoIglasses_HabilitaOlhoEsquerdo()
{
	glStencilFunc(GL_NOTEQUAL, 1, 1);   
	glStencilOp(GL_KEEP,GL_KEEP, GL_KEEP);
}
// **********************************************************************
//  void HabilitaDesenhoNoStencil()
//
//
// **********************************************************************
void StereoIglasses_HabilitaOlhoDireito()
{
	glStencilFunc(GL_EQUAL, 1, 1);   
	glStencilOp(GL_KEEP,GL_KEEP, GL_KEEP);
}


// **********************************************************************
//  void Normalise(XYZ *p)
//
//
// **********************************************************************
void Normalise(XYZ *p)
{
   double length;

   length = sqrt(p->x * p->x + p->y * p->y + p->z * p->z);
   if (length != 0) {
      p->x /= length;
      p->y /= length;
      p->z /= length;
   } else {
      p->x = 0;
      p->y = 0;
      p->z = 0;
   }
}

// **********************************************************************
//  void Normalise(XYZ *p)
//
//
// **********************************************************************
XYZ CalcNormal(XYZ p,XYZ p1,XYZ p2)
{
   XYZ n,pa,pb;

   pa.x = p1.x - p.x;
   pa.y = p1.y - p.y;
   pa.z = p1.z - p.z;
   pb.x = p2.x - p.x;
   pb.y = p2.y - p.y;
   pb.z = p2.z - p.z;
   Normalise(&pa);
   Normalise(&pb);
  
   n.x = pa.y * pb.z - pa.z * pb.y;
   n.y = pa.z * pb.x - pa.x * pb.z;
   n.z = pa.x * pb.y - pa.y * pb.x;
   Normalise(&n);

   return(n);
}

// **********************************************************************
//  void StereoIglasses_PosicOlhoEsquerdo()
//
//
// **********************************************************************
void StereoIglasses_PosicOlhoEsquerdo()
{
	XYZ r;
	double ratio,radians,wd2,ndfl;
	double left,right,top,bottom,
		   znear=0.1,zfar=10000;

   
   /* Clip to avoid extreme stereo */
    znear = camera.focallength / 5;

	/* Misc stuff */
	ratio  = camera.screenwidth / (double)camera.screenheight;
	radians = DTOR * camera.aperture / 2;
	wd2     = znear * tan(radians);
	ndfl    = znear / camera.focallength;

	/* Derive the two eye positions */
	CROSSPROD(camera.vd,camera.vu,r);
	Normalise(&r);
	r.x *= camera.eyesep / 2.0;
	r.y *= camera.eyesep / 2.0;
	r.z *= camera.eyesep / 2.0;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	left  = - ratio * wd2 + 0.5 * camera.eyesep * ndfl;
	right =   ratio * wd2 + 0.5 * camera.eyesep * ndfl;
	top    =   wd2;
	bottom = - wd2;
	glFrustum(left,right,bottom,top,znear,zfar);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	gluLookAt(camera.vp.x - r.x,camera.vp.y - r.y,camera.vp.z - r.z,
			  camera.vp.x - r.x + camera.vd.x,
			  camera.vp.y - r.y + camera.vd.y,
		      camera.vp.z - r.z + camera.vd.z,
			0,1,0);


}
// **********************************************************************
//  void StereoIglasses_PosicOlhoDireito()
//
//
// **********************************************************************
void StereoIglasses_PosicOlhoDireito()
{
	XYZ r;
	double ratio,radians,wd2,ndfl;
	double left,right,top,bottom,
		   znear=0.1,zfar=10000;

	/* Clip to avoid extreme stereo */	
	znear = camera.focallength / 5;

	/* Misc stuff */
	ratio  = camera.screenwidth / (double)camera.screenheight;
	radians = DTOR * camera.aperture / 2;
	wd2     = znear * tan(radians);
	ndfl    = znear / camera.focallength;

	/* Derive the two eye positions */
	CROSSPROD(camera.vd,camera.vu,r);
	Normalise(&r);
	r.x *= camera.eyesep / 2.0;
	r.y *= camera.eyesep / 2.0;
	r.z *= camera.eyesep / 2.0;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	left  = - ratio * wd2 - 0.5 * camera.eyesep * ndfl;
	right =   ratio * wd2 - 0.5 * camera.eyesep * ndfl;
	top    =   wd2;
	bottom = - wd2;
	glFrustum(left,right,bottom,top,znear,zfar);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();


	gluLookAt(camera.vp.x + r.x,camera.vp.y + r.y,camera.vp.z + r.z,
			camera.vp.x + r.x + camera.vd.x,
			camera.vp.y + r.y + camera.vd.y,
			camera.vp.z + r.z + camera.vd.z,
			0,1,0);

}

// **********************************************************************
// void StereoIglasses_CameraPos()
//
//
// **********************************************************************
void StereoIglasses_CameraPos(float px, float py, float pz, float ax, float ay, float az)
{

	camera.vp.x = px;
	camera.vp.y = py;
	camera.vp.z = pz;

	camera.vd.x = ax - px;
	camera.vd.y = ay - py;
	camera.vd.z = az - pz;

}
// **********************************************************************
//
//
//
// **********************************************************************
void StereoIglasses_CameraHome(int Largura, int Altura)
{
	
	//XYZ origin = {0,0,550};
	//XYZ origin = {0,0,10};
/*
	camera.aperture = 60;
	camera.focallength = 40;
	camera.eyesep = camera.focallength / 20;
	camera.pr = origin;
	camera.vp = origin;

	camera.vd = origin; 
	camera.vd.z = -1;

	camera.vu.x = 0;  
	camera.vu.y = 1; 
	camera.vu.z = 0;

	camera.screenheight = Altura;
	camera.screenwidth = Largura;
*/
}

// **********************************************************************
//
//
//
// **********************************************************************
void StereoIglasses_Init(int Largura, int Altura )
{
	InitStencil(Largura, Altura);
	//StereoIglasses_CameraHome(Largura, Altura);
		camera.aperture = 60;

	camera.focallength = 40;
	camera.eyesep = camera.focallength / 20;
//	camera.pr = origin;
//	camera.vp = origin;

//	camera.vd = origin; 
//	camera.vd.z = -1;

	camera.vu.x = 0;  
	camera.vu.y = 1; 
	camera.vu.z = 0;

	camera.screenheight = Altura;
	camera.screenwidth = Largura;


	//StereoIglasses_CameraPos();
}

// ************************************************
//   void ChangeResolution(int vmode)
//
// ************************************************
void ChangeResolution(int vmode)
{
	DEVMODE dm;
	int ret;
	
	
	dm.dmSize = sizeof(DEVMODE);
	
	
	EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dm);
	
	
	switch (vmode)
	{ 
	case VM_640x480:
		dm.dmPelsWidth = 640;
		dm.dmPelsHeight = 480;
		break;
	case VM_800x600:
		dm.dmPelsWidth = 800;
		dm.dmPelsHeight = 600;
		break;
	case VM_1024x768:
		dm.dmPelsWidth = 1024;
		dm.dmPelsHeight = 768;
		break;
	case VM_1400x1050:
		dm.dmPelsWidth = 1400;
		dm.dmPelsHeight = 1050;
		break;
	default:
		printf("Invalid video mode/n");
		return;
		break;
	}
	
	ret = ChangeDisplaySettings(&dm, CDS_FULLSCREEN | CDS_TEST);
	switch (ret)
	{
	case DISP_CHANGE_SUCCESSFUL: 
		printf("The settings change was successful/n");
		ret = ChangeDisplaySettings(&dm, CDS_FULLSCREEN);//CDS_UPDATEREGISTRY);
		switch(ret)
		{
		case DISP_CHANGE_SUCCESSFUL: 
			printf("Resolution successfully changed!/n");
			break;
		case DISP_CHANGE_RESTART:
			printf("A reboot is necessary before the changes will take effect");
			break;
		default: printf ("Unable to change resolution!");
		}
		break;
		case DISP_CHANGE_RESTART: 
			printf("The computer must be restarted in order for the graphics mode to work./n");
			break;
		case DISP_CHANGE_BADFLAGS:
			printf("An invalid set of flags was passed in./n");
			break;
		case DISP_CHANGE_BADPARAM: 
			printf("An invalid parameter was passed in./n");
			break;
		case DISP_CHANGE_FAILED:
			printf("The display driver failed the specified graphics mode./n");
			break;
		case DISP_CHANGE_BADMODE:
			printf("The graphics mode is not supported./n"); 
			break;
		case DISP_CHANGE_NOTUPDATED: 
			printf("Unable to write settings to the registry./n");
			break;
			
	}
}

// ************************************************
//   void removeBorder (void)
//
// ************************************************
void RemoveBorder (void) {
	static int IS_FULL_NOW = FALSE;
	double error;

	LONG style;
	static LONG oldStyle;

	HWND hwnd;

	//changeResolution (RES_640x480);
	hwnd = GetActiveWindow();
	error = GetLastError();

	if (error)
		printf ("0 - Erro: %f.\n", error);

	if (IS_FULL_NOW) {
		style = oldStyle;
		IS_FULL_NOW = TRUE;
	}
	else {
		oldStyle = GetWindowLong (hwnd, GWL_STYLE);
		style = WS_POPUP | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN ;
		//style = WS_VISIBLE ;

		IS_FULL_NOW = FALSE;
	}

	SetWindowLong (hwnd, GWL_STYLE, style);
}


