/*****************************************************
 **
 **  Memory Client Test
 **
 **  File: MemoryClientTest.cpp
 **  Author: Rafael S. Garcia.
 **  Mail: rafael_sgarcia@terra.com.br
 **  Version: 1.0
 **
 **  Last Update: 27.January.2003.
 **
 *****************************************************/
#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <string.h>

#include <gl\gl.h>
#include <gl\glu.h>
#include "glut.h"

#include "RemoteMemory.h"	




float ratio;
float viewAngle = 100;

float observerX = 0;

double xRotate = 0.0;
double yRotate = 0.0;
double zRotate = 0.0;

double inc[3];
double xInc = 0.5;
double yInc = 0.5;
double zInc = 0.5;





/************************************************
 ** setLight ( void )
 ************************************************/
void setLight ( void ) {
	GLfloat lightAmbient[]   = { 0.24725f,  0.1995f,   0.07f } ;
	GLfloat lightDiffuse[]   = { 0.75164f,  0.60648f,  0.22648f,  1.0f };
	GLfloat lightSpecular[]  = { 0.626281f, 0.555802f, 0.366065f, 1.0f };
	GLfloat positionLight0[] = { 3.0f,		3.0f,      0.0f,      1.0f };
	GLfloat positionLight1[] = {-3.0f,	   -3.0f,      0.0f,      1.0f };
	GLfloat specularity[]    = { 1.0f,      1.0f,      1.0f,      1.0f };

	// Habilita o uso de iluminao
	glEnable ( GL_LIGHTING );

	// Ativa o uso da luz ambiente
	glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, lightAmbient );

	// Define os parametros da Luz nmero Zero
	glLightfv ( GL_LIGHT0, GL_AMBIENT,  lightAmbient );
	glLightfv ( GL_LIGHT0, GL_DIFFUSE,  lightDiffuse );
	glLightfv ( GL_LIGHT0, GL_SPECULAR, lightSpecular );
	glLightfv ( GL_LIGHT0, GL_POSITION, positionLight1 );
	glEnable  ( GL_LIGHT0 );

	// Ativa o "Color Tracking"
	glEnable ( GL_COLOR_MATERIAL );

	// Define a reflectancia do material
	glMaterialfv ( GL_FRONT,GL_SPECULAR, specularity );

	// Define a concentrao do brilho.
	// Quanto maior o valor do Segundo parametro, mais
	// concentrado ser o brilho. (Valores vlidos: de 0 a 128)
	glMateriali ( GL_FRONT, GL_SHININESS, 20 );
}




/************************************************
 ** setObserver ( void )
 ************************************************/
void setObserver ( void ) {
	glMatrixMode ( GL_MODELVIEW );
	glLoadIdentity();
	gluLookAt ( 
		observerX, 0, 5,
		observerX, 0, 0,
		0.0f, 1.0f, 0.0f
	);
}




/************************************************
 ** drawCube ( void )
 ************************************************/
void drawCube ( void ) {
	glBegin ( GL_QUADS );
		// Front Face.
		glColor3f ( 1, 1, 0 );
		glVertex3f ( -1.0f, -1.0f,  1.0f );
		glVertex3f (  1.0f, -1.0f,  1.0f );
		glVertex3f (  1.0f,  1.0f,  1.0f );
		glVertex3f ( -1.0f,  1.0f,  1.0f );

		// Back Face
		glColor3f ( 0, 0, 1 );
		glVertex3f ( -1.0f, -1.0f, -1.0f );
		glVertex3f ( -1.0f,  1.0f, -1.0f );
		glVertex3f (  1.0f,  1.0f, -1.0f );
		glVertex3f (  1.0f, -1.0f, -1.0f );

		// Top Face
		glColor3f ( 1, 0, 1 );
		glVertex3f ( -1.0f,  1.0f, -1.0f );
		glVertex3f ( -1.0f,  1.0f,  1.0f );
		glVertex3f (  1.0f,  1.0f,  1.0f );
		glVertex3f (  1.0f,  1.0f, -1.0f );

		// Bottom Face
		glColor3f ( 1, 1, 1 );
		glVertex3f ( -1.0f, -1.0f, -1.0f );
		glVertex3f (  1.0f, -1.0f, -1.0f );
		glVertex3f (  1.0f, -1.0f,  1.0f );
		glVertex3f ( -1.0f, -1.0f,  1.0f );

		// Right face
		glColor3f ( 0, 0.4, 1 );
		glVertex3f ( 1.0f, -1.0f, -1.0f );
		glVertex3f ( 1.0f,  1.0f, -1.0f );
		glVertex3f ( 1.0f,  1.0f,  1.0f );
		glVertex3f ( 1.0f, -1.0f,  1.0f );

		// Left Face
		glColor3f ( 1, 0, 0 );
		glVertex3f ( -1.0f, -1.0f, -1.0f );
		glVertex3f ( -1.0f, -1.0f,  1.0f );
		glVertex3f ( -1.0f,  1.0f,  1.0f );
		glVertex3f ( -1.0f,  1.0f, -1.0f );
	glEnd();
}




/************************************************
 ** readRotate ( void )
 ************************************************/
void readRotate ( void ) {
	ReadFromRemoteMemoryIP ( inc, 50 * sizeof(double), sizeof(double) * 3 );
	xInc = inc[0];
	yInc = inc[1];
	zInc = inc[2];
}




/************************************************
 ** displayFunction ( void )
 ************************************************/
void displayFunction ( void ) {
	glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	setLight();
	setObserver();
	readRotate();

	xRotate += xInc;
	yRotate += yInc;
	zRotate += zInc;

	glLoadIdentity();
	glPushMatrix();
		glTranslatef ( -1.0, 0.0, -5.0 );
		glRotatef ( xRotate, 1.0, 0.0, 0.0 );
		glRotatef ( yRotate, 0.0, 1.0, 0.0 );
		glRotatef ( zRotate, 0.0, 0.0, 1.0 );
		drawCube();
	glPopMatrix();
	glPushMatrix();
		glTranslatef ( +1.0f, 0.0, -5.0 );
		glRotatef ( xRotate, 1.0, 0.0, 0.0 );
		glRotatef ( yRotate, 0.0, 1.0, 0.0 );
		glRotatef ( -zRotate, 0.0, 0.0, 1.0 );
		drawCube();
	glPopMatrix();

	glutSwapBuffers();
}




/************************************************
 ** setViewFrustum ( void )
 ************************************************/
void setViewFrustum ( void ) {
	float Near, left, right, bottom, top;
	float viewAngleRad;

	glMatrixMode ( GL_PROJECTION );
	glLoadIdentity();

	viewAngleRad = viewAngle * 3.14f/180;

	Near = 1;
	left = -Near * tan ( viewAngleRad / 2 );
	right = -left;

	bottom = left * ratio;
	top = -left * ratio;

	glFrustum ( left, right, bottom, top, Near, 100 );
	glMatrixMode ( GL_MODELVIEW );

	setObserver();
}




/************************************************
 ** reshapeFunction ( int, int )
 ************************************************/
void reshapeFunction ( int w, int h ) {
	if ( h == 0 )
		h = 1;

	ratio = 1.0f * h / w;

	// Reset the coordinate system before modifying
	glMatrixMode ( GL_PROJECTION );
	glLoadIdentity();
	
	// Set the viewport to be the entire window
    glViewport ( 0, 0, w, h );

	// Set the clipping volume
	setViewFrustum();
}




/************************************************
 ** refreshAll ( void )
 ************************************************/
void refreshAll ( void ) {
	setViewFrustum();
	glutPostRedisplay();
}




/************************************************
 ** keyboardFunction ( unsigned char, int, int )
 ************************************************/
void keyboardFunction ( unsigned char key, int x, int y ) {
	switch ( key )  {
		case 27:        
			exit (0); 
			break;      

		case 'a':
			yInc -= 0.5;
			WriteOnRemoteMemoryUDP ( &yInc, 51 * sizeof ( double ), sizeof ( double ) );
			break;

		case 'd':
			yInc += 0.5;
			WriteOnRemoteMemoryUDP ( &yInc, 51 * sizeof ( double ), sizeof ( double ) );
			break;

		case 'w':
			xInc -= 0.5;
			WriteOnRemoteMemoryUDP ( &xInc, 50 * sizeof ( double ), sizeof ( double ) );
			break;

		case 's':
			xInc += 0.5;
			WriteOnRemoteMemoryUDP ( &xInc, 50 * sizeof ( double ), sizeof ( double ) );
			break;

		case 'z':
			zInc -= 0.5;
			WriteOnRemoteMemoryUDP ( &zInc, 52 * sizeof ( double ), sizeof ( double ) );
			break;

		case 'x':
			zInc += 0.5;
			WriteOnRemoteMemoryUDP ( &zInc, 52 * sizeof ( double ), sizeof ( double ) );
			break;

		default:
		  break;
	}
}




/************************************************
 ** initCallBack ( void )
 ************************************************/
void initCallbacks ( void ) {
	glutDisplayFunc  ( displayFunction );  
	glutReshapeFunc  ( reshapeFunction );
	glutKeyboardFunc ( keyboardFunction );
	glutIdleFunc     ( refreshAll );
}




/************************************************
 ** init ( void )
 ************************************************/
void init ( void ) {
	glShadeModel ( GL_SMOOTH );
	glClearColor ( 1.0f, 1.0f, 1.0f, 0.0f );
	glEnable ( GL_DEPTH_TEST );
}




// **********************************************************************
//  void main ( int argc, char** argv )
// **********************************************************************
void main ( int argc, char** argv ) {
	glutInitDisplayMode ( GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA ); 

	if (argc < 2) {
		printf("Command line parameters: ServerIP\n");
		exit(1);
	}

	char serverIP[30];
	strcpy ( serverIP, argv[1] );
 	glutCreateWindow ( "Memory Client Test" ); 

	int result = ConnectToRemoteMemory ( serverIP );

	if ( !result ) {
		printf ( "Unable to connect to Memory Server at %s.\n\n", serverIP );
		exit (0);
	}
	inc[0] = xInc;
	inc[1] = yInc;
	inc[2] = zInc;
	WriteOnRemoteMemoryIP ( inc, 50 * sizeof(double), sizeof(double) * 3 );

	printf ( "Use the keys to increment or decrement the rotate velocity:\n" );
	printf ( "w - s: X Rotate.\n" );
	printf ( "a - d: Y Rotate.\n" );
	printf ( "z - x: Z Rotate.\n" );	

	init ();
	initCallbacks();

	glutMainLoop ();
}
