// ************************************************
// LoadObj.cpp
//	Sample file for SmallVR
//  Authors: Marcio Sarroglia Pinho,
//	Email: pinho@pucrs.br
//	Homepages:
//		http://grv.inf.pucrs.br/
//      http://www.inf.pucrs.br/pinho
//
//	Porto Alegre, Brazil, 2011.
// ************************************************

#ifdef WIN32
#include <windows.h>
#endif

#include <iostream>
using namespace std;

#include <GL\gl.h>

// SmallVR header file
#include "SmVR.h"

#include "OBJ\SmVR_COBJLoader.h"
float WindowRatio;

// ******************************
// User stuff
//     see "PosicUser" function
//  on how to use these variables
// ******************************
// User position
SmVR_CPoint	User(0,2,-2);
// Target position
SmVR_CPoint	Target(0,0,0);

// Root Object to represent the entire scene
SmVR_CGeometricObject *RootObject;

// Viewing angle in degrees
float ViewAngle = 90;

// ******************************
// Light stuff
// ******************************
GLint   especMaterial = 60;
GLfloat posicaoLuz1   [4] = { 0.0, 1.0, 1.0, 1.0};
GLfloat luzAmbiente   [4] = { 0.5,  0.5, 0.5, 1.0};
GLfloat luzDifusa     [4] = { 0.7,  0.7, 0.7, 1.0}; // "cor" do objeto
GLfloat luzEspecular  [4] = { 0.4,  0.4, 0.4, 1.0}; // brilho do objeto
GLfloat especularidade[4] = { 0.17,  0.17, 0.17, 1.0}; // capacidade de brilho do material


// **********************************************************************
//  void PosicUser()
//
//
// **********************************************************************
void PosicUser()
{
    // Set the clipping volume
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(ViewAngle,WindowRatio,0.01,2000);

    // Set the user Position
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(User.X,User.Y, User.Z,
              Target.X,Target.Y, Target.Z,
              0.0f,1.0f,0.0f);
}
// **********************************************************************
//  void reshape( int w, int h )
//		trata o redimensionamento da janela OpenGL
//
// **********************************************************************
void reshape( int w, int h )
{
    // Prevent a divide by zero, when window is too short
    // (you cant make a window of zero width).
    if(h == 0)
        h = 1;

    WindowRatio = 1.0f * w / h;
    // Reset the coordinate system before modifying
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // Set the viewport to be the entire window
    glViewport(0, 0, w, h);

    PosicUser();

}
// **********************************************************************
//  void display( void )
//
//
// **********************************************************************
void display( void )
{

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    PosicUser();

    glMatrixMode(GL_MODELVIEW);

    RootObject->Render();

    glutSwapBuffers();
}

// **********************************************************************
//  void specialFunc (int key, int x, int y)
//
// **********************************************************************
void specialFunc (int key, int x, int y)
{
    SmVR_CGeometricObject *temp;
    temp = RootObject->FindChildObject("Objeto Teste");

    switch (key)
    {
    case GLUT_KEY_PAGE_UP:
        break;

    case GLUT_KEY_PAGE_DOWN:
        break;

    case GLUT_KEY_LEFT:
        temp->RotateBy(-10, 0,1,0);
        glutPostRedisplay(); // fora o redesenho da tela
        break;

    case GLUT_KEY_RIGHT:
        temp->RotateBy(10, 0,1,0);
        glutPostRedisplay(); // fora o redesenho da tela
        break;

    case GLUT_KEY_UP:
        temp->RotateBy(10, 1,0,0);
        glutPostRedisplay(); // fora o redesenho da tela
        break;

    case GLUT_KEY_DOWN:
        temp->RotateBy(-10, 1,0,0);
        glutPostRedisplay(); // fora o redesenho da tela
        break;
    }

}

// **********************************************************************
//  void keyboard ( unsigned char key, int x, int y )
//
// **********************************************************************
void keyboard ( unsigned char key, int x, int y )
{
    switch ( key )
    {
    case '+':
        ViewAngle--;
        glutPostRedisplay();
        break;
    case '-':
        ViewAngle++;
        glutPostRedisplay();
        break;
    case 27:
        exit(0);
        break;
    }
}
// **********************************************************************
//  void init(void)
//		Inicializa os parmetros globais de OpenGL
//
// **********************************************************************
void init(void)
{

    glShadeModel(GL_SMOOTH); // Habilita o modelo de tonalizacao de Gouraud
    glClearColor(1.0f, 1.0f, 1.0f, 0.0f); // Cor de fundo de tela

    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

    glMaterialfv(GL_FRONT,GL_SPECULAR, especularidade);  // Define a refletancia do material
    glMateriali(GL_FRONT,GL_SHININESS,especMaterial);    // Define a concentracao do brilho
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, luzAmbiente); // Ativa o uso da luz ambiente

    // Define os parametros da luz de numero 0
    glLightfv(GL_LIGHT0, GL_AMBIENT , luzAmbiente );
    glLightfv(GL_LIGHT0, GL_DIFFUSE , luzDifusa   );
    glLightfv(GL_LIGHT0, GL_SPECULAR, luzEspecular);
    glLightfv(GL_LIGHT0, GL_POSITION, posicaoLuz1 );

    glEnable(GL_COLOR_MATERIAL); // Habilita a definicao da cor do material a partir da cor corrente
    glEnable(GL_LIGHTING);       // Habilita o uso de iluminacao
    glEnable(GL_LIGHT0);         // Habilita a luz de numero 0
    glEnable(GL_DEPTH_TEST);     // Habilita o depth-buffering
    glEnable(GL_NORMALIZE);      // Habilita a normalizacao das normais
    glDisable(GL_CULL_FACE);

}

// **********************************************************************
//  void main ( int argc, char** argv )
//
//
// **********************************************************************
int  main ( int argc, char** argv )
{

    // GLUT initialization
    glutInit            ( &argc, argv );
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);// | GLUT_DOUBLE | GLUT_RGBA );
    glutInitWindowPosition (0,0);
    glutInitWindowSize  ( 700, 500 );
    glutCreateWindow    ( "SmallVR Test - OBJ Loader" );

    // initialize OpenGL parameters
    init ();

    // initialize the library
    RootObject = SmVR_Init(NULL);

    // Cria o objeto SmallVR
    SmVR_CGeometricObject *TestObject;

    // Cria um objeto para carregar o modelo de arquivo
    SmVR_COBJLoader *obj1 = new SmVR_COBJLoader();
    // Executa a carga do objeto

    obj1->Load("..//TestFiles//Objetos3D//plant.obj");

    // Teste com os demais objetos disponveis:
    // abajur.obj carinha01.obj carinha02.obj  carinha03.obj
    // chao.obj  iPad80s.obj  mesa.obj  mesaL.obj poltrona.obj sofa.obj
    // PLANT.obj
    SmVR_CPoint Min,Max;

    obj1->GetBoundaries(&Min, &Max);
    cout << "Minimo:" << Min.X << "  " << Min.Y << "  " << Min.Z << endl;
    cout << "Maximo:" << Max.X << "  " << Max.Y << "  " << Max.Z << endl;

    // Associa o objeto ligo de arquivo com o objeto SmallVR
    // Isto  feito passando o objeto "obj1" na construtora da classe
    // do objeto SmallVR
    TestObject = new SmVR_CGeometricObject("Objeto Teste", obj1);

    // Adiciona o objeto recm criado ao grafo de cena
    RootObject->AddChild(TestObject);
    TestObject->RotateBy(90,0,1,0);

    // GLUT stuff
    glutDisplayFunc ( display );
    glutReshapeFunc ( reshape );
    glutKeyboardFunc ( keyboard );
    glutSpecialFunc ( specialFunc );
    glutIdleFunc ( display );

    glutMainLoop ( );

    return 0;
}

