// ************************************************
// FirstSmallVR.cpp
//	Sample file for SmallVR to test REMOVE Method
//
//  Author: Marcio Serolli Pinho
//	Email: pinho@inf.pucrs.br		 
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2002.
// ************************************************

#ifdef WIN32
#include <windows.h>
#endif

#include <GL\gl.h>

// SmallVR header file
#include "SmVR.h"

float WindowRatio;

SmVR_CPoint	User(0,0,20);

// Root Object to represent the entire scene
SmVR_CGeometricObject *RootObject;

// **********************************************************************
// int DrawTRI(void *p)
//
// **********************************************************************
int DrawTRI(void *p) 
{ 
	// Draw a triangle
	glBegin (GL_TRIANGLES); 
		glColor3f(1,0,0);
		glVertex3f(0,0,0); 
		glColor3f(0,0,1);
		glVertex3f(10,0,0); 
		glColor3f(0,1,0);
		glVertex3f(5,10,0); 
	glEnd(); 

  return 1;
} 
// **********************************************************************
//  void PosicUser()
//
//
// **********************************************************************
void PosicUser()
{
	// Set the clipping volume
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90,WindowRatio,0.01,200);

	// Set the user Position
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(User.X,User.Y, User.Z,
		      User.X,User.Y, User.Z-3,
			  0.0f,1.0f,0.0f);
}
// **********************************************************************
//  void reshape( int w, int h )
//		trata o redimensionamento da janela OpenGL
//
// **********************************************************************
void reshape( int w, int h )
{
	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	WindowRatio = 1.0f * w / h;
	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

	PosicUser();

}
// **********************************************************************
//  void display( void )
//
//
// **********************************************************************
void display( void )
{

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	PosicUser();

	glMatrixMode(GL_MODELVIEW);

	RootObject->Render();
	
	glutSwapBuffers();
}
// **********************************************************************
//  void keyboard ( unsigned char key, int x, int y )   
//
// **********************************************************************
void keyboard ( unsigned char key, int x, int y )  
{
	SmVR_CGeometricObject *temp1;

	switch ( key ) 
	{
	    case 27:   
			exit(0);   
			break;
		case 'a': // left
			temp1 = RootObject->FindChildObject("Parent 1");
			temp1->TranslateBy(-0.2f,0,0);
			break;
		case 's': // right
			temp1 = RootObject->FindChildObject("Parent 1");
			temp1->TranslateBy(0.2f,0,0);
			break;
		case 'k': // left
			temp1 = RootObject->FindChildObject("Parent 2");
			temp1->TranslateBy(-0.2f,0,0);
			break;
		case 'l': // right
			temp1 = RootObject->FindChildObject("Parent 2");
			temp1->TranslateBy(0.2f,0,0);
			break;
		case 'r': // remove 
			temp1 = RootObject->FindChildObject("Parent 1");
			temp1->Remove(RootObject->FindChildObject("Parent 2"));
			break;


	}
}
// **********************************************************************
//  void init(void)
//		Inicializa os parmetros globais de OpenGL
//
// **********************************************************************
void init(void)
{
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f); // Fundo de tela preto
	glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );

	glShadeModel(GL_SMOOTH);						
	glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_DEPTH_TEST);
	glEnable ( GL_CULL_FACE );
}
// **********************************************************************
//  void main ( int argc, char** argv )
//
//
// **********************************************************************
void main ( int argc, char** argv )   
{
	SmVR_CGeometricObject *TestObject;

	// GLUT initialization
	glutInit            ( &argc, argv ); 
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);// | GLUT_DOUBLE | GLUT_RGBA ); 
	glutInitWindowPosition (0,0);
	glutInitWindowSize  ( 700, 500 ); 
	glutCreateWindow    ( "SmallVR Test" ); 
		
	// initialize OpenGL parameters
	init ();

	// initialize the library
	RootObject = SmVR_Init(NULL);

	// Create a new object and Add the object to the scene
	TestObject = new SmVR_CGeometricObject("Parent 1", DrawTRI);
	RootObject->AddChild(TestObject);

	TestObject->AddChild(new SmVR_CGeometricObject("Child 1", DrawTRI));

	(RootObject->FindChildObject("Child 1"))->TranslateBy(0,-1,0.5);
	(RootObject->FindChildObject("Child 1"))->ScaleBy(0.3f,0.3f,1);

	TestObject->AddChild(new SmVR_CGeometricObject("Child 2", DrawTRI));

	(RootObject->FindChildObject("Child 2"))->TranslateBy(6,-1,0.5);
	(RootObject->FindChildObject("Child 2"))->ScaleBy(0.3f,0.3f,1);
	(RootObject->FindChildObject("Child 2"))->RotateBy(180,0,0,1);

	TestObject->AddChild(new SmVR_CGeometricObject("Child 3", DrawTRI));

	(RootObject->FindChildObject("Child 3"))->TranslateBy(8,-1,0.5);
	(RootObject->FindChildObject("Child 3"))->ScaleBy(0.3f,0.3f,1);
	(RootObject->FindChildObject("Child 3"))->RotateBy(90,0,0,1);

	TestObject->TranslateBy(-15,0,0);


	// Create a new object and Add the object to the scene
	TestObject = new SmVR_CGeometricObject("Parent 2", DrawTRI);
	RootObject->AddChild(TestObject);


	// GLUT stuff
	glutDisplayFunc ( display );  
	glutReshapeFunc ( reshape );
	glutKeyboardFunc ( keyboard );
	glutIdleFunc ( display );

	glutMainLoop ( );          

}

