/*****************************************************
 **
 **  Virtual Reality Interactive Environment
 **
 **  File: VRInteractiveEnv.cpp
 **  Author: Marcio S. Pinho, Mauro C. C. dos Santos and
 **          Regis A. P. Kopper.
 **
 **  Mail: pinho@inf.pucrs.br,
 **        maurocharao@yahoo.com
 **        regis@webstuff.com.br
 **
 **  Version: 1.0
 **
 **  Last Update: 2 October 2003
 **
 *****************************************************/

#include <windows.h>
#include <GL\gl.h>
#include <GL\glu.h>
#include <GL\glut.h>
#include "SmVR.h"
#include "CollisionDetection\timer.h"
#include "OBJ\SmVR_COBJLoader.h"
#include <iostream>
#include <string>
#include <fstream>

using namespace std;

const int numObjs = 1;
char *Objetos[numObjs] = {"objLoaded"};

GLint janela;
SmVR_CGeometricObject *RootObject;

SmVR_CPoint Zero(0, 0, 0);
SmVR_CPoint *pt_user = new SmVR_CPoint(0, 0, 0);
SmVR_CPoint *pt_alvo = new SmVR_CPoint(0, 0, 0);

SmVR_CGeometricObject *user = NULL, *alvo = NULL;

GLfloat angle = 90, WindowRatio, val_near = .03, val_far = 900;

GLint   especMaterial = 60;
GLfloat posicaoLuz1   [4] = { 100.0, 300.0, 0.0, 1.0};
GLfloat luzAmbiente   [4] = { 0.4,  0.4, 0.4, 1.0};
GLfloat luzDifusa     [4] = { 0.7,  0.7, 0.7, 1.0}; // "cor" do objeto
GLfloat luzEspecular  [4] = { 0.5,  0.5, 0.5, 1.0}; // brilho do objeto
GLfloat especularidade[4] = { 0.17,  0.17, 0.17, 1.0}; // capacidade de brilho do material

float frameRate = 0;

Timer timerRefresh;

/************************************************
 ** void SetaTituloJanela()
 ************************************************/
void SetaTituloJanela(){
  char titulo[200];
  sprintf(titulo,"Teste %.2f frames/segundo - User:(%.2f,%.2f,%.2f) - Alvo:(%.2f,%.2f,%.2f)",frameRate,pt_user->X, pt_user->Y, pt_user->Z,pt_alvo->X, pt_alvo->Y, pt_alvo->Z);
  glutSetWindowTitle(titulo);
}

/************************************************
 ** void PosicUser()
 ************************************************/
void PosicUser() {
  glutSetWindow(janela);
  // Set the clipping volume
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(angle, WindowRatio, val_near, val_far);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  posicaoLuz1[0] = pt_alvo->X;
  posicaoLuz1[2] = pt_alvo->Z;
  glLightfv(GL_LIGHT0, GL_POSITION, posicaoLuz1);
  gluLookAt(pt_user->X, pt_user->Y, pt_user->Z, // Posicao do observador
            pt_alvo->X, pt_alvo->Y, pt_alvo->Z, // Ponto que o observador esta olhando
            0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
}

/************************************************
 ** void ChangeSize(GLsizei w, GLsizei h)
 **
 ** callback para tratar redimensionamento da janela
 ** calcula a relacao de aspecto e redefine a window no SRU
 **
 ************************************************/
void ChangeSize(GLsizei w, GLsizei h) {
  // Prevent a divide by zero, when window is too short
  // (you cant make a window of zero width)
  if(h == 0)
    h = 1;

  WindowRatio = 1.0f * w / h;
  glutSetWindow(janela);
  // Reset the coordinate system before modifying
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  // Set the viewport to be the entire window
  glViewport(0, 0, w, h);
  PosicUser();
}

/************************************************
 ** void SetupRC()
 **
 ** Inicializa caracteristicas para geracao da imagem
 ** como window no SRU
 **
 ************************************************/
void SetupRC() {
  glShadeModel(GL_SMOOTH);
  glClearColor(0.2f, 0.2f, 0.9f, 1.0f);
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

  glMaterialfv(GL_FRONT,GL_SPECULAR, especularidade);
  glMateriali(GL_FRONT,GL_SHININESS,especMaterial);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, luzAmbiente);

  glLightfv(GL_LIGHT0, GL_AMBIENT , luzAmbiente );
  glLightfv(GL_LIGHT0, GL_DIFFUSE , luzDifusa   );
  glLightfv(GL_LIGHT0, GL_SPECULAR, luzEspecular);
  glLightfv(GL_LIGHT0, GL_POSITION, posicaoLuz1 );

  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
}

/************************************************
 ** void gerar_animacao()
 ************************************************/
void gerar_animacao() {
  user->GetPointInOCS(Zero, pt_user, RootObject);
  alvo->GetPointInOCS(Zero, pt_alvo, RootObject);
  glutSetWindow(janela); // Para repintar a janela principal
  glutPostRedisplay();
}

/************************************************
 ** void RenderScene(void)
 ************************************************/
void RenderScene(void) {
  frameRate = 1/timerRefresh.partial();
  timerRefresh.start();

  SetaTituloJanela();

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  PosicUser();

  RootObject->Render();
  glutSwapBuffers();
}

/************************************************
 ** void keyboard(unsigned char key, int x, int y)
 ************************************************/
void keyboard(unsigned char key, int x, int y) {
  SmVR_CGeometricObject *tmpApontador = RootObject->FindChildObject("apontador"), *tmpApontadorEnv;
  int i;
  bool hasChild = false;
  switch(key) {
    case 27: exit(1);
  }
}

/************************************************
 ** void SpecialKeys(int key, int x, int y)
 ************************************************/
void SpecialKeys(int key, int x, int y) {
  switch(key) {
    case GLUT_KEY_LEFT     :
          if(glutGetModifiers() == GLUT_ACTIVE_SHIFT){
              user->TranslateByOnOBJCS(.1, 0,  0, alvo);
            }
          else
            user->RotateBy( 1, 0, 1, 0);
      break;
    case GLUT_KEY_RIGHT    :
          if(glutGetModifiers() == GLUT_ACTIVE_SHIFT){
              user->TranslateByOnOBJCS(-.1, 0,  0, alvo);
          }else
            user->RotateBy( -1, 0, 1, 0);
      break;
    case GLUT_KEY_UP       :
          if(glutGetModifiers() == GLUT_ACTIVE_SHIFT)
            user->RotateBy(-1, 1, 0, 0);
          else{
              user->TranslateByOnOBJCS(0, 0, .1, alvo);
          }
      break;
    case GLUT_KEY_DOWN     :
          if(glutGetModifiers() == GLUT_ACTIVE_SHIFT)
            user->RotateBy(1, 1, 0, 0);
          else{
              user->TranslateByOnOBJCS(0, 0, -.1, alvo);
          }
      break;
    case GLUT_KEY_PAGE_UP  :
            user->TranslateByOnOBJCS(0,  .03, 0, alvo);
      break;
    case GLUT_KEY_PAGE_DOWN:
            user->TranslateByOnOBJCS(0,  -.03, 0, alvo);
      break;
  }
}

/************************************************
 ** int main(int argc, char **argv)
 ********************************1****************/
int main(int argc, char **argv) {
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
  glutInitWindowSize(800, 600);
  janela = glutCreateWindow("Teste");
  //glutFullScreen();

  RootObject = SmVR_Init(NULL);

  user = new SmVR_CGeometricObject("user");

  alvo = new SmVR_CGeometricObject("alvo");
  alvo->TranslateBy(0, 0, 1);

  user->AddChild(alvo);
  RootObject->AddChild(user);

  SmVR_CGeometricObject *tmp;
  
  string filename;
  
  cout << "Path+nome do objeto.obj a ser carregado: ";
  cin >> filename;
  char *str = (char*)filename.c_str();

  SmVR_COBJLoader *ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  if(ObjectFromFile->Load(str))
  {
    RootObject->AddChild(tmp = new SmVR_CGeometricObject(Objetos[0], ObjectFromFile));
  } 
  
  cout << endl << "Path+nome do objeto.tri a ser criado: ";
  cin >> filename;
  ofstream outfile(filename.c_str());

  int vn[3];

  TVertices *v  = ObjectFromFile->GetVertices();
  int nfaces = ObjectFromFile->GetNFaces();

  outfile << nfaces << endl;

  for (int f = 0; f < nfaces; f++)
  {
	ObjectFromFile->GetFace(f,vn);
	for (int n=0; n<3; n++)
	{			
		outfile << v[vn[n]].x << " " << v[vn[n]].y << " " << v[vn[n]].z << " ";
	}
	outfile << endl;
  }

  user->TranslateBy(0,0,-1.5);

  user->GetPointInOCS(Zero, pt_user, RootObject);
  alvo->GetPointInOCS(Zero, pt_alvo, RootObject);

  glutDisplayFunc(RenderScene); // Especifica a funcao de desenho
  glutSpecialFunc(SpecialKeys); // Trata as teclas especiais
  glutKeyboardFunc(keyboard);   // Trata as teclas normais
  glutReshapeFunc(ChangeSize);  // Especifica a funcao chamada quando altera o tamanho da janela
  SetupRC();                    // Executa a inicializacao de parametros de exibicao

  glutIdleFunc(gerar_animacao);
  glutMainLoop();
  return 0;
}
