// ************************************************
// TestAddChild.cpp
//	Sample file for SmallVR
////  Author: Marcio Serolli Pinho
//	Email: pinho@inf.pucrs.br		 
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2002.
// ************************************************

#ifdef WIN32
#include <windows.h>
#endif

#include <GL\gl.h>

// SmallVR header file
#include "SmVR.h"

float WindowRatio;

SmVR_CPoint	User(0,0,10);

// Root Object to represent the entire scene
SmVR_CGeometricObject *RootObject;

// **********************************************************************
//  void DefineLuz(void)
//
//
// **********************************************************************
void DefineLight(void)
{
  // Define colors for a golden object
  GLfloat LuzAmbiente[]   = {0.24725f, 0.1995f, 0.07f } ;
  GLfloat LuzDifusa[]   = {0.75164f, 0.60648f, 0.22648f, 1.0f };
  GLfloat LuzEspecular[] = {0.626281f, 0.555802f, 0.366065f, 1.0f };
  GLfloat PosicaoLuz0[]  = {3.0f, 3.0f, 0.0f, 1.0f };
  GLfloat PosicaoLuz1[]  = {-3.0f, -3.0f, 0.0f, 1.0f };
  GLfloat Especularidade[] = {1.0f, 1.0f, 1.0f, 1.0f };

   // ****************  Light source 0

  glEnable ( GL_COLOR_MATERIAL );

   // Enable the use of light
  glEnable(GL_LIGHTING);

  // Set ambient light
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);
  // Define light 0 parameters
  glLightfv(GL_LIGHT0, GL_AMBIENT, LuzAmbiente);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, LuzDifusa  );
  glLightfv(GL_LIGHT0, GL_SPECULAR, LuzEspecular  );
  glLightfv(GL_LIGHT0, GL_POSITION, PosicaoLuz0 );
  glEnable(GL_LIGHT0);

  // Enable the Color Tracking
  glEnable(GL_COLOR_MATERIAL);

  // Define the material reflectance
  glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

  // Define shininess concentration
  // As higher this value is the more concentrate
  // the shininess will be. The valid values are form 0 to 128
  glMaterialf(GL_FRONT,GL_SHININESS,51.2f);

   // ****************  Light source 1

  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);

  
  glLightfv(GL_LIGHT1, GL_AMBIENT, LuzAmbiente);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, LuzDifusa  );
  glLightfv(GL_LIGHT1, GL_SPECULAR, LuzEspecular  );
  glLightfv(GL_LIGHT1, GL_POSITION, PosicaoLuz1 );
  glEnable(GL_LIGHT1);

  glEnable(GL_COLOR_MATERIAL);

  glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

  glMateriali(GL_FRONT,GL_SHININESS,20);

}

#define HALF_EGDE_LENGHT 0.5
// **********************************************************************
//  void DesenhaCubo()
//
//
// **********************************************************************
void DrawCube()
{
	glBegin ( GL_QUADS );
		// Front Face
		glNormal3f(0,0,1);
		glVertex3f(-HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f(-HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		// Back Face
		glNormal3f(0,0,-1);
		glVertex3f(-HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f(-HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		// Top Face
		glNormal3f(0,1,0);
		glVertex3f(-HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f(-HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		// Bottom Face
		glNormal3f(0,-1,0);
		glVertex3f(-HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f(-HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		// Right face
		glNormal3f(1,0,0);
		glVertex3f( HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f( HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		// Left Face
		glNormal3f(-1,0,0);
		glVertex3f(-HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
		glVertex3f(-HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f(-HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT);
		glVertex3f(-HALF_EGDE_LENGHT,  HALF_EGDE_LENGHT, -HALF_EGDE_LENGHT);
	glEnd();
}

// **********************************************************************
// int DrawTest(void *p)
//
// **********************************************************************
int DrawTest1(void *p) 
{ 
  glColor3f(1.0,0.0,0.0);
  glPushMatrix();
	  glutSolidCube(1);  
  glPopMatrix();	   
  return 1;
} 

int DrawTest2(void *p) 
{ 
  glColor3f(0.0,1.0,0.0);
  glPushMatrix();
	  glutSolidCube(1);  
  glPopMatrix();	   
  return 1;
} 

int DrawTest3(void *p) 
{ 
  glColor3f(0.0,0.0,1.0);
  glPushMatrix();
	  glutSolidCube(1);  
  glPopMatrix();	   
  return 1;
} 

int DrawTest4(void *p) 
{ 
  glColor3f(0.2,0.9,1.0);
  glPushMatrix();
	  glutSolidCube(1);  
  glPopMatrix();	   
  return 1;
} 

int DrawTest5(void *p) 
{ 
  glColor3f(0.8,0.5,1.0);
  glPushMatrix();
	  glutSolidCube(1);  
  glPopMatrix();	   
  return 1;
} 


// **********************************************************************
// int DrawTest(void *p)
//
// **********************************************************************
int DrawTest0(void *p) 
{ 
	DrawCube(); 	

  return 1;
} 
// **********************************************************************
//  void PosicUser()
//
//
// **********************************************************************
void PosicUser()
{
	// Set the clipping volume
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90,WindowRatio,0.01,200);

	// Set the user Position
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(User.X,User.Y, User.Z,
		      User.X,User.Y, User.Z-3,
			  0.0f,1.0f,0.0f);
}
// **********************************************************************
//  void reshape( int w, int h )
//		trata o redimensionamento da janela OpenGL
//
// **********************************************************************
void reshape( int w, int h )
{
	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	WindowRatio = 1.0f * w / h;
	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

	PosicUser();

}
// **********************************************************************
//  void display( void )
//
//
// **********************************************************************
void display( void )
{

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	PosicUser();

	glMatrixMode(GL_MODELVIEW);

	DefineLight();

	RootObject->Render();
	
	glutSwapBuffers();
}


// **********************************************************************
//  void keyboard ( unsigned char key, int x, int y )
//
//
// **********************************************************************
void keyboard ( unsigned char key, int x, int y )  
{
	switch ( key ) 
	{
	    case 27:        // Teminate the program
			exit(0);    
			break;  
						// Moves the user
		case 'A' : 
		case 'a' : 
			User.X --;
			break;        				
		case 's' : 
		case 'S' : 
			User.X ++;
			break;        				
		case 'w' : 
		case 'W' : 
			User.Y ++;
			break;        				
		case 'z' : 
		case 'Z' : 
			User.Y --;
			break;        				
		case 'q' : 
		case 'Q' : 
			User.Z --;
			break;        				
		case 'x' : 
		case 'X' : 
			User.Z ++;
			break;    
  }
}

// **********************************************************************
//  void init(void)
//		Inicializa os parmetros globais de OpenGL
//
// **********************************************************************
void init(void)
{
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f); // Fundo de tela preto
	glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );

	glShadeModel(GL_SMOOTH);						
	glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_DEPTH_TEST);
	glEnable ( GL_CULL_FACE );
}


// **********************************************************************
// void specialFunc (int key, int x, int y) 
//
// **********************************************************************
void specialFunc (int key, int x, int y) {


	SmVR_CGeometricObject *Base;

	Base = RootObject->FindChildObject("Test Object 1");
    

	switch (key) {
		case GLUT_KEY_UP:
			Base->RotateBy (-5, 1, 0, 0);
			break;

		case GLUT_KEY_DOWN:
			Base->RotateBy (5, 1, 0, 0);
			break;

		case GLUT_KEY_LEFT:
			Base->RotateBy (-5, 0, 1, 0);
			break;

		case GLUT_KEY_RIGHT:
			Base->RotateBy (5, 0, 1, 0);
			break;

		case GLUT_KEY_PAGE_UP:
			Base->RotateBy (-5, 0, 0, 1);
			break;

		case GLUT_KEY_PAGE_DOWN:
			Base->RotateBy (5, 0, 0, 1);
			break;
	}

}

// **********************************************************************
//  void main ( int argc, char** argv )
//
//
// **********************************************************************
void main ( int argc, char** argv )   
{
	SmVR_CGeometricObject *Obj0,*Obj1, *Obj2, *Obj3, *Obj4, *Obj5, *Obj6, *Obj7, *Obj8;

	// GLUT initialization
	glutInit            ( &argc, argv ); 
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);// | GLUT_DOUBLE | GLUT_RGBA ); 
	glutInitWindowPosition (0,0);
	glutInitWindowSize  ( 700, 500 ); 
	glutCreateWindow    ( "SmallVR Test" ); 
		
	// initialize OpenGL parameters
	init ();

	// initialize the library
	RootObject = SmVR_Init(NULL);

	// Create a new object
	Obj0 = new SmVR_CGeometricObject("Chao", DrawTest1);	
	Obj1 = new SmVR_CGeometricObject("Test Object 1", DrawTest2);	
	Obj2 = new SmVR_CGeometricObject("Test Object 2", DrawTest3);
	Obj3 = new SmVR_CGeometricObject("Test Object 3", DrawTest4);
	Obj4 = new SmVR_CGeometricObject("Test Object 4", DrawTest5);
	Obj5 = new SmVR_CGeometricObject("Test Object 5", DrawTest1);
	Obj6 = new SmVR_CGeometricObject("Test Object 6", DrawTest2);
	Obj7 = new SmVR_CGeometricObject("Test Object 7", DrawTest3);
	Obj8 = new SmVR_CGeometricObject("Test Object 9", DrawTest5);


	// Add the objects to the scene root
	RootObject->AddChild(Obj0);
	RootObject->AddChild(Obj1);
	RootObject->AddChild(Obj2);
	RootObject->AddChild(Obj3);
	RootObject->AddChild(Obj4);
	RootObject->AddChild(Obj5);
	RootObject->AddChild(Obj6);
	RootObject->AddChild(Obj7);
	RootObject->AddChild(Obj8);

	Obj1->TranslateBy(1.5,0,0);
	Obj2->TranslateBy(3.0,0,0);
	Obj3->TranslateBy(4.5,0,0);
	Obj4->TranslateBy(6.0,0,0);
	Obj5->TranslateBy(7.5,0,0);
	Obj6->TranslateBy(9.0,0,0);
	Obj7->TranslateBy(10.5,0,0);
	Obj8->TranslateBy(12.0,0,0);

	Obj1->ScaleBy(1,1.5,1);
	Obj2->ScaleBy(1,1.5,1);
	Obj3->ScaleBy(1,1.5,1);
	Obj4->ScaleBy(1,1.5,1);

	RootObject->AddChild(Obj0);
	RootObject->AddChild(Obj1);
	Obj1->AddChild(Obj2);
	Obj2->AddChild(Obj3);
	Obj3->AddChild(Obj4);
	Obj4->AddChild(Obj5);
	Obj5->AddChild(Obj6);
	Obj6->AddChild(Obj7);
	Obj7->AddChild(Obj8);



/*
    RootObject->AddChild(Obj0);
	RootObject->AddChild(Obj1);
	Obj1->AddChild(Obj2);
	Obj2->AddChild(Obj3);
	Obj3->AddChild(Obj4);
*/


/*
	Obj1->TranslateByOnOBJCS(0,1.5,0, RootObject);
	Obj2->TranslateByOnOBJCS(0,1.5,0, RootObject);
	Obj3->TranslateByOnOBJCS(0,1.5,0, RootObject);
	Obj4->TranslateByOnOBJCS(0,1.5,0, RootObject);




	// Add the objects to the scene root
	RootObject->AddChild(Obj0);
	RootObject->AddChild(Obj1);
	Obj1->AddChild(Obj2);
	Obj2->AddChild(Obj3);
	Obj3->AddChild(Obj4);
	Obj4->AddChild(Obj5);
	Obj5->AddChild(Obj6);
	Obj6->AddChild(Obj7);
	Obj7->AddChild(Obj8);

	Obj1->TranslateBy(1.5,0,0);
	Obj2->TranslateBy(1.5,0,0);
	Obj3->TranslateBy(1.5,0,0);
	Obj4->TranslateBy(1.5,0,0);
	Obj5->TranslateBy(1.5,0,0);
	Obj6->TranslateBy(1.5,0,0);
	Obj7->TranslateBy(1.5,0,0);
	Obj8->TranslateBy(1.5,0,0);

	Obj1->ScaleBy(1,1.5,1);
	Obj2->ScaleBy(1,1.5,1);
	Obj3->ScaleBy(1,1.5,1);
	Obj4->ScaleBy(1,1.5,1);

*/	

	// GLUT stuff
	glutDisplayFunc ( display );  
	glutReshapeFunc ( reshape );
	glutKeyboardFunc ( keyboard );
	glutSpecialFunc (specialFunc);

	glutIdleFunc ( display );

	glutMainLoop ( );          

}

