/*****************************************************
 **
 **  Virtual Reality Interactive Environment
 **
 **  File: VRInteractiveEnv.cpp
 **  Author: Marcio S. Pinho, Mauro C. C. dos Santos and
 **          Regis A. P. Kopper.
 **
 **  Mail: pinho@inf.pucrs.br,
 **        maurocharao@yahoo.com
 **        regis@webstuff.com.br
 **
 **  Version: 1.0
 **
 **  Last Update: 2 October 2003
 **
 *****************************************************/

#include <windows.h>
#include <GL\gl.h>
#include <GL\glu.h>
#include <GL\glut.h>
#include <conio.h>
#include "SmVR.h"
#include "CollisionDetection\SmVR_CPath.h"
#include "CollisionDetection\SmVR_CEnvelope.h"
#include "CollisionDetection\timer.h"
#include "OBJ\SmVR_COBJLoader.h"
#include "Devices\IsotrackII\SmVR_CIsotrack.h"
#include "Devices\IGlasses\SmVR_CIGlasses.h"
#include <Stereo\StereoIglasses.h>

int STracker;
const int numObjs = 7;
char *Objetos[numObjs] = {"planta", "bola", "sofa", "mesa", "extinguisher", "blinds", "heater"};

float azul    [3] = { 0, 0, 1};
float branco  [3] = { 1, 1, 1};
float verde   [3] = { 0,.8, 0};
float vermelho[3] = { 1, 0, 0};
float amarelo [3] = { 1, 1, 0};
float cinza   [3] = {.5,.5,.5};
float preto   [3] = { 0, 0, 0};

GLint janela;
SmVR_CGeometricObject *RootObject;
SmVR_CPath *scene;

SmVR_CDevice *Isotrack, *Iglasses;

SmVR_CPoint Zero(0, 0, 0);
SmVR_CPoint *pt_user = new SmVR_CPoint(0, 0, 0);
SmVR_CPoint *pt_alvo = new SmVR_CPoint(0, 0, 0);

SmVR_CGeometricObject *user = NULL, *alvo = NULL;

GLfloat angle = 90, WindowRatio, val_near = .03, val_far = 900;

GLint   especMaterial = 60;
GLfloat posicaoLuz1   [4] = { 100.0, 300.0, 0.0, 1.0};
GLfloat luzAmbiente   [4] = { 0.4,  0.4, 0.4, 1.0};
GLfloat luzDifusa     [4] = { 0.7,  0.7, 0.7, 1.0}; // "cor" do objeto
GLfloat luzEspecular  [4] = { 0.5,  0.5, 0.5, 1.0}; // brilho do objeto
GLfloat especularidade[4] = { 0.17,  0.17, 0.17, 1.0}; // capacidade de brilho do material

float frameRate = 0;

Timer timerRefresh;
GLTexture *texturea=NULL; 

int desenhaCubo(void *p){
  if(!p) glColor3f(0,0,1);
  else glColor3f(0,1,1);
  glBegin ( GL_QUADS );
    // Front Face
    glNormal3f(0,0,1);
    glVertex3f(-.5, -.5,  .5);
    glVertex3f( .5, -.5,  .5);
    glVertex3f( .5,  .5,  .5);
    glVertex3f(-.5,  .5,  .5);
    // Back Face
    glNormal3f(0,0,-1);
    glVertex3f(-.5, -.5, -.5);
    glVertex3f(-.5,  .5, -.5);
    glVertex3f( .5,  .5, -.5);
    glVertex3f( .5, -.5, -.5);
    // Top Face
    glNormal3f(0,1,0);
    glVertex3f(-.5,  .5, -.5);
    glVertex3f(-.5,  .5,  .5);
    glVertex3f( .5,  .5,  .5);
    glVertex3f( .5,  .5, -.5);
    // Bottom Face
    glNormal3f(0,-1,0);
    glVertex3f(-.5, -.5, -.5);
    glVertex3f( .5, -.5, -.5);
    glVertex3f( .5, -.5,  .5);
    glVertex3f(-.5, -.5,  .5);
    // Right face
    glNormal3f(1,0,0);
    glVertex3f( .5, -.5, -.5);
    glVertex3f( .5,  .5, -.5);
    glVertex3f( .5,  .5,  .5);
    glVertex3f( .5, -.5,  .5);
    // Left Face
    glNormal3f(-1,0,0);
    glVertex3f(-.5, -.5, -.5);
    glVertex3f(-.5, -.5,  .5);
    glVertex3f(-.5,  .5,  .5);
    glVertex3f(-.5,  .5, -.5);
  glEnd();
  return 1;
}

GLTexture *texture1=NULL; 
GLTexture *texture2=NULL;

int desenhaSala(void* p) 
{
  glPushAttrib(GL_ALL_ATTRIB_BITS); 
  int *x = (int*)p;
  if (!texture1 && *x != 0)
  {
     texture1 = new GLTexture();
     texture1->Load("..\\TestFiles\\Objetos3D\\index.jpg");
  }
  else if (!texture2)
  {
     texture2 = new GLTexture();
     texture2->Load("..\\TestFiles\\Objetos3D\\wood_floor.jpg");
  }
  if (*x != 0){
    texture1->Use();
    glColor3f(1,1,1);
    glBegin(GL_QUADS);
      glTexCoord2f (0.0, 0.0); glVertex3f(-.5, 0, -.5);
	  glTexCoord2f (1.0, 0.0); glVertex3f(-.5, 0,  .5);
	  glTexCoord2f (1.0, 1.0); glVertex3f( .5, 0,  .5);
	  glTexCoord2f (0.0, 1.0); glVertex3f( .5, 0, -.5);
    glEnd();
  }
  else{
    texture2->Use();
    glColor3f(1,1,1);
    glBegin(GL_QUADS);
	  glTexCoord2f (0.0 , 0.0 ); glVertex3f(-.5, 0, -.5);
	  glTexCoord2f (10.0, 0.0 ); glVertex3f(-.5, 0,  .5);
	  glTexCoord2f (10.0, 10.0); glVertex3f( .5, 0,  .5);
	  glTexCoord2f (0.0 , 10.0); glVertex3f( .5, 0, -.5);
    glEnd();
  }
  return 1;
}

/************************************************
 ** int desenhaQuad(void *p)
 ************************************************/
int desenhaQuad(void* p) 
{
  glPushAttrib(GL_ALL_ATTRIB_BITS); 
  
  glColor3f(1,1,1);
  glBegin(GL_QUADS);
	glVertex3f(-.5, 0, -.5);
	glVertex3f(-.5, 0,  .5);
	glVertex3f( .5, 0,  .5);
	glVertex3f( .5, 0, -.5);
  glEnd();
  return 1;
}

/************************************************
 ** void SetaTituloJanela()
 ************************************************/
void SetaTituloJanela(){
  char titulo[200];
  sprintf(titulo,"Teste %.2f frames/segundo - User:(%.2f,%.2f,%.2f) - Alvo:(%.2f,%.2f,%.2f)",frameRate,pt_user->X, pt_user->Y, pt_user->Z,pt_alvo->X, pt_alvo->Y, pt_alvo->Z);
  glutSetWindowTitle(titulo);
}

/************************************************
 ** void PosicUser()
 ************************************************/
void PosicUser() {
  glutSetWindow(janela);
  // Set the clipping volume
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(angle, WindowRatio, val_near, val_far);

  // Set the user Position
  // Especifica sistema de coordenadas do modelo
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  posicaoLuz1[0] = pt_alvo->X;
  posicaoLuz1[2] = pt_alvo->Z;
  glLightfv(GL_LIGHT0, GL_POSITION, posicaoLuz1);
  gluLookAt(pt_user->X, pt_user->Y, pt_user->Z, // Posicao do observador
            pt_alvo->X, pt_alvo->Y, pt_alvo->Z, // Ponto que o observador esta olhando
            0.0f, 1.0f, 0.0f); // Define o lado de cima do cenario 3D
}

/************************************************
 ** void ChangeSize(GLsizei w, GLsizei h)
 **
 ** callback para tratar redimensionamento da janela
 ** calcula a relacao de aspecto e redefine a window no SRU
 **
 ************************************************/
void ChangeSize(GLsizei w, GLsizei h) {
  // Prevent a divide by zero, when window is too short
  // (you cant make a window of zero width)
  if(h == 0)
    h = 1;

  WindowRatio = 1.0f * w / h;
  glutSetWindow(janela);
  // Reset the coordinate system before modifying
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  // Set the viewport to be the entire window
  glViewport(0, 0, w, h);
  PosicUser();
}

void ilumina_objeto(SmVR_CPoint *min,SmVR_CPoint *max){
    GLfloat posicaoLuzObj[4][4] = { { min->X*.02 , max->Y*.02 , min->Z*.02 , 1.0 },
                                    { max->X*.02 , max->Y*.02 , min->Z*.02 , 1.0 },
                                    { min->X*.02 , max->Y*.02 , max->Z*.02 , 1.0 },
                                    { max->X*.02 , max->Y*.02 , max->Z*.02 , 1.0 }
                                  };
    
    for(int i=1;i<=4;i++){
      glLightfv(GL_LIGHT0+i, GL_AMBIENT , luzAmbiente );
      glLightfv(GL_LIGHT0+i, GL_DIFFUSE , luzDifusa   );
      glLightfv(GL_LIGHT0+i, GL_SPECULAR, luzEspecular);
      glLightfv(GL_LIGHT0+i, GL_POSITION, posicaoLuzObj[i] );
      glEnable(GL_LIGHT0+i);         // Habilita a luz de numero i
    }
    
}

/************************************************
 ** void SetupRC()
 **
 ** Inicializa caracteristicas para geracao da imagem
 ** como window no SRU
 **
 ************************************************/
void SetupRC() {
  glShadeModel(GL_SMOOTH);
  glClearColor(0.2f, 0.2f, 0.9f, 1.0f);
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

  glMaterialfv(GL_FRONT,GL_SPECULAR, especularidade);
  glMateriali(GL_FRONT,GL_SHININESS,especMaterial);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, luzAmbiente);

  glLightfv(GL_LIGHT0, GL_AMBIENT , luzAmbiente );
  glLightfv(GL_LIGHT0, GL_DIFFUSE , luzDifusa   );
  glLightfv(GL_LIGHT0, GL_SPECULAR, luzEspecular);
  glLightfv(GL_LIGHT0, GL_POSITION, posicaoLuz1 );

  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
}

/************************************************
 ** void gerar_animacao()
 ************************************************/
void gerar_animacao() {
  user->GetPointInOCS(Zero, pt_user, RootObject);
  alvo->GetPointInOCS(Zero, pt_alvo, RootObject);
  glutSetWindow(janela); // Para repintar a janela principal
  glutPostRedisplay();
}

/************************************************
 ** void RenderScene(void)
 ************************************************/
void RenderScene(void) {
  frameRate = 1/timerRefresh.partial();
  timerRefresh.start();

  SetaTituloJanela();

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  if (STracker == 1)
  {
   // Isotrack->UpdateAllSensors(0,0,0,1,1,1);
    Iglasses->UpdateAllSensors();
  }
  
  int i = 0;
  bool hasChild = false;
  SmVR_CGeometricObject *tmpApontadorEnv = RootObject->FindChildObject("mao");
  while (i < numObjs)
  {
      if(tmpApontadorEnv->Collided(RootObject->FindChildObject(Objetos[i])) && !hasChild)
      {
        RootObject->FindChildObject(Objetos[i])->GetFirstChild()->SetVisibility(1);
        hasChild = true;
      }
      else
      {
        RootObject->FindChildObject(Objetos[i])->GetFirstChild()->SetVisibility(0);
      }
      i++;
  }

  PosicUser();

  RootObject->Render();
  glutSwapBuffers();
}

void approach(int t)
{
    SmVR_CGeometricObject *tmpApontador = RootObject->FindChildObject("apontador"), *tmp1;
    int i = 0;
    while(i < numObjs)
    {
        if((tmp1 = tmpApontador->FindChildObject(Objetos[i])) != NULL)
        {
          if(t)
          {
            tmp1->TranslateBy(0,0,-.05);
          }
          else
          {
            tmp1->TranslateBy(0,0,.05);
          }
        }
        i++;
    }
}

void rotate(int t)
{
    SmVR_CGeometricObject *tmpApontador = RootObject->FindChildObject("apontador"), *tmp1;
    
    int i = 0;
    while(i < numObjs)
    {
        if((tmp1 = tmpApontador->FindChildObject(Objetos[i])) != NULL)
        {
          if(!t)
          {
            tmp1->RotateByOnOBJCS(2,1,0,0,tmp1);
          }
          else if(t == 1)
          {
            tmp1->RotateByOnOBJCS(-2,1,0,0,tmp1);
          }
          else if(t == 2)
          {
            tmp1->RotateByOnOBJCS(2,0,1,0,tmp1);
          }
          else
          {
            tmp1->RotateByOnOBJCS(-2,0,1,0,tmp1);
          }
        }
        i++;
    }
}

/************************************************
 ** void keyboard(unsigned char key, int x, int y)
 ************************************************/
void keyboard(unsigned char key, int x, int y) {
  SmVR_CGeometricObject *tmpApontador = RootObject->FindChildObject("apontador"), *tmpApontadorEnv;
  int i;
  bool hasChild = false;
  switch(key) {
    case 'w': case 'W': tmpApontador->RotateBy(-.5,1,0,0); 
                        break;
    case 's': case 'S': tmpApontador->RotateBy(.5,1,0,0); 
                        break;
    case 'a': case 'A': tmpApontador->RotateBy(.5,0,1,0); 
                        break;
    case 'd': case 'D': tmpApontador->RotateBy(-.5,0,1,0); 
                        break;
    case 'e': case 'E': tmpApontadorEnv = RootObject->FindChildObject("mao");
                        hasChild = false;
                        i = 0;
                        while(i < numObjs)
                        {
                          if(tmpApontadorEnv->Collided(RootObject->FindChildObject(Objetos[i])) && !hasChild)
                          {
                            tmpApontadorEnv->AddChild(RootObject->FindChildObject(Objetos[i]));
                            hasChild = true;
                          }
                          else
                          {
                            RootObject->AddChild(RootObject->FindChildObject(Objetos[i]));
                          }
                          i++;
                        }
                        break;
    case 'q': case 'Q': i = 0;
                        while(i < numObjs)
                        {
                          RootObject->AddChild(tmpApontador->FindChildObject(Objetos[i]));
                          i++;
                        }
                        break;
    case 't': case 'T': rotate(0); break;
    case 'g': case 'G': rotate(1); break;
    case 'f': case 'F': rotate(2); break;
    case 'h': case 'H': rotate(3); break;
    case 'r': case 'R': if (STracker) 
                        {
                           Iglasses->Reset(NULL);
                           //Isotrack->Reset(NULL);
                        }
                        break;
    case 'z': case 'Z': approach(1);
                        break;
    case 'x': case 'X': approach(0);
                        break;
    case 27: exit(1);
  }
}

/************************************************
 ** void SpecialKeys(int key, int x, int y)
 ************************************************/
void SpecialKeys(int key, int x, int y) {
  SmVR_CPath     *tmpPath =     (SmVR_CPath *)(RootObject->FindChildObject("scene"));
  SmVR_CEnvelope *tmpEnv  = (SmVR_CEnvelope *)(RootObject->FindChildObject("usrEnvelope"));
  SmVR_CEnvelope *tmpTeto = (SmVR_CEnvelope *)(RootObject->FindChildObject("teto"));
  SmVR_CEnvelope *tmpChao = (SmVR_CEnvelope *)(RootObject->FindChildObject("chao"));

  switch(key) {
    case GLUT_KEY_LEFT     :
          if(glutGetModifiers() == GLUT_ACTIVE_SHIFT){
              user->TranslateByOnOBJCS(.1, 0,  0, alvo);
              if (tmpPath->colided(tmpEnv) || tmpChao->colided(tmpEnv) || tmpTeto->colided(tmpEnv)/* || CollisionTest()*/) user->TranslateByOnOBJCS(-.1, 0, 0, alvo);
            }
          else
            user->RotateBy( 1, 0, 1, 0);
      break;
    case GLUT_KEY_RIGHT    :
          if(glutGetModifiers() == GLUT_ACTIVE_SHIFT){
              user->TranslateByOnOBJCS(-.1, 0,  0, alvo);
              if (tmpPath->colided(tmpEnv) || tmpChao->colided(tmpEnv) || tmpTeto->colided(tmpEnv)/* || CollisionTest()*/) user->TranslateByOnOBJCS(-.1, 0, 0, alvo);
          }else
            user->RotateBy( -1, 0, 1, 0);
      break;
    case GLUT_KEY_UP       :
          if(glutGetModifiers() == GLUT_ACTIVE_SHIFT)
            user->RotateBy(-1, 1, 0, 0);
          else{
              user->TranslateByOnOBJCS(0, 0, .1, alvo);
              //if (tmpPath->colided(tmpEnv) || tmpChao->colided(tmpEnv) || tmpTeto->colided(tmpEnv)/* || CollisionTest()*/) user->TranslateByOnOBJCS(0, 0, -.1, alvo);
          }
      break;
    case GLUT_KEY_DOWN     :
          if(glutGetModifiers() == GLUT_ACTIVE_SHIFT)
            user->RotateBy(1, 1, 0, 0);
          else{
              user->TranslateByOnOBJCS(0, 0, -.1, alvo);
              //if (tmpPath->colided(tmpEnv) || tmpChao->colided(tmpEnv) || tmpTeto->colided(tmpEnv)/* || CollisionTest()*/) user->TranslateByOnOBJCS(0, 0, .1, alvo);
          }
      break;
    case GLUT_KEY_PAGE_UP  :
            user->TranslateByOnOBJCS(0,  .03, 0, alvo);
            //if (tmpPath->colided(tmpEnv) || tmpChao->colided(tmpEnv) || tmpTeto->colided(tmpEnv)/* || CollisionTest()*/) user->TranslateByOnOBJCS(0, -.03, 0, alvo);
      break;
    case GLUT_KEY_PAGE_DOWN:
            user->TranslateByOnOBJCS(0,  -.03, 0, alvo);
            //if (tmpPath->colided(tmpEnv) || tmpChao->colided(tmpEnv) || tmpTeto->colided(tmpEnv)/* || CollisionTest()*/) user->TranslateByOnOBJCS(0,  .03, 0, alvo);
      break;
  }
}

/************************************************
 ** int main(int argc, char **argv)
 ********************************1****************/
int main(int argc, char **argv) {
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
  glutInitWindowSize(800, 600);
  janela = glutCreateWindow("Teste");
  glutFullScreen();

  RootObject = SmVR_Init(NULL);

  /*printf("Deseja usar o IsoTrack?\n[1] - Sim\n[0] - Nao\n\n");
  scanf("%d",&STracker);*/
  STracker = 0;
  
  user = new SmVR_CGeometricObject("user");

  alvo = new SmVR_CGeometricObject("alvo");
  alvo->TranslateBy(0, 0, 1);

  user->AddChild(alvo);
  RootObject->AddChild(user);

  SmVR_CEnvelope *usrEnvelope = new SmVR_CEnvelope("usrEnvelope");
  usrEnvelope->ScaleBy(.7, 1.35, .3);
  usrEnvelope->TranslateBy(0,-.365,0);
  usrEnvelope->SetVisibility(0);
  user->AddChild(usrEnvelope);

  SmVR_CGeometricObject *apontador = new SmVR_CGeometricObject("apontador");

  SmVR_CObjFromFile *ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  ObjectFromFile->Load("..\\testfiles\\Objetos3D\\apontador.obj");
  SmVR_CGeometricObject *hand = new SmVR_CGeometricObject("mao",ObjectFromFile);
  hand->RotateBy(180,1,0,0);
  hand->SetVisibility(1);

  apontador->AddChild(hand);

  apontador->TranslateBy(-.325,0,.82);

  if (STracker == 1)
  {
    
    //Isotrack = new SmVR_CIsotrack();
	//Isotrack->Init((void*)"COM1");

    SmVR_CGeometricObject *pai = new SmVR_CGeometricObject("pai");
    user->AddChild(pai);
    SmVR_CGeometricObject *tracker = new SmVR_CGeometricObject("tracker",desenhaCubo);
    pai->AddChild(tracker);
    pai->ScaleBy(.00001,.00001,.00001);
    pai->TranslateBy(-.0225,0,.12);
    pai->RotateBy(180,0,1,0);
    pai->SetVisibility(0);
    tracker->AddChild(apontador);

	//Isotrack->Bind(0, tracker);
	//Isotrack->Reset(NULL);
	
    Iglasses = new SmVR_CIGlasses();
    Iglasses->Init((char *)"COM1");
    Iglasses->Bind(0, user);
    Iglasses->Reset(NULL);
    Iglasses->UpdateAllSensors();
  }
  else
  {
    user->AddChild(apontador);
  }

  int tex1 = 0;
  int tex2 = 1;
  int tex3 = 2;

  SmVR_CGeometricObject *sala;
  //quad baixo
  RootObject->AddChild(sala = new SmVR_CGeometricObject("sala1", desenhaSala));
  sala->SetRenderFunctionData(&tex1);
  sala->ScaleBy(5,1,4);
  sala->TranslateBy(0,-1.65,0);
  SmVR_CEnvelope *chao = new SmVR_CEnvelope("chao");
  chao->ScaleBy(5,0,4);
  chao->TranslateBy(0,-1.65,0);
  chao->SetVisibility(0);
  RootObject->AddChild(chao);
  //quad frente
  RootObject->AddChild(sala = new SmVR_CGeometricObject("sala2", desenhaSala));
  sala->SetRenderFunctionData(&tex2);
  sala->ScaleBy(5,3,1);
  sala->RotateBy(90,0,0,1);
  sala->RotateBy(90,1,0,0);
  sala->TranslateBy(0,-.15,2);
  //quad tras
  RootObject->AddChild(sala = new SmVR_CGeometricObject("sala3", desenhaSala));
  sala->SetRenderFunctionData(&tex2);
  sala->ScaleBy(5,3,1);
  sala->RotateBy(90,0,0,1);
  sala->RotateBy(90,1,0,0);
  sala->RotateBy(180,1,0,0);
  sala->TranslateBy(0,-.15,-2);
  //quad cima
  RootObject->AddChild(sala = new SmVR_CGeometricObject("sala4", desenhaSala));
  sala->SetRenderFunctionData(&tex3);
  sala->ScaleBy(5,1,4);
  sala->TranslateBy(0,1.35,0);
  SmVR_CEnvelope *teto = new SmVR_CEnvelope("teto");
  teto->ScaleBy(5,0,4);
  teto->TranslateBy(0,1.35,0);
  teto->SetVisibility(0);
  RootObject->AddChild(teto);
  //quad esquerda
  RootObject->AddChild(sala = new SmVR_CGeometricObject("sala5", desenhaSala));
  sala->SetRenderFunctionData(&tex2);
  sala->ScaleBy(3,3,4);
  sala->RotateBy(90,0,1,0);
  sala->RotateBy(90,1,0,0);
  sala->RotateBy(90,0,1,0);
  sala->RotateBy(180,0,1,0);
  sala->TranslateBy(2.5,-.15,0);
  //quad direita
  RootObject->AddChild(sala = new SmVR_CGeometricObject("sala6", desenhaSala));
  sala->SetRenderFunctionData(&tex2);
  sala->ScaleBy(3,3,4);
  sala->RotateBy(90,0,1,0);
  sala->RotateBy(90,1,0,0);
  sala->RotateBy(90,0,1,0);
  sala->TranslateBy(-2.5,-.15,0);
  
  SmVR_CGeometricObject *apontadorP;
  apontador->AddChild(apontadorP = new SmVR_CGeometricObject("apontadorP", desenhaQuad));
  apontadorP->RotateBy(90,1,0,0);
  apontadorP->ScaleBy(.0075, 1, 1);
  apontadorP->TranslateBy(-.0225,.26,.13);
  apontadorP->SetVisibility(0);
  
  if(STracker) RootObject->FindChildObject("pai")->TranslateBy(-.075,-.03,.01);

  SmVR_CGeometricObject *tmp;
  SmVR_CEnvelope *tmpEnv;
  
  ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  if(ObjectFromFile->Load("..\\TestFiles\\Objetos3D\\plant.obj"))
  {
    tmpEnv = new SmVR_CEnvelope("plantaEnvelope");
    tmpEnv->ScaleBy(.26,.52,.26);
    tmpEnv->TranslateBy(0,.065,0);
    tmpEnv->SetVisibility(0);
    RootObject->AddChild(tmp = new SmVR_CGeometricObject("planta", ObjectFromFile));
    tmp->AddChild(tmpEnv);
    tmp->TranslateBy(0,-.952,.15);
  } 
  
  ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  if(ObjectFromFile->Load("..\\TestFiles\\Objetos3D\\couch.obj"))
  {
    tmpEnv = new SmVR_CEnvelope("sofaEnvelope");
    tmpEnv->ScaleBy(2.45,1,.945);
    tmpEnv->TranslateBy(0,.23,-.02);
    tmpEnv->SetVisibility(0);
    RootObject->AddChild(tmp = new SmVR_CGeometricObject("sofa", ObjectFromFile));
    tmp->AddChild(tmpEnv);
    tmp->TranslateBy(0,-1.35,1.4);
  }
  
  ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  if(ObjectFromFile->Load("..\\TestFiles\\Objetos3D\\soccerball.obj"))
  {
    tmpEnv = new SmVR_CEnvelope("bolaEnvelope");
    tmpEnv->ScaleBy(.2,.2,.2);
    tmpEnv->SetVisibility(0);
    RootObject->AddChild(tmp = new SmVR_CGeometricObject("bola", ObjectFromFile));
    tmp->AddChild(tmpEnv);
    tmp->TranslateBy(.15,-1.029,-.1);
  }
  
  ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  if(ObjectFromFile->Load("..\\TestFiles\\Objetos3D\\mesa1.obj"))
  {
    tmpEnv = new SmVR_CEnvelope("mesaEnvelope");
    tmpEnv->ScaleBy(1.2,.43,1.5);
    tmpEnv->TranslateBy(0,.29,0);
    tmpEnv->SetVisibility(0);
    RootObject->AddChild(tmp = new SmVR_CGeometricObject("mesa", ObjectFromFile));
    tmp->AddChild(tmpEnv);
    tmp->TranslateBy(0,-1.64,.25);
    tmp->RotateBy(90,0,1,0);
  }
  
  ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  if(ObjectFromFile->Load("..\\TestFiles\\Objetos3D\\extinguisher.obj"))
  {
    tmpEnv = new SmVR_CEnvelope("extEnvelope");
    tmpEnv->ScaleBy(.2,.54,.2);
    tmpEnv->TranslateBy(0,.28,0);
    tmpEnv->SetVisibility(0);
    RootObject->AddChild(tmp = new SmVR_CGeometricObject("extinguisher", ObjectFromFile));
    tmp->AddChild(tmpEnv);
    tmp->RotateBy(180,0,1,0);
    tmp->TranslateBy(1.2,-.2,1.86);
  }
  
  ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  if(ObjectFromFile->Load("..\\TestFiles\\Objetos3D\\BLINDS.obj"))
  {
    tmpEnv = new SmVR_CEnvelope("persEnvelope");
    tmpEnv->ScaleBy(1.67,.77,.05);
    tmpEnv->TranslateBy(-.5,.3,-1.77);
    tmpEnv->SetVisibility(0);
    RootObject->AddChild(tmp = new SmVR_CGeometricObject("blinds", ObjectFromFile));
    tmp->AddChild(tmpEnv);
    tmp->TranslateBy(.55,-.4,3.65);
  }
  
  ObjectFromFile = new SmVR_COBJLoader(USE_RAPID);
  if(ObjectFromFile->Load("..\\TestFiles\\Objetos3D\\heater.obj"))
  {
    tmpEnv = new SmVR_CEnvelope("heaterEnvelope");
    tmpEnv->ScaleBy(.1,.48,.56);
    tmpEnv->SetVisibility(0);
    RootObject->AddChild(tmp = new SmVR_CGeometricObject("heater", ObjectFromFile));
    tmp->RotateBy(90,0,1,0);
    tmp->AddChild(tmpEnv);
    tmp->TranslateBy(2.42,-1.4,1);
  }
  
  char scen_file[128];
  sprintf(scen_file, "%s", "..\\TestFiles\\auxCol.dat");
  scene = new SmVR_CPath(scen_file, "scene");

  RootObject->AddChild(scene);
  
  scene->SetVisibility(0);
  
  user->TranslateBy(0,0,-1.5);

  user->GetPointInOCS(Zero, pt_user, RootObject);
  alvo->GetPointInOCS(Zero, pt_alvo, RootObject);

  glutDisplayFunc(RenderScene); // Especifica a funcao de desenho
  glutSpecialFunc(SpecialKeys); // Trata as teclas especiais
  glutKeyboardFunc(keyboard);   // Trata as teclas normais
  glutReshapeFunc(ChangeSize);  // Especifica a funcao chamada quando altera o tamanho da janela
  SetupRC();                    // Executa a inicializacao de parametros de exibicao

  glutIdleFunc(gerar_animacao);
  glutMainLoop();
  return 0;
}
